// wrapped by build app
define("dojoe/gauge/analoggauge/AnalogStyle", ["dijit","dojo","dojox","dojo/require!dojo/fx,dojo/fx/easing,dojox/gfx,dojox/gfx/matrix,dojoe/gauge/common/_Style,dojoe/gauge/common/AbstractGauge,dojo/number"], function(dijit,dojo,dojox){
/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
dojo.provide("dojoe.gauge.analoggauge.AnalogStyle");

dojo.require("dojo.fx");
dojo.require("dojo.fx.easing");
dojo.require("dojox.gfx");
dojo.require("dojox.gfx.matrix");
dojo.require("dojoe.gauge.common._Style");
dojo.require("dojoe.gauge.common.AbstractGauge"); // for the GaugeThreshold
dojo.require("dojo.number");

dojo.declare("dojoe.gauge.analoggauge.AnalogStyle", [dojoe.gauge.common._Style], {

    /** colorMap: Object
     *          Mapping from dojox.widget.AnalogGaugeThreshold.***_STATUS values 
     *         to colors.
	 */
        colorMap: null,
	/**
     *  Gauge color
     */
	gcolor: "#79818b",

	trendColor: "#ffffff",
	
	//OID review : red will confuse the end user, use white
    //needleColor: "#be1e2d",
    needleColor: "white",
    ceilingColor: "#be1e2d",
    
    trendValue : 0,
	topLabel: "",
	leftLabel: "",
	labelPosition: "",
	
	// showThresholdStrip: String
	//		Decides if the threshold strip are to be shown at the edge of Analog Gauge or not.
	//		If true enables the threshold strip else not.
	//showThresholdStrip : "on", 
	showThresholdStrip:"on",
	_currentState:"unknown",
	_addWarning:false,

    constructor: function(args){
        dojo.mixin(this, args);
        // if the colorMap wasn't set, use a default one.
        if(this.colorMap == null){
            this.colorMap = {};
            this.colorMap[dojoe.gauge.common.GaugeThreshold.CLEAR_STATUS         ]   = "#33cc14";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.NORMAL_STATUS        ]   = "green";//#33cc14";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.CRITICAL_STATUS      ]   = "red"; //#e51717";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.FATAL_STATUS         ]   = "#222222";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.MAJOR_STATUS         ]   = "#FFB31A";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.MINOR_STATUS         ]   = "#ffff1a";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.INDETERMINANT_STATUS ]   = "#2EA8E5";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.NOT_APPLICABLE_STATUS]   = "#6963A6";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.UNKNOWN_STATUS       ]   = "#A6A6A6";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.UNDEFINED_STATUS     ]   = "#9294f3";
            this.colorMap[dojoe.gauge.common.GaugeThreshold.INFORMATIONAL_STATUS ]   = "#2ea8e5";
        }
	},


	valueFormatter: function(value){
        return value+"";
    },
    
    getDrawOrder: function(){
        return ["drawBackground", "drawOdometerBacking", "drawOdometerText", "drawHighlight", "drawTicks", "drawBorder", "drawNeedle", "drawNubbin"];
    },
    
    getDrawSemantics: function(){
		if(this.showThresholdStrip == "on") {
			return {
				background: ["drawBackground", "drawOdometerBacking", "drawBorder"],
				ticks: ["drawTicks"],
				needle: "drawNeedle",
				foreground: ["drawHighlight"],
				updateTicks: ["drawBackground", "drawTicks", "drawNeedle"],
				updateValue: ["drawOdometerText", "drawNeedle"]
			}; 
		} else {
				return {
				background: ["drawBackground", "drawOdometerBacking", "drawBorder"],
				ticks: ["drawTicks"],
				needle: "drawNeedle",
				foreground: ["drawHighlight"],
				updateTicks: ["drawBackground", "drawTicks", "drawNeedle"],
				updateValue: ["drawBackground", "drawOdometerText", "drawNeedle"]
			};
		}
    },
        
    reset: function(){
        // summary:
        //          Deletes any locally-stored shape objects when the gauge 
        //          needs to be reset.
		if(this._backGroup)
			this._destroyContent(this._backGroup);
		if(this._needleGroup)
			this._destroyContent(this._needleGroup);
		if(this._ceilingGroup)
			this._destroyContent(this._ceilingGroup);
		if(this._trendGroup)
			this._destroyContent(this._trendGroup);
		if(this._odometerTextGroup)
			this._destroyContent(this._odometerTextGroup);
		if(this._ticksGroup)
			this._destroyContent(this._ticksGroup);
			
        delete this._backGroup;
		delete this._needleGroup;
        delete this._ceilingGroup;
        delete this._trendGroup
        delete this._odometerTextGroup;
		delete this._ticksGroup;
		
		this._backGroup=null;
		this._needleGroup = null;
		this._ceilingGroup = null;
		this._trendGroup = null;
		this._odometerTextGroup = null;
		this._ticksGroup = null;
        
    },
    
	_destroyContent:function(g){
		if(g.children){
			for(var i=0;i<g.children.length;++i){
				var s = g.children[i];
				//dojox.gfx.shape.dispose(s);
				if(s && s.children){
					this._destroyContent(s);
				}
				s.destroy();
			}
			//g.clear();
		}
		g.clear();
	},
	
	/**
	 * Note you can use the word rainbow for  threshold color
	 */
	_getColor: function(state){
		  var c=this.getThresholdFromValue(state.endValue, state.thresholds);
		  this._currentState = this.getTranslatedName(c.status);
	      if(this.showThresholdStrip == "on")
			return "#A6A6A6";
					  
		  if(c.status == "")
			return "#A6A6A6"
		  return this.colorMap[c.status];
	},
	
	getStatus: function(){
		return this._currentState;
	},
	
    _backGroup: null,
	drawBackground: function(state){
		//this._createOrClearGroup('_backGroup', state);
        if(this._backGroup ===null){
           this._backGroup = state.group.createGroup();
		}
       //this._backGroup.clear();
		var bgShape = {cx: 0, cy: 0, r: 50};
		var face = this._backGroup.children[0];
		if(!face){
			face = this._backGroup.createCircle(bgShape);
		}else{
			face.setShape(bgShape);
		}
        var c=this._getColor(state);
		face.setStroke(null).setFill({type: "linear", x1: 0, y1: -50, x2: 0, y2: 50, 
//			colors: [{offset: 0, color: "#79818b"}, {offset: 1, color: "#97a3b3"}]});
			colors: [{offset: 0, color: c}, {offset: 1, color: "#97a3b3"}]});

		if(this.showThresholdStrip == "on") {
			this._drawThresholds(state, this._backGroup);
		}
        return this._backgroup;
    },
    
	_drawThresholds: function(state, surface){		
		var radius=50-3;
		var big=0;
		var sweep_flag=1;
		this.cx = 0;
		this.cy = 0;
		for(var i=0; i<state.thresholds.length; i++) {
			if(state.thresholds[i].name == "")
				continue;
			var radians = {};
			if(i < state.thresholds.length-1) {
				if(state.thresholds[i].start < state.minValue && state.thresholds[i+1].start <= state.minValue)
					continue;
				else if(state.thresholds[i].start < state.minValue && state.thresholds[i+1].start > state.minValue)
					state.thresholds[i].start = state.minValue;
					
				radians.start = this._getRadianForThresholds(state.thresholds[i].start, state.minValue, state.maxValue, 225, 315);
				radians.end = this._getRadianForThresholds(state.thresholds[i+1].start, state.minValue, state.maxValue, 225, 315);
			} else {
				if(state.thresholds[i].start < state.minValue)
					state.thresholds[i].start = state.minValue;
				radians.start = this._getRadianForThresholds(state.thresholds[i].start, state.minValue, state.maxValue, 225, 315);
				radians.end = this._getRadianForThresholds(state.maxValue, state.minValue, state.maxValue, 225, 315);
			}	
			
			console.debug("radians : " , radians);
			var color = this.colorMap[state.thresholds[i].name];
			var a1 = radians.start;
			var a2 = radians.end;
			var x1=this.cx+radius*Math.sin(a1);
			var y1=this.cy-radius*Math.cos(a1);
			var x2=this.cx+radius*Math.sin(a2);
			var y2=this.cy-radius*Math.cos(a2);
			
			// to create a boundary around the threshold strip
			var p1=this.cx+(radius - (.09 * radius))*Math.sin(a1);
			var q1=this.cy-(radius - (.09 * radius))*Math.cos(a1);
			var p2=this.cx+(radius - (.09 * radius))*Math.sin(a2);
			var q2=this.cy-(radius - (.09 * radius))*Math.cos(a2);
			
			
			if((a2 - a1) >= Math.PI){
				big=1;
			} else if((a2/Math.PI > Math.PI/2 && a2/Math.PI < Math.PI) && (a1/Math.PI > Math.PI/2 && a1/Math.PI < Math.PI)) {
				big=0;
			}
			
			var arcs = surface.createPath();
			arcs.moveTo(x1, y1);
			//arcs.lineTo(x1, y1);
			arcs.arcTo(radius, radius,0,big,sweep_flag,x2,y2);
			arcs.lineTo(p2, q2);
			arcs.arcTo((radius-(.09 * radius)), (radius-(.09 * radius)),0,big,0,p1,q1);		
			//arcs.moveTo(x1, y1);
			arcs.lineTo(x1, y1);
			arcs.setFill(color);
			
			//arcs.closePath().setFill(color);
		
			
			var arcBoundary = surface.createPath();
			arcBoundary.moveTo(p1, q1);
			arcBoundary.arcTo((radius-(.09 * radius)), (radius-(.09 * radius)),0,big,sweep_flag,p2,q2);
			arcBoundary.setStroke({color:"#0c0c0c", width:"0.1px"});
			console.log("Threshold Strip created...");
		}
	},
	_getRadians: function(/*Number*/angle){
        return angle*Math.PI/180;
    },
	
	_createOrClearGroup: function(group, state){
        if(this[group] == null){
            this[group] = state.group.createGroup();
        }else{
            this[group].clear();
        }
        return this[group];
    },
	
	_ticksGroup:null,
    drawTicks: function(state){
		//this._createOrClearGroup('_ticksGroup', state);
		if (!this._ticksGroup)
           this._ticksGroup= state.group.createGroup();
        else
			this._ticksGroup.clear();
        //var result = state.group.createGroup();
        
        var startVal = state.minValue;
        var endVal = state.maxValue;
        var startAngle = 315;
        var endAngle = 45;
        var minorTickInterval = state.minorTickSeparation;
        var majorTickInterval = state.majorTickSeparation;
        var outerRadius = 47;
        var minorTickInnerRadius = 41;
        var majorTickInnerRadius = 35;
        var textRadius = 30;
        
		if(minorTickInterval>0){
			if((endVal - startVal)/minorTickInterval > 100)
				minorTickInterval = Math.floor((endVal - startVal)/100);
	        for(var i = startVal; i <= endVal; i = i+minorTickInterval){
	            var radians = this._getRadianForValue(i, startVal, endVal, startAngle, endAngle);
	            
	            var tick = this._ticksGroup.createLine({
	                x1: minorTickInnerRadius * Math.sin(radians),
	                y1: minorTickInnerRadius * Math.cos(radians),
	                x2: outerRadius * Math.sin(radians),
	                y2: outerRadius * Math.cos(radians)
	            });
	            tick.setStroke({width: 0.3, color: "#363636"}).setFill(null);
	            
	            // draw the start and end labels
	            // [Work Item 44900] Numeric comparison of min and max when not configured to a column
	            // Handling Javascript quirk with addition of decimal values. Rounding to 2 decimal places.
				var st= ""+ dojo.number.format(i, {round: 2});
				if (this.orient=='rtl') {
	              st=""+ dojo.number.format((endVal-i), {round: 2});
				}
				
				if(i == startVal || dojo.number.format(i, {round:2}) == endVal){
	                var text = this._ticksGroup.createText({
	                    x: textRadius * Math.sin(radians),
	                    y: textRadius * Math.cos(radians) + 2,
	                    text: st,
	                    align: "middle",
	                    textDir: this.getGfxTextDir()
	                });
	                text.setFont({
	                    size: "4px",
	                    family: "Verdana"
	                }).setFill("#363636").setStroke(null);
	                if (this.orient=='rtl') 
	                   text.setTransform(dojox.gfx.matrix.flipX);
				}
	        }
		}
        
        // draw the major ticks and the labels
		if(majorTickInterval>0){
			if((endVal - startVal)/majorTickInterval > 100)
				majorTickInterval = Math.floor((endVal - startVal)/100);
			for(var i = startVal+majorTickInterval; i < endVal; i = i+majorTickInterval){
	            var radians = this._getRadianForValue(i, startVal, endVal, startAngle, endAngle);
	            
	            var tick = this._ticksGroup.createLine({
	                x1: majorTickInnerRadius * Math.sin(radians),
	                y1: majorTickInnerRadius * Math.cos(radians),
	                x2: outerRadius * Math.sin(radians),
	                y2: outerRadius * Math.cos(radians)
	            });
	            tick.setStroke({width: 0.3, color: "#363636"}).setFill(null);
	            
	            // draw the labels
	            var st= ""+dojo.number.format(i, {round: 2});
				if (this.orient=='rtl') {
	              st=""+ dojo.number.format((endVal - i), {round: 2});
				}
	
				var text = this._ticksGroup.createText({
	                x: textRadius * Math.sin(radians),
	                y: textRadius * Math.cos(radians) + 2,
	                text: st,
	                align: "middle",
	                textDir: this.getGfxTextDir()
	            });
	            text.setFont({
	                size: "4px",
	                family: "Verdana"
	            }).setFill("#363636").setStroke(null);
	            if (this.orient=='rtl') 
	            	text.setTransform(dojox.gfx.matrix.flipX);
			}
		}
        
        var bgShape = {
			x: 14+5, y: 33.5-5,
			width: 7, height: 7,
            src: dojo.moduleUrl("dojoe.gauge.common.images", "warning.png")
		};
		console.info("AnalogGauge _addWarning:"+this._addWarning);
		if(this._addWarning) {
			if(!this.warningNode) {
				var warningNode = this._ticksGroup.createImage(bgShape);
			} else {
				this.warningNode.setShape(bgShape);
			}
		} else {
			if(this.warningNode) {
				bgShape.src = "";
				this.warningNode.setShape(bgShape);
			}
		}
        return this._ticksGroup;
    },
    
    drawBorder: function(state){
        var result = state.group.createGroup();
        
        var border = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojoe.gauge.analoggauge.AnalogStyle", "default_border.png")
        });
        return result;
    },
    
    drawHighlight: function(state){
        var result = state.group.createGroup();
        
        var highlight = result.createImage({
            x: -50, y: this.showThresholdStrip == "on"?-44 : -50, 
            width: 100, height: this.showThresholdStrip == "on"?80:100, 
            src: dojo.moduleUrl("dojoe.gauge.analoggauge.AnalogStyle", "default_light.png")
        });
        
        return result;
    },
    
    drawOdometerBacking: function(state){
        var result = state.group.createGroup();
        
        var odometerBacking = result.createImage({
            x: -50, y: -50, 
            width: 100, height: 100, 
            src: dojo.moduleUrl("dojoe.gauge.analoggauge.AnalogStyle", "default_odometer.png")
        });
        
        return result;
    },
    
    _odometerTextGroup: null,
    _odometerText: null,
    drawOdometerText: function(state){
		//if(isNaN(state.endValue))
			//state.endValue = state.minValue;
        var fontSize = "5px";
        var align = "middle";
        var x = 14;
        var y = 33.5;
		var displaytext="";
		if(this.getValueVisible() ){
			// 80344: TWL 1.1.0.2 : [No Data Status] Analog Gauge/Volume Bar Values are not translated
			if(isNaN(state.endValue))
				displaytext+=this.valueFormatter(this.getTranslatedName(state.endValue));
			else
				displaytext+=this.valueFormatter(state.endValue);
			// if(this.valueStyle && this.valueStyle.fontSize){
				// fontSize=this.valueStyle.fontSize+"px";
			// }
		}
		if(this.getValueVisible() && this.getLabelVisible()){
			displaytext+=" ";
		}
		if(this.getLabelVisible()){
			displaytext+=state.units;
            // Work as design, not set to font size
			// if(!this.getValueVisible() && this.labelStyle && this.labelStyle.fontSize){
				// fontSize=this.labelStyle.fontSize+"px";
			// }
		}
		
        if(this._odometerTextGroup === null){
            this._odometerTextGroup = state.group.createGroup();
            this._odometerText = this._odometerTextGroup.createText({
                x: x, y: y, 
                text: displaytext, 
                align: align,
                textDir: this.getGfxTextDir()
            }).setFill("#000000");
            this._odometerText.setFont({
                size: fontSize, family: "Verdana, sans-serif"
            });
            if (this.orient=='rtl') 
                this._odometerText.setTransform(dojox.gfx.matrix.flipX);
		}
        
        
        // Animate it! (if it's an integer value to an integer value... decimals look weird)
        if(this.dontAnimate && this.getValueVisible()){
			this._odometerText.setShape({
                x: x, y: y, 
                text: this.valueFormatter(state.startValue), 
                align: align
            });
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentValue = Math.floor(percent * (state.endValue - state.startValue) + state.startValue);
                    if(this._odometerText != null){
						if (step==1) {
                           currentValue = state.endValue;
						   if (state.formatOptions!=null) 
                             currentValue=    dojo.number.format(currentValue, state.formatOptions);
						}
						this._odometerText.setShape({
                            x: x, y: y, 
                            text: this.valueFormatter(currentValue) + "" + state.units, 
                            align: align
                        });
                    }
                })
            }).play();
        } else {
			var currentValue = state.endValue;
			if (state.formatOptions!=null) 
                  currentValue = dojo.number.format(state.endValue, state.formatOptions);
			this._odometerText.setShape({
                x: x-14, y: y, 
                text: displaytext, 
                align: "middle"
            });
        }
		/*var bgShape = {
			x: x + 5, y: y-5,
			width: 7, height: 7,
            src: dojo.moduleUrl("dojoe.gauge.common.images", "warning.png")
		};
		if(this._addWarning) {
			if(!this._odometerTextGroup.children[1]) {
				var warningImage = this._odometerTextGroup.createImage(bgShape);
			} else {
				this._odometerTextGroup.children[1].setShape(bgShape);
			}
		} else {
			if(this._odometerTextGroup.children[1]) {
				bgShape.src = "";
				this._odometerTextGroup.children[1].setShape(bgShape);
			}
		}*/
        return this._odometerTextGroup;
    },
    
    _getRadianForValue: function(value, minValue, maxValue, minAngle, maxAngle){
        var minRadians = minAngle * Math.PI / 180.0;
        var maxRadians = maxAngle * Math.PI / 180.0;
        var radianRange = maxRadians - minRadians;
        if((value - minValue) > (maxValue - minValue)){
        	return minRadians + radianRange;
        }else{
        	return minRadians + radianRange * ((value - minValue) / (maxValue - minValue));
        }
    },
    
    _getRadianBounds: function(startValue, endValue, minValue, maxValue, minAngle, maxAngle){
        var result = {};
        result.start = this._getRadianForValue(startValue, minValue, maxValue, minAngle, maxAngle);
        result.end = this._getRadianForValue(endValue, minValue, maxValue, minAngle, maxAngle);
        return result;
    },
    
	 _getRadianForThresholds: function(value, minValue, maxValue, minAngle, maxAngle){
        var minRadians = minAngle * Math.PI / 180.0;
        var maxRadians = maxAngle * Math.PI / 180.0;
        var radianRange = 2 * Math.PI - (maxRadians - minRadians);
        if((value - minValue) > (maxValue - minValue)){
        	return minRadians + radianRange;
        }else{
        	return minRadians + radianRange * ((value - minValue) / (maxValue - minValue));
        }
    },
	
    _needleGroup: null,
    drawNeedle: function(state){
	
		if(this.showThresholdStrip == "on") {
			var c=this.getThresholdFromValue(state.endValue, state.thresholds);
			this._currentState = this.getTranslatedName(c.name);
		}
		
		if(isNaN(state.endValue))
			state.endValue = state.minValue - 1;
			
        if(this._needleGroup === null){
            this._needleGroup = state.group.createGroup();
            
            var pointerShadow = this._needleGroup.createPath();
            pointerShadow.moveTo({x: -1.0, y: -11.667}).lineTo({x: 0.5, y: -36.433 - 11.667}).lineTo({x: 2.0, y: -11.67}).lineTo({x: -1.0, y: -11.667});
            pointerShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var pointer = this._needleGroup.createPath();
            pointer.moveTo({x: -1.5, y: -11.667}).lineTo({x: 0, y: -36.433 - 11.667}).lineTo({x: 1.5, y: -11.67}).lineTo({x: -1.5, y: -11.667});
            pointer.setStroke(null).setFill(this.needleColor);
            
            
            var baseShadow = this._needleGroup.createRect({x: -1.0, y: -11.222, width: 3.0, height: 11.222});
            baseShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var base = this._needleGroup.createRect({x: -1.5, y: -11.222, width: 3.0, height: 11.222});
            base.setStroke(null).setFill("#414042");
        }
        
		if(state.endValue < state.minValue)
			state.endValue = state.minValue - ((state.maxValue - state.minValue)/60);
		
		if(state.startValue < state.minValue)
			state.startValue = state.minValue - ((state.maxValue - state.minValue)/60);
			
        var radianBounds = this._getRadianBounds(state.startValue, state.endValue, state.minValue, state.maxValue, -135, 135);
        
        var startRadians = radianBounds.start;
        var endRadians = radianBounds.end;
		if(state.endValue > state.maxValue)
			endRadians = endRadians + (22/(40 * 7));
		
		if(state.startValue > state.maxValue)
			startRadians = startRadians + (22/(40 * 7));
			
        this._needleGroup.setTransform(dojox.gfx.matrix.rotate(startRadians));
        
        if(!this.dontAnimate){
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentRadians = percent * (endRadians - startRadians) + startRadians;
                    if(this._needleGroup){
                        this._needleGroup.setTransform(dojox.gfx.matrix.rotate(currentRadians));
                    }
                })
            }).play();
        } else {
            this._needleGroup.setTransform(dojox.gfx.matrix.rotate(endRadians));
        }
        
        return this._needleGroup;
    },
    
    _ceilingGroup: null,
    drawCeilingIndicator: function(state){
        if(this._ceilingGroup === null){
            this._ceilingGroup = state.group.createGroup();
            
            var ballShadow = this._ceilingGroup.createCircle({cx: 0, cy: 0.5, r: 3.33});
            ballShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var ball = this._ceilingGroup.createCircle({cx: 0, cy: 0.0, r: 3.33});
            ball.setStroke(null).setFill(this.ceilingColor);
            
            var ballHighlight = this._ceilingGroup.createEllipse({cx: 0, cy: -0.5, rx: 2.67, ry: 2.266});
            ballHighlight.setStroke(null).setFill({type: "linear", x1: 0, y1: -2.7, x2: 0, y2: 1.7, colors: [{offset: 0, color: "rgba(209,211,212,1.0)"}, {offset: 1, color: "rgba(209,211,212,0.0)"}]});
        }
        
        //if(!state.startCeilingValue || !state.endCeilingValue){        
        // Line above causes problems if startCeilingValue is 0 which is very possible!
        if(!state.showCeiling){
            this._ceilingGroup.clear();
            return this._ceilingGroup;
        }
        
        var radianBounds = this._getRadianBounds(state.startCeilingValue, state.endCeilingValue, state.minValue, state.maxValue, 315, 45);
        
        var startRadians = radianBounds.start;
        var endRadians = radianBounds.end;
        
        function generateMatrix(radians){
            return {xx: 1, xy: 0, yx: 0, yy: 1, dy: 42 * Math.cos(radians), dx: 42 * Math.sin(radians)};
        }
        
        this._ceilingGroup.setTransform(generateMatrix(startRadians));
        
        if(!this.dontAnimate){
            // Animate it!
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentRadians = percent * (endRadians - startRadians) + startRadians;
                    
                    if(this._ceilingGroup){
                        this._ceilingGroup.setTransform(generateMatrix(currentRadians));
                    }
                })
            }).play();
        } else {
            this._ceilingGroup.setTransform(generateMatrix(endRadians));
        }
        
        return this._ceilingGroup;
    },
    
    _trendGroup: null,
    drawTrendIndicator: function(state){
        if(this._trendGroup === null){
            this._trendGroup = state.group.createGroup();
            
            var pointerShadow = this._trendGroup.createPath();
            pointerShadow.moveTo({x: -1.0, y: -11.667}).lineTo({x: 0.5, y: -36.433 - 11.667}).lineTo({x: 2.0, y: -11.67}).lineTo({x: -1.0, y: -11.667});
            pointerShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var pointer = this._trendGroup.createPath();
            pointer.moveTo({x: -1.5, y: -11.667}).lineTo({x: 0, y: -36.433 - 11.667}).lineTo({x: 1.5, y: -11.67}).lineTo({x: -1.5, y: -11.667});
            pointer.setStroke(null).setFill(this.trendColor);
            
            
            var baseShadow = this._trendGroup.createRect({x: -1.0, y: -11.222, width: 3.0, height: 11.222});
            baseShadow.setStroke(null).setFill("rgba(0,0,0,0.5)");
            
            var base = this._trendGroup.createRect({x: -1.5, y: -11.222, width: 3.0, height: 11.222});
            base.setStroke(null).setFill("#414042");
        }
        
        //if(!state.startTrendValue || !state.endTrendValue){
        // Line above causes problems if startTrendValue is 0 which is very possible!
        if(!state.showTrend){
            this._trendGroup.clear();
            return this._trendGroup;
        }
        
        if(this.trendValue < state.endValue){
        	this.trendValue = state.endValue;
        }
        
        var radianBounds = this._getRadianBounds(this.trendValue, this.trendValue, state.minValue, state.maxValue, -135, 135);
        
        var startRadians = radianBounds.start;
        var endRadians = radianBounds.end;
        
        this._trendGroup.setTransform(dojox.gfx.matrix.rotate(startRadians));
        
        if(!this.dontAnimate){
            new dojo._Animation({
                curve: [0, 1], 
                duration: 1000, 
                easing: dojo.fx.easing.quadratic,
                onAnimate: dojo.hitch(this, function(step){
                    var percent = step;
                    var currentRadians = percent * (endRadians - startRadians) + startRadians;
                    
                    if(this._trendGroup){
                        this._trendGroup.setTransform(dojox.gfx.matrix.rotate(currentRadians));
                    }
                })
            }).play();
        } else {
            this._trendGroup.setTransform(dojox.gfx.matrix.rotate(endRadians));
        }
        
        return this._trendGroup;
    },
    
    drawNubbin: function(state){
        var result = state.group.createGroup();
        
        var shadow = result.createCircle({cx: 0, cy: 1, r: 6.666});
        shadow.setStroke(null).setFill("rgba(65,64,66,0.5)");;
        
        var nubbin = result.createCircle({cx: 0, cy: 0, r: 6.666});
        nubbin.setStroke(null).setFill("#414042");
        
        var highlight = result.createEllipse({cx: 0, cy: -1.4, rx: 5.6, ry: 4.6});
        highlight.setStroke(null).setFill({type: "linear", x1: 0, y1: -6, x2: 0, y2: 4, colors: [{offset: 0, color: "rgba(209,211,212,1.0)"}, {offset: 1, color: "#414042"}]});
        
        return result;
    }
});
});
