/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("dojoe/map/Map", [ 
  "dojo/_base/declare", 
  "dojo/_base/lang",
  "dojo/_base/Deferred",
  "dijit/_WidgetBase",
  "dijit/Tooltip",
  "dojo/query",
  "dojoe/Util", 
  "dojo/io/script",
  "dojox/widget/Standby",
  "dojo/_base/xhr"
], function(declare, lang, Deferred, _WidgetBase, Tooltip, query, Util, script, Standby, xhr) {
    return declare("dojoe.map.Map", [_WidgetBase], {
        mapType : "local",
        apiKey : "",
        mapLocalURL : "/ibm/console/secure/isclite/scripts/tipdojo/dojoe/map/geojson/countries.geojson",
        zoomLayers : null,
        zoomToObj : null,
        map : null,
        mapDivId: null,
        initCallback: null,
		markerLabelEnabled: false,
		markerResourceIconEnabled: true,
        
        types: {
            'GOOGLE': 'google',
            'BING': 'bing',
            'LOCAL': 'local',
            'OSM': 'osm'
        },     
                
        createMap: function(mapDivId, mapArgs, initCallback) {
            console.log("Map - initMap - Start");
            
            this.mapDivId = mapDivId;
            this.initCallback = initCallback;
                        
            if(this.mapDivId) {
                this._setParams(mapArgs);
                this._loadPrereqs();
            }
        },
        
        addLayer: function(layer) {
            this.map.addLayer(layer);
        },     
    
        getCoordinates : function(lon, lat) {
            console.log("Map: getCoordinates :BEGIN");
			console.debug("Maps.js:getCoordinates():lat:"+lat);
			console.debug("Maps.js:getCoordinates():lon:"+lon);
			
            var lonLat = new OpenLayers.LonLat(lon, lat);
            var proj = this.map.getProjectionObject();
            
            if (this.mapType == this.types.GOOGLE) {
                lonLat = lonLat.transform(new OpenLayers.Projection("EPSG:4326"), new OpenLayers.Projection("EPSG:900913"));
            } else {
                lonLat = lonLat.transform(new OpenLayers.Projection("EPSG:4326"), proj);
            }            
            
            return lonLat;
            console.log("Map: getCoordinates :END");
        },
        
        placeMarkerOnMap: function(markerData, onSelect) {
		
			console.debug("Map.js:placeMarkerOnMap():markerData.lat:"+markerData.lat);
			console.debug("Map.js:placeMarkerOnMap():markerData.lon:"+markerData.lon);
		
			//Bobby: 06Feb2014, transform is painfully slow for out of range values, ignore them for now
			if(markerData.lat < -180 || markerData.lat > 360){
				console.debug("lat Outside bounds");
				return;
			}	
			if(markerData.lon < -180 || markerData.lon > 360){
				console.debug("lon Outside bounds");
				return;
			}	
		
            if(markerData.zoomTo > 0) {
                this.zoomToObj.enabled = true;
                this.zoomToObj.lat = markerData.lat;
                this.zoomToObj.lon = markerData.lon;
                this.zoomToObj.zoom = markerData.zoomTo;
                this.onPortletResize();
            }
            
            var createLayer = true;
            
            for ( var j = 0; j < this.zoomLayers.length; j++) {
                if ((this.zoomLayers[j].minZoom == markerData.minZoom) && (this.zoomLayers[j].maxZoom == markerData.maxZoom)) {
                    console.log("Adding marker to already existing zoom layer:"+j);
                    createLayer = false;
                    this._addMarker(this.zoomLayers[j].layer, markerData);
                }
            }

            //Layer doesn't exist, create new layer
            if (createLayer == true) {
                var name = "Zoom Layer (" + markerData.minZoom + "," + markerData.maxZoom + ")";
                console.log("Creating new zoom layer for marker: " + name);
				//console.debug("placeMarkerOnMap:markerData.bgImage:"+markerData.bgImage);
				//console.debug("placeMarkerOnMap:markerData.url:"+markerData.bgImage.url);
				//console.debug("placeMarkerOnMap:markerData.height:"+markerData.bgImage.height);
				//console.debug("placeMarkerOnMap:markerData.width:"+markerData.bgImage.width);
			
				var styleMapArgs = {
					externalGraphic : '${UIMapMarkerImage}',
					fontWeight: 'bold',
					graphicWidth : '${graphicWidth}',
					graphicHeight : '${graphicHeight}',
					graphicYOffset : '${graphicYOffset}',
					graphicXOffset : '${graphicXOffset}',
					title : '${UIMapMarkerTooltip}'
				};
				
				console.debug("Map.js:placeMarkerOnMap():markerLabelEnabled:"+this.markerLabelEnabled);
				if(this.markerLabelEnabled){
					styleMapArgs['label'] = '${UIMapMarkerLabel}';
					styleMapArgs['labelYOffset'] = -10;
					styleMapArgs['labelXOffset'] = 10;
				}
				
				//44471: Impact/Pulse: Ability to disable the default icon image in the map widget
				console.debug("Map.js:placeMarkerOnMap():markerResourceIconEnabled:"+this.markerResourceIconEnabled);
				if(this.markerResourceIconEnabled){
					if(markerData.bgImage.url && markerData.bgImage.height){
						styleMapArgs['backgroundGraphic'] = '${UIMapMarkerBgImage}';
						styleMapArgs['backgroundWidth'] = '${graphicBgWidth}';
						styleMapArgs['backgroundHeight'] = '${graphicBgHeight}';
						//console.debug("markerData.bgImage.width:"+markerData.bgImage.width);
						//44720: Map Widget: Markers are slightly off from the lat/long provided
						//Bobby, 13Feb2014: Need to adjust status as resource icon position was changed for #44720
						styleMapArgs['graphicXOffset'] = markerData.bgImage.width/5;
						
						if(this.markerLabelEnabled){
							styleMapArgs['labelYOffset'] = -20;
							styleMapArgs['labelXOffset'] = 0;
						}
					}
				}			
				
				var layer = new OpenLayers.Layer.Vector(name, {
						styleMap : new OpenLayers.StyleMap(styleMapArgs)
					}
				);		

				/*var layer = new OpenLayers.Layer.Vector(name, {
						styleMap : new OpenLayers.StyleMap({
							externalGraphic : '${UIMapMarkerImage}',
							//label : '${UIMapMarkerLabel}',
							fontWeight: 'bold',
							//labelYOffset: -20,
							graphicWidth : '${graphicWidth}',
							graphicHeight : '${graphicHeight}',
							graphicYOffset : '${graphicYOffset}',
							graphicXOffset : '${graphicXOffset}',
							title : '${UIMapMarkerTooltip}',
							backgroundGraphic: '${UIMapMarkerBgImage}',
							backgroundWidth: '${graphicBgWidth}',
							backgroundHeight: '${graphicBgHeight}'
						})
					}
				);*/					
                
                this.map.addLayer(layer);

                if (markerData.minZoom > 0) {
                    layer.setVisibility(false);
                }

                var zoomLayer = {
                    minZoom : markerData.minZoom,
                    maxZoom : markerData.maxZoom,
                    layer : layer
                };
                
                this.zoomLayers.push(zoomLayer);

                //Add Marker to Newly Created Layer
                this._addMarker(layer, markerData);

                //Add control when marker is selected
                var selectControl = new OpenLayers.Control.SelectFeature(layer, {
                    onSelect : onSelect
                });
                
                this.map.addControl(selectControl);
                selectControl.activate();
            }
        },
        
        registerEvent: function(type, listener) {
            this.map.events.register(type, this, listener);
        },
        
        setCenter: function(LonLat, zoomTo) {
            this.map.setCenter(LonLat);
            this.map.zoomTo(zoomTo);
        },
        
        getCenter: function() {
            var center = new Object();
            var mapCenter = this.map.getCenter();
            
            mapCenter.transform(this.map.getProjectionObject(), new OpenLayers.Projection("EPSG:4326"));
            
            center.lat = mapCenter.lat;
            center.lon = mapCenter.lon;
            center.zoom = this.map.getZoom();
            
            return center;
        },
        
        getMapDiv: function() {
            if(this.map) {
                return this.map.div;
            }
            
        },
        
        getBaseLayer: function() {
            if(this.map) {
                return this.map.baseLayer;
            }
            
            return null;
        },
        
        getNumZoomLevels: function() {
            return this.map.getNumZoomLevels();
        },
        
        updateSize: function() {
            if(this.map && this.map.baseLayer) {
                this.map.updateSize();
            }            
        },
        
        zoomToExtent: function(bounds, closest) {
            this.map.zoomToExtent(bounds, closest);
        },
        
        destroy: function() {
            if(this.map) {
                this.map.destroy();
            }
        },
        
        onPortletResize: function() {
            if(this.map) {
                this.updateSize();
                
                if (this.zoomToObj.enabled == true && this.map.baseLayer) {
                    var LonLat = this.getCoordinates(this.zoomToObj.lon, this.zoomToObj.lat);
                    this.setCenter(LonLat, this.zoomToObj.zoom);
                }
            }
        },
        
        _destroyMarker: function(layer, id) {
            for(var i = 0; i < layer.features.length; i++) {
                var feature = layer.features[i];
                
                if(feature.data.id === id) {
                    layer.removeFeatures(feature);
                }
            }
        },
        
        destroyMarkers: function(ids) {            
            for(var i = 0; i < this .zoomLayers.length; i++) {
                var layer = this.zoomLayers[i].layer;
                for(var j = 0; j < ids.length; j++) {
                    var id = ids[j];
                    this._destroyMarker(layer, id);
                }
            }
        },
        
        destroyMarker: function(id) {
            for(var i = 0; i < this .zoomLayers.length; i++) {
                var layer = this.zoomLayers[i].layer;
                this._destroyMarker(layer, id);
            }
        },
        
        destroyAllMarkers: function() {
            for(var i = 0; i < this.zoomLayers.length; i++) {
                var layer = this.zoomLayers[i].layer
                
                this.map.removeLayer(layer);
            }
            
            this.zoomLayers = [];
        },
        
        setOnContextMenu: function(onContextMenu) {
            this.onContextMenu = onContextMenu;
        },
        
        onContextMenu: function(feature, event) {
            //To be (or not to be) overridden (that is the question)
        },
        
        _addMarker : function(layer, markerData) {
            console.log("Map: addMarker :BEGIN");
            
            //Destroy the marker if it already exists
            this._destroyMarker(layer, markerData.id);
            
            if(markerData.lat && markerData.lon) {
                var LonLat = this.getCoordinates(markerData.lon, markerData.lat);            
                
                var point = new OpenLayers.Geometry.Point(LonLat.lon, LonLat.lat);
        
				var marker;
				//console.debug("_addMarker:markerData.bgImage:"+markerData.bgImage);
				//console.debug("_addMarker:markerData.image.xOffset:"+markerData.image.xOffset);
				//console.debug("_addMarker:markerData.image.yOffset:"+markerData.image.yOffset);
		
				marker = new OpenLayers.Feature.Vector(point, {
							UIMapMarkerMinZoom: markerData.minZoom,
							UIMapMarkerMaxZoom: markerData.maxZoom,
							UIMapMarkerZoomTo:  markerData.zoomTo,                    
							UIMapMarkerTooltip: markerData.tooltip,                    
							UIMapMarkerLabel:   markerData.label,
							UIMapMarkerLon:     markerData.lon,
							UIMapMarkerLat:     markerData.lat,
							UIMapMarkerImage:   markerData.image.url,
							graphicHeight:      markerData.image.height,
							graphicWidth:       markerData.image.width,
							graphicYOffset:     markerData.image.yOffset,
							graphicXOffset:     markerData.image.xOffset,
							UIMapMarkerBgImage: markerData.bgImage.url,
							graphicBgHeight:    markerData.bgImage.height,
							graphicBgWidth:     markerData.bgImage.width,
							id:                 markerData.id
				});              
				layer.addFeatures([ marker ]);
			}				
            console.log("Map: addMarker :END");
        },  
        
        _onContextMenu: function(e) {
            var feature = null;
            if(!e){ //dear IE... 
                var e = window.event; 
                e.returnValue = false; 
            } 
            
            for(var i = 0; i < this.zoomLayers.length; i++) {
                var layer = this.zoomLayers[i].layer;
                
                feature = layer.getFeatureFromEvent(e);
            }
            
            this.onContextMenu(feature, e);

            return false; //Prevent display of browser context menu 
        },
        
        _setParams: function(mapArgs) {
            this.zoomLayers = [];
            this.zoomToObj = new Object();
            this.zoomToObj.enabled = false;
            this.zoomToObj.lat = 0;
            this.zoomToObj.lon = 0;
            this.zoomToObj.zoom = 0;
            
            for(var arg in mapArgs) {       
                var value = mapArgs[arg];
				//console.debug("Map.js:_setParams:arg:"+arg);
				//console.debug("Map.js:_setParams:value:"+value);
                switch(arg) {
                case "mapType":
                    this.mapType = value;
                    break;
                    
                case "apiKey":
                    this.apiKey = value;
                    break;
                    
                case "mapLocalUrl":
                    this.mapLocalURL = value;
                    break;    
                    
                case "mapCenterLat":    
                    this.zoomToObj.lat = value;
                    break;
                
                case "mapCenterLon":
                    this.zoomToObj.lon = value;
                    break;
                
				case "markerLabelEnabled":
                    if(value == "true") this.markerLabelEnabled = true;
                    break;
					
				case "markerResourceIconEnabled":
                    if(value == "true") this.markerResourceIconEnabled = true;
					else this.markerResourceIconEnabled = false; //since initialized to true.
                    break;	
				
                case "mapCenterZoom":
                    this.zoomToObj.zoom = value;
                    
                    if(this.zoomToObj.zoom > 0) {
                        this.zoomToObj.enabled = true;
                    }
                    
                    break;
                }
            }
            
            if(!this.apiKey) {
                this.apiKey = '';
            }
        },
        
        _initMap : function() {
            this.map = new OpenLayers.Map(this.mapDivId);
            
            switch(this.mapType) {
                case this.types.BING:
                    this._initBing();
                    break;
                case this.types.GOOGLE:
                    this._initGoogle();
                    break;
                case this.types.OSM:
                    this._initOsm();
                    break;
                case this.types.LOCAL:
                    this._initLocal();
                    break;
                default:
                    this._initLocal();
                break;                
           }
    
           this.updateSize();
           //Problem with google maps
           if (this.mapType != this.types.GOOGLE) {
               this.map.zoomToMaxExtent();
           }
    
           this.registerEvent('zoomend', function(event) {
                var zoomLevel = this.map.getZoom();
    
                for ( var i = 0; i < this.zoomLayers.length; i++) {
                    var layer = this.zoomLayers[i].layer;
                    if (zoomLevel >= this.zoomLayers[i].minZoom && zoomLevel <= this.zoomLayers[i].maxZoom) {
                        console.log("Showing layer: " + layer.name);
                        layer.setVisibility(true);
                    } else {
                        console.log("Hidding layer: " + layer.name);
                        layer.setVisibility(false);
                    }
                }
            });   
            
            //Set initial zoom
            this.onPortletResize();
            
            this.map.div.oncontextmenu = lang.hitch(this, this._onContextMenu);            
            
            if(this.initCallback) {
                if(this.mapType === this.types.GOOGLE) {
                    _mapApiReady.then(lang.hitch(this, function() { 
                        this.initCallback();
                    }))
                } else {
                    this.initCallback();
                }
            }
            
    
            console.log("Map: initMap :END");
        },
            
        _initLocal : function() {
            console.log("Map: _initLocal :BEGIN");
			console.debug("this.mapLocalURL:"+ this.mapLocalURL);
			xhr.get({
				//url: this.mapLocalURL+"/\index.json", 
				url: this.mapLocalURL, 
				sync: true,
				handleAs: "json",
				load: lang.hitch(this, "_processData") 
			});
			console.log("Map: _initLocal :END");
		},	
			
		_processData : function(geoJSONIndex) {
			console.log("Map: _processData :BEGIN");
			console.debug("this.geoJSONIndex:"+geoJSONIndex);
			console.debug("this.geoJSONIndex.length :"+geoJSONIndex.length);
			
			
			/*var isBase = true;
			for ( var geoJSON in geoJSONIndex) {
                var name = "GeoJSON-" + i;
                console.log("Adding geoJSON layer: " + name + " - " + geoJSON);
                var GeoJSONLayer = new OpenLayers.Layer.Vector(name, {
                        isBaseLayer : isBase,
                        strategies : [ new OpenLayers.Strategy.Fixed() ],
                        protocol : new OpenLayers.Protocol.HTTP({
							url : this.mapLocalURL+"/"+geoJSONIndex[geoJSON],
							format : new OpenLayers.Format.GeoJSON(),
							wrapeDateLine: true
						})
                });
                
                this.map.addLayers([ GeoJSONLayer ]);
				isBase = false;
            }*/
			
            
            var urls = this.mapLocalURL.split(":");			
			console.debug("Map.js:urls:"+urls);
            
            for ( var i = 0; i < urls.length; i++) {
                var name = "GeoJSON-" + i;
                var isBase = false;
                if (i == 0) {
                    isBase = true;
                }
                console.log("Adding geoJSON layer: " + name);
                var GeoJSONLayer = new OpenLayers.Layer.Vector(name, {
                        isBaseLayer : isBase,
                        strategies : [ new OpenLayers.Strategy.Fixed() ],
                        protocol : new OpenLayers.Protocol.HTTP({
                        url : urls[i],
                        format : new OpenLayers.Format.GeoJSON(),
                        wrapeDateLine: true
                    })
                });
                
                this.map.addLayers([ GeoJSONLayer ]);
            }
    
            console.log("Map: _processData :END");
        },
    
        _initOsm : function() {
            console.log("Map: _initOsm :BEGIN");
            var osm = new OpenLayers.Layer.OSM();
            this.map.addLayer(osm);
            console.log("Map: _initOsm :END");
        },
    
        _initBing : function() {
            console.log("Map: _initBing :BEGIN");
            
            if(this.apiKey && this.apiKey != "") {
                var road = new OpenLayers.Layer.Bing({
                    name : "Road",
                    key : this.apiKey,
                    type : "Road",
                    wrapeDateLine: true                    
                });
                
                this.map.addLayers([road]);
            }
            
            console.log("Map: _initBing :END");
        },
    
        _initGoogle : function() {
            console.log("Map: _initGoogle :BEGIN");
            
            this._initGoogleApi();
            
            _mapApiReady.then(lang.hitch(this, function() {
                var gmap = new OpenLayers.Layer.Google("Google Streets", {
                    numZoomLevels: 20
                });
                
                this.map.addLayer(gmap);
        
                console.log("Map: _initGoogle :END");
                
                this.onPortletResize();
            }));
        },
        
        _initGoogleApi: function() {
            if(!_mapApiRequested) {
                _mapApiRequested = true;
                var tag = document.createElement("script");
                tag.type="text/javascript";
                tag.src= 'https://maps.googleapis.com/maps/api/js?key=' + this.apiKey + '&language=' + dojo.locale + '&sensor=false&callback=_mapApiLoaded&libraries=places';
                document.body.appendChild(tag);
            }
        },
        
        _loadPrereqs: function() {
            this._loadStylesheet({query: 'Map_common.css', href: "/ibm/console/secure/isclite/scripts/tipdojo/themes/common/dojoe/ssd/quickedit/themes/Map_common.css"});
            this._loadStylesheet({query: 'google.css', href: "/ibm/console/secure/isclite/scripts/tipdojo/themes/common/dojoe/ssd/quickedit/themes/google.css"});
            this._loadStylesheet({query: 'openlayers/theme/default/style.css', href: "/ibm/console/secure/isclite/scripts/tipdojo/openlayers/theme/default/style.css"});

            this._loadOpenLayers();
        },        
                
        _loadOpenLayers: function() {
            var elements = query("script[src*='OpenLayers.debug.js']")
            
            if(elements.length == 0) {
                var tag = document.createElement("script");
                tag.src= "/ibm/console/secure/isclite/scripts/tipdojo/openlayers/OpenLayers.debug.js";
                document.body.appendChild(tag);
                
                var handle = dojo.connect(tag, "load", lang.hitch(this, function(evt) {
                    handle.remove();
                    this._initMap();
                }));
            } else if(typeof(OpenLayers) == 'undefined') {
                var handle = dojo.connect(elements[0], "load", lang.hitch(this, function(evt) {
                    handle.remove();
                    this._initMap();
                }));
            } else {
                this._initMap();
            }
        },
        
        _loadStylesheet: function(sArgs) {
            if(query("link[href*='" + sArgs.query + "']").length == 0) {
                var l = document.createElement("link");
                l.href= sArgs.href;
                l.rel="stylesheet";
                l.type="text/css";
                if(dojo.isIE < 9) {
                    document.getElementsByTagName('head')[0].appendChild(l);
                }else {
                    document.head.appendChild(l);
                }
            }
        }
    });
});

//MUST LEAVE THIS GLOBAL IN ORDER TO RENDER GOOGLE MAP API ACROSS ALL THE DIFFERENT DASHLETS
window._mapApiLoaded = function() {
    _mapApiReady.resolve();
};

//setup google maps Api
if(typeof(_mapApiReady) == 'undefined') {
    _mapApiReady = new dojo.Deferred();
}

if(typeof(_mapApiRequested) == 'undefined') {
    _mapApiRequested = false;
} 