/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
/******************************************************* {COPYRIGHT-TOP-RM} ***
* Licensed Materials - Property of IBM
*
* "Restricted Materials of IBM"
*
* 5724-C51
* (C) Copyright IBM Corp. 2011 All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
******************************************************* {COPYRIGHT-END-RM} ***/
define("dojoe/timeselector/Time8601", [
        "dojo/date",
        "dojo/date/stamp",
        "dojo/_base/lang",
        "dojo/_base/declare"
        ], function(date, stamp, lang, declare){
    
    var DurationTime8601 = declare("dojoe.timeselector.DurationTime8601", 
        null,
        {
          duration: null,
          _last: true,
          _realTime: false,
          _years: 0,
          _months: 0,
          _days: 0,
          _hours: 0,
          _minutes: 0,
          _seconds: 0,
          
          /**
           * Creates an instance of DurationTime8601. kwArgs accepts the following properties:
           *
           * Required properties:
           * duration - time in ISO8601. Supported format:
           *        -P0Y0M0DT24H0M  Relative period: The last 24 hours from current time
           *        +P0Y0M0DT24H0M  Relative period: The next 24 hours from current time 
           */
          constructor: function(kwArgs) {
            lang.mixin(this, kwArgs);
            this._parseDuration();
          },
          
          /** 
           * Returns a boolean whether this duration indicates the past
           */   
          isLast: function() {
            return this._last;
          },

          isRealTime: function() {
            return this._realTime;
          },
          
          getYears: function() {
            return this._years;
          },
          
          getMonths: function() {
            return this._months;
          },
          
          getDays: function() {
            return this._days;
          },
          
          getHours: function() {
            return this._hours;
          },
          
          getMinutes: function() {
            return this._minutes;
          },
          
          getSeconds: function() {
            return this._seconds;
          },
          
          _parseDuration: function() {
            if ( !this.duration ) {
              return;
            }
            if ( this.duration.substring(0, 1) == "+" ) {
              // + means future duration
              this._last = false;
            }
            // iterate the duration string starting from position 2 skipping +/- and P 
            var lastDigitPosition = 2;
            var processedMonth = false;
            for ( var i = 2; i < this.duration.length; i++ ) {
              if ( this.duration[i] == "Y" ) {
                this._years = this.duration.substring(lastDigitPosition, i);
                lastDigitPosition = i+1; // next char should be the digit 
              } else if ( this.duration[i] == "M" && !processedMonth) {
                this._months = this.duration.substring(lastDigitPosition, i);
                processedMonth = true;
                lastDigitPosition = i+1; // next char should be the digit 
              } else if ( this.duration[i] == "D" ) {
                this._days= this.duration.substring(lastDigitPosition, i);
                lastDigitPosition = i+2; // 2nd char should be the digit because of "T"
              } else if ( this.duration[i] == "H" ) {
                this._hours = this.duration.substring(lastDigitPosition, i);
                lastDigitPosition = i+1; // next char should be the digit 
              } else if ( this.duration[i] == "M" ) {
                this._minutes = this.duration.substring(lastDigitPosition, i);
                lastDigitPosition = i+1; // next char should be the digit 
              } else if ( this.duration[i] == "S" ) {
                this._seconds = this.duration.substring(lastDigitPosition, i);
                // Last one...
                break;
              }
            }
            if ( this._years == 0 && this._months == 0 && this._days == 0 && this._hours == 0 && this._minutes == 0 && this._seconds == 0 ) {
              // all 0's. We want real time
              this._realTime = true;
            }
          }
    });
    var Time8601 = declare("dojoe.timeselector.Time8601", null, {
        time: null,
          _discreteTime: false,
          _interval: false,
          _duration: false,
          _relativePeriod: false,
          
          /**
           * Creates an instance of Time8601. kwArgs accepts the following properties:
           *
           * Required properties:
           * time - time in ISO8601. Supported format:
           *        -P0Y0M0DT24H0M  Relative period: The last 24 hours from current time
           *        +P0Y0M0DT24H0M  Relative period: The next 24 hours from current time
           *        20070301T130000Z/20080511T153000Z   Interval between March 1, 2007 and May 11, 2008
           *        20070301T130000Z Discrete time: March 1, 2007 at 1pm UTC time
           *    Future supported format:
           *        20070301T130000Z/P0Y0M0DT24H0M  Relative period: The 24 hours starting from March 1, 2007
           *        P0Y0M0DT24H0M/20070301T130000Z  Relative period: The 24 hours ending on March 1, 2007   
           */
          constructor: function(kwArgs) {
            lang.mixin(this, kwArgs);
            this._parseTime();
          },
          
          isInterval: function() {
            return this._interval;
          },
          
          isDuration: function() {
            return this._duration;
          },
          
          isDiscreteTime: function() {
            return this._discreteTime;
          },
          
          /**
           * Returns a boolean whether the time contains a start or end time and a duration.
           * If true, call the get start time and end time functions to check which one is 
           * available and call get duration function to get the duration
           */
          isRelativePeriod: function() {
            return this._relativePeriod;
          },
          
          /**
           * Returns the Date object of the discrete time.
           * Null if not available
           */
          getDiscreteDate: function() {
            return stamp.fromISOString(this._discreteTime);
          },
          
          /**
           * Returns the ISO8601 representation of the discrete time.
           * Null if not available
           */
          getDiscreteISOString: function() {
            return this._discreteTime;
          },
          
          /**
           * Returns the Date object of the start time.
           * Null if not available
           */
          getStartDateTime: function() {
            return this._startTimeDate;
          },
          
          /**
           * Returns the Date object of the end time.
           * Null if not available
           */
          getEndDateTime: function() {
            return this._endTimeDate;
          },
          
          /**
           * Returns the ISO8601 representation of the start time.
           * Null if not available
           */
          getStartTimeISOString: function() {
            return this._startTime;
          },
          
          /**
           * Returns the ISO8601 representation of the end time.
           * Null if not available
           */
          getEndTimeISOString: function() {
            return this._endTime;
          },
          
          /**
           * Returns dojoe.timeselector.DurationTime8601 object
           */
          getDuration: function() {
            return this._durationTime;
          },
          
          _parseTime: function() {
            if ( this.time ) {
              if ( this.time.indexOf("/") != -1 ) {
                this._interval = true;
                this._parseInterval();
              } else if ( this._isDuration(this.time) ) {
                this._duration = true;
                this._durationTime = new DurationTime8601({duration: this.time});
              } else {
                this._discreteTime = true;
              }
            }
          },
          
          /**
           * Checks if the string is a representation of duration in ISO8601
           */
          _isDuration: function(str) {
            var duration = false;
            var patt1=/[+|-]P\d+Y\d+M\d+DT\d+H\d+M/gi;
			//change from Karen Gdaniec / Michael Miller
            if ( str.match(patt1) ) {
              duration = true;
            }
            return duration;
          },
          
          _parseInterval: function() {
            var range = this.time.split("/");
            this._startTime = range[0];
            this._startTimeDate = this._convertToDateTimeObject(this._startTime);
            this._endTime = range[1];
            this._endTimeDate = this._convertToDateTimeObject(this._endTime);
            // check if the startTime or the endTime is a duration
            if ( this._isDuration(this._startTime) ) {
              this._durationTime = new DurationTime8601({duration: this._startTime});
            } else if ( this._isDuration(this._endTime) ) {
              this._durationTime = new DurationTime8601({duration: this._endTime});
            }
          },
          /**
           * Convert a ISO8601 time to Date object
           * @return a Date object
           */
          _convertToDateTimeObject: function(iso8601Time) {
			//changes from Karen Gdaniec / Michael Miller
            var converted = null;
            var src, work;
            src = lang.trim(iso8601Time);
            if (src && src.length >= 15) {
                // We expect time string in form of YYYYMMDDTHHmmssZ. (ex 20130606T100000Z)
                // Convert to YYYY-MM-DDTHH:mm:ssZ for standard ISO8601 parsing
                work = src.substr(0,4)+ "-" + src.substr(4,2) + "-" + src.substr(6,5) + 
                  ":" + src.substr(11,2) + ":" + src.substr(13);
                //console.log("Time8601._convertToDateTimeObject work:" + work);
                converted = stamp.fromISOString(work);
            } else {
                // old way (should not be used)
                src = src.replace(/T/g, " ");            
                src = src.replace(/Z/g, "");
                converted = dojo.date.locale.parse(src, {datePattern: "yyyyMMdd", timePattern: "HHmmss", fullYear: true});                
            }            
            //console.log("Time8601._convertToDateTimeObject iso8601Time:" + src + " converted:" + converted);
            return converted;
          }
          
        });

    return Time8601;
});
