define("dojox/grid/enhanced/plugins/QuickFilter", [
	'dojo/_base/declare',
	'dojo/_base/array',
	'dojo/_base/html',
	'dojo/_base/lang',
	'dojo/_base/event',
	'dojo/cache',
	'dojo/keys',
	'dojo/i18n',
	'dijit/_Widget',
	'dijit/_TemplatedMixin',
	'dijit/_WidgetsInTemplateMixin',
	'../_Plugin',
	'../../EnhancedGrid',
	'./filter/_FilterExpr',
	'./filter/FilterLayer',
	'dijit/form/TextBox',
	'dijit/form/Button',
	'dijit/form/ComboButton',
	'dijit/Menu',
	'dijit/MenuItem',
	'./ToolBar',
	"dojo/i18n!../nls/QuickFilter"
], function(declare, array, html, lang, evt, cache, keys, i18n, _Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _Plugin, EnhancedGrid){

var ns = lang.getObject("dojox.grid.enhanced.plugins"),
	fns = lang.getObject("dojox.grid.enhanced.plugins.filter");

var _QuickFilterPane = declare('dojox.grid.enhanced.plugins._QuickFilterPane', 
	[_Widget, _TemplatedMixin, _WidgetsInTemplateMixin], {

	templateString: cache("dojox.grid","enhanced/templates/QuickFilter.html"),

	widgetsInTemplate: true,

	grid: null,

	plugin: null,

	filterTimeout: 700,

	filterOnInput: true,

	postMixInProperties: function(){
		var nls = this.nls = i18n.getLocalization("dojox.grid.enhanced", "QuickFilter");
		this.placeHolder = nls.placeHolder;
		this.filterMenuLabel = nls.filterMenuLabel;
		this.advFilterMenuLabel = nls.advancedFilterMenuLabel;
	},

	postCreate: function(){
		this.filterBox.domNode.appendChild(this.clearBtn);
		this.filterBox.focusNode.setAttribute('aria-label', this.nls.waiFilterBox);
		this.singleBtn.focusNode.setAttribute('aria-label', this.nls.waiMainButtonLabel);
		this.comboBtn.buttonNode.setAttribute('aria-label', this.nls.waiMainButtonLabel);
		var g = this.grid;
		if(g.plugins.filter){
			html.addClass(this.domNode, 'dojoxGridQuickFilterAdvanced');
		}else{
			var filterLayer = this.isServerSide ? new fns.ServerSideFilterLayer(this) :
				new fns.ClientSideFilterLayer({
					cacheSize: this.filterCacheSize,
					fetchAll: this.fetchAllOnFirstFilter,
					getter: function(datarow, cell, rowIndex){
						return cell.get(rowIndex, datarow);
					}
				});
			ns.wrap(g, "_storeLayerFetch", filterLayer);
			this.connect(g, "_onDelete", lang.hitch(filterLayer, "invalidate"));

			this.connect(g.layer('filter'), "onFilterDefined", function(filter){
				if(!filter){
					this.filterBox.set('value', '');
					html.removeClass(this.domNode, 'dojoxGridQuickFilterCanClear');
				}else if(filter._name === 'any' && filter._operands.length && array.every(filter._operands, function(op){
					return op._name === 'contains' && op._operands[0]._colArg;
				})){
					var str = filter._operands[0]._operands[1]._value;
					if(array.every(filter._operands, function(op){
						return op._operands[1]._value === str;
					})){
						this.filterBox.set('value', str);
						html.addClass(this.domNode, 'dojoxGridQuickFilterCanClear');
					}
				}
			});
		}
		g.focus.addArea({
			name: 'quickFilter',
			onFocus: lang.hitch(this, this._onFocusField),
			onBlur: lang.hitch(this, this._onBlurField)
		});
		g.focus.addArea({
			name: 'quickFilterBtn',
			onFocus: lang.hitch(this, this._focusBtn)
		});
		this.plugin.onStartUp = lang.hitch(this, function(){
			g.focus.placeArea('quickFilter', 'after', 'toolbar');
			g.focus.placeArea('quickFilterBtn', 'after', 'quickFilter');
			if(g.plugins.filter){
				this.connect(g, "onFilterDefined", function(filter){
					if(filter.length === 1 && filter[0].column === 'anycolumn' && filter[0].condition === 'contains'){
						this.filterBox.set('value', filter[0].value);
						html.addClass(this.domNode, 'dojoxGridQuickFilterCanClear');
					}else if(!filter.length){
						this.filterBox.set('value', '');
						html.removeClass(this.domNode, 'dojoxGridQuickFilterCanClear');
					}
				});
			}
		});
	},

	_filter: function(){
		var str = this.filterBox.get('value'), g = this.grid;
		if(g.plugins.filter){
			g.setFilter(str && {
				"type": "string",
				"column": "anycolumn",
				"condition": "contains",
				"value": str
			});
		}else{
			var expr = null;
			if(str){
				var cells = g.layout.cells, i, exprs = [];
				for(i = cells.length - 1; i >= 0; --i){
					col = cells[i];
					if(col.filterable || col.filterable === undefined){
						exprs.push(new fns.Contains(
							new fns.StringExpr(this.isServerSide ? col.name : col, true), 
							new fns.StringExpr(str)
						));
					}
				}
				expr = new fns.LogicANY(exprs);
			}
			g.layer('filter').filterDef(expr);
			this._updateGrid();
		}
	},

	_onInput: function(e){
		var _this = this;
		setTimeout(function(){
			html.toggleClass(_this.domNode, 'dojoxGridQuickFilterCanClear', _this.filterBox.get('value'));
		}, 0);
		if(this.filterOnInput && e.keyCode !== keys.TAB){
			if(this._filterTimeoutHandler){
				clearTimeout(this._filterTimeoutHandler);
			}
			this._filterTimeoutHandler = setTimeout(function(){
				_this._filterTimeoutHandler = null;
				_this._filter();
			}, this.filterTimeout);
		}
	},

	_onClearBtnClick: function(){
		html.removeClass(this.domNode, 'dojoxGridQuickFilterCanClear');
		this.grid.layer('filter').filterDef(null);
		this._updateGrid();
	},

	_showAdvancedFilter: function(){
		this.grid.plugin('filter').filterDefDialog.showDialog();
	},

	_updateGrid: function(){
		var g = this.grid;
		g.showMessage(g.loadingMessage);
		setTimeout(lang.hitch(g, g._refresh), 200);
	},

	_onFocusField: function(e){
		this.grid.focus.currentArea('quickFilter');
		if(e && e.stopPropagation){
			evt.stop(e);
		}
		html.setSelectable(this.grid.domNode, true);
		this.filterBox.focus();
		return true;
	},

	_onBlurField: function(e){
		html.setSelectable(this.grid.domNode, false);
		return true;
	},

	_onFocusBtn: function(){
		this.grid.focus.currentArea('quickFilterBtn');
	},

	_focusBtn: function(e){
		if(e && e.stopPropagation){
			evt.stop(e);
		}
		this[this.grid.plugins.filter ? 'comboBtn' : 'singleBtn'].focus();
		return true;
	},

	_onKey: function(e){
		if(!this.filterOnInput && e.keyCode === keys.ENTER){
			this._filter();
		}
	}
});

var QuickFilter = declare('dojox.grid.enhanced.plugins.QuickFilter', _Plugin, {

	name: 'quickFilter',

	constructor: function(grid, args){
		var p = this.quickFilter = new _QuickFilterPane(lang.mixin({
			grid: grid,
			plugin: this
		}, args));
		p.placeAt(grid.plugin('toolbar').toolbar.domNode);
		p.startup();
	},

	destroy: function(){
		this.quickFilter.destroyRecursive();
	}
});

EnhancedGrid.registerPlugin(QuickFilter, {
	"dependency": ["toolbar"]
});

return QuickFilter;
});

