/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("ibm/tivoli/tip/navwidget/groupwidget/dijit/AnalogGaugeGroupView",
	[
		"dojo/_base/declare",
		"ibm/tivoli/navmodel/Dataset",
		"dojoe/gauge/analoggauge/AnalogGauge",
		"ibm/tivoli/tip/navwidget/groupwidget/dijit/GroupWidgetView",
		"dojoe/Util",
		"dojox/widget/Standby"
	],
	function(declare, Dataset, AnalogGauge, GroupWidgetView, Util, Standby) {
		return declare("ibm.tivoli.tip.navwidget.groupwidget.dijit.AnalogGaugeGroupView", [AnalogGauge], {
		
		analogGauge:null,
		id:null,
		node:null,
		config:null,
		widgetParams:null,
		columnArray:null,
		
		postMixInProperties: function() {
			console.log("postMixInProperties : AnalogGaugeGroupView");
			//this.inherited("postMixInProperties", arguments);
		},
		
		startup:function() {
			console.info("AnalogGaugeGroupView-startup");
			var w=this.node.clientWidth;
			var h=this.node.clientHeight;
			this.widgetParams["value"] = 5;
			this.widgetParams["width"] = w;
			this.widgetParams["height"] = h;
			this.widgetParams["majorTicksColor"] = "white";
			this.widgetParams["_currentValue"] = 5;
			this.widgetParams["majorTicksColor"] = "black";
			if(this.config.topLabel != undefined)
				this.widgetParams["topLabelPresent"] = true;
			if(this.config.leftLabel != undefined)
				this.widgetParams["leftLabelPresent"] = true;
				
			this.analogGauge = new dojoe.gauge.analoggauge.AnalogGauge(this.widgetParams, this.node);		
			this.analogGauge.setValue(5);
			switch(this.alignProp) {
				case "right":
							this.analogGauge.domNode.style.cssFloat="right";
							break;
				case "center":
							this.analogGauge.domNode.style.marginLeft="auto";
							this.analogGauge.domNode.style.marginRight="auto";
							break;
				case "left":
							this.analogGauge.domNode.style.cssFloat="left";
							break;
				default:
							this.analogGauge.domNode.style.marginLeft="auto";
							this.analogGauge.domNode.style.marginRight="auto";
			}
			if(!this.columnArray)
				this.columnArray = new Array();
			this.columnArray.push(this.config.value);
			if(this.config.topLabelCustom != "true")
				this.columnArray.push(this.config.topLabel);
			if(this.config.leftLabelCustom != "true")
				this.columnArray.push(this.config.leftLabel);
			if(this.config.maxValue)
				this.columnArray.push(this.config.maxValue);
			if(this.config.minValue)
				this.columnArray.push(this.config.minValue);
			this.analogGauge.startup();
		},
		
		setConfig: function(json) {
			console.debug("GW-AG-JSON-", json);
			this.config = {};
			var thresholdArray = new Array();
			this.widgetParams={};
			if(json) {
				for(var i=0; i<json.length; i++) {
					//if(json[i].name === "leftLabel") this.config.leftLabel = json[i].value;
					var name = json[i].name;
					var value = json[i].value;
					if(value=="") {
						continue;
					}
					if(name === "TopLabel") this.config.topLabel = value;
					else if(name === "TopLabelCustom") this.config.topLabelCustom = value;
					else if(name === "LeftLabel") this.config.leftLabel = value;
					else if(name === "LeftLabelCustom") this.config.leftLabelCustom = value;
					else if(name === "Value") this.config.value = value;
					else if(name === "Normal") thresholdArray.push({"start":value,"status":"Normal","name":"Normal"});
					else if(name === "Minor") thresholdArray.push({"start":value,"status":"Minor","name":"Minor"});
					else if(name === "Major") thresholdArray.push({"start":value,"status":"Major","name":"Major"});
					else if(name === "Informational") thresholdArray.push({"start":value,"status":"Informational","name":"Informational"});
					else if(name === "Critical") thresholdArray.push({"start":value,"status":"Critical","name":"Critical"});
					else if(name === "Fatal") thresholdArray.push({"start":value,"status":"Fatal","name":"Fatal"});
					else if(name === "Unknown") thresholdArray.push({"start":value,"status":"Unknown","name":"Unknown"});
					else if(name === "MinValue") {
						if(isNaN(value))
							this.config.minValue = value;
						else
							this.widgetParams["minValue"] = Number(value);
					} else if(name === "MaxValue") {
						if(isNaN(value))
							this.config.maxValue = value;
						else
							this.widgetParams["maxValue"] = Number(value);
					}
					else if(name === "ShowThresholdStrip") {
						this.widgetParams["showThresholdStrip"] = value;
					} else if(name === "MajorTicksSeparation") {
						this.widgetParams["majorTickSeparation"] = parseInt(value, 10);
					} else if(name === "MinorTicksSeparation") {
						this.widgetParams["minorTickSeparation"] = parseInt(value, 10);
					} else if(name == "align") {
						this.alignProp = value;
					} else if(name === "ColorPicker") {
						this.widgetParams["fontColor"] = value;
					} else if(name === "FontSize") {
						this.widgetParams["fontSize"] = value;
					} else if(name === "FontFamily") {
						this.widgetParams["fontFamily"] = value;
					} else if(name === "Unit") {
						this.widgetParams["units"] = value;
					}
				}
				this.widgetParams["thresholds"]=thresholdArray;
			}
			if(this.config.topLabelCustom == "true") {
				this.widgetParams["topLabel"] = this.config.topLabel;
			}
			if(this.config.leftLabelCustom == "true") {
				this.widgetParams["leftLabel"] = this.config.leftLabel;
			}
		},
		
		setItem: function(items) {
			console.log("GW-VB-setItem");
			var topLabel = null;
			var leftLabel = null;
			var reading = null;
			var minValue = this.analogGauge.minValue;
			var maxValue = this.analogGauge.maxValue;
			if (typeof items != "undefined" && items != null)
			{
				var item = null;
				for (var index=0; index<items.length; index++) {
					item = items[index];
					if(item.getProperty(this.config.value) && typeof(item.getProperty(this.config.value).getValue) != "undefined"){
						if(item.getProperty(this.config.value).getValueType != "undefined" && item.getProperty(this.config.value).getValueType() == "status") {
							reading = item.getProperty(this.config.value).getValue();
						}else{
							reading = item.getProperty(this.config.value).getValue();
							var valProp = item.getProperty(this.config.value);
							if(typeof(valProp.getValueMax) != "undefined" && valProp.getValueMax() && valProp.getValueMax() != 0
								&& valProp.getValueType != "undefined" && valProp.getValueType() == "percentage") {
								reading = (reading * 100)/valProp.getValueMax();
							}
						}
					}
					if(this.config.topLabel != "None" && item.getProperty(this.config.topLabel) && typeof(item.getProperty(this.config.topLabel).getValue) != "undefined" ) {
						var allTopLabelProp;
						if(item.getProperty(this.config.topLabel).getValueType != "undefined" && item.getProperty(this.config.topLabel).getValueType() == "status") {
							topLabel = escapeHTML(item.getProperty(this.config.topLabel).getValue().getValue());
							allTopLabelProp = item.getProperty(this.config.topLabel).getValue();
						}else{
							topLabel = escapeHTML(item.getProperty(this.config.topLabel).getValue());
							allTopLabelProp = item.getProperty(this.config.topLabel);
						}
						if(allTopLabelProp && typeof(allTopLabelProp.getValueMax) != "undefined" && allTopLabelProp.getValueMax() && allTopLabelProp.getValueMax() != 0
							&& allTopLabelProp.getValueType != "undefined" && allTopLabelProp.getValueType() == "percentage") {
							topLabel = (topLabel * 100)/allTopLabelProp.getValueMax();
						}
					}
					if(this.config.leftLabel != "None" && item.getProperty(this.config.leftLabel) && typeof(item.getProperty(this.config.leftLabel).getValue) != "undefined" ){
						var allLeftLabelProp;
						if(item.getProperty(this.config.leftLabel).getValueType != "undefined" && item.getProperty(this.config.leftLabel).getValueType() == "status") {
							leftLabel = escapeHTML(item.getProperty(this.config.leftLabel).getValue().getValue());		
							allLeftLabelProp = item.getProperty(this.config.leftLabel).getValue();
						}else{
							leftLabel = escapeHTML(item.getProperty(this.config.leftLabel).getValue());	
							allLeftLabelProp = item.getProperty(this.config.leftLabel);
						}
						if(allLeftLabelProp && typeof(allLeftLabelProp.getValueMax) != "undefined" && allLeftLabelProp.getValueMax() && allLeftLabelProp.getValueMax() != 0
							&& allLeftLabelProp.getValueType != "undefined" && allLeftLabelProp.getValueType() == "percentage") {
							leftLabel = (leftLabel * 100)/allLeftLabelProp.getValueMax();
						}		
					}
					if(this.config.minValue != "None" && item.getProperty(this.config.minValue) && typeof(item.getProperty(this.config.minValue).getValue) != "undefined" ){					
						var minValueProp;
						if(item.getProperty(this.config.minValue).getValueType != "undefined" && item.getProperty(this.config.minValue).getValueType() == "status") {
							minValue = item.getProperty(this.config.minValue).getValue().getValue();
							minValueProp = item.getProperty(this.config.minValue).getValue();
						}else{
							minValue = item.getProperty(this.config.minValue).getValue();
							minValueProp = item.getProperty(this.config.minValue);
						}
						if(minValueProp && typeof(minValueProp.getValueMax) != "undefined" && minValueProp.getValueMax() && minValueProp.getValueMax() != 0
							&& minValueProp.getValueType != "undefined" && minValueProp.getValueType() == "percentage") {
							minValue = (minValue * 100)/minValueProp.getValueMax();
						}
					}
					if(this.config.maxValue != "None" && item.getProperty(this.config.maxValue) && typeof(item.getProperty(this.config.maxValue).getValue) != "undefined" ){
						var maxValueProp;
						if(item.getProperty(this.config.maxValue).getValueType != "undefined" && item.getProperty(this.config.maxValue).getValueType() == "status") {
							maxValue = item.getProperty(this.config.maxValue).getValue().getValue();
							maxValueProp = item.getProperty(this.config.maxValue).getValue();
						}else{
							maxValue = item.getProperty(this.config.maxValue).getValue();
							maxValueProp = item.getProperty(this.config.maxValue);
						}
						if(maxValueProp && typeof(maxValueProp.getValueMax) != "undefined" && maxValueProp.getValueMax() && maxValueProp.getValueMax() != 0
							&& maxValueProp.getValueType != "undefined" && maxValueProp.getValueType() == "percentage") {
							maxValue = (maxValue * 100)/maxValueProp.getValueMax();
						}
					}
				}
				if(item != null) {
					if(typeof(this.analogGauge.updateTicks) == "function")
						this.analogGauge.updateTicks(minValue, maxValue);
					this.analogGauge.setLabelAndValue(topLabel, leftLabel, reading);
					this.analogGauge.itemId = item.getId();				
					if(this.analogGauge.queryForOslcResourceURI != "false" && item.getProperty("OslcResourceURI"))
						this.analogGauge.oslcResourceURI = item.getProperty("OslcResourceURI").getValue();
				} else {
					// if item is null, set the state configured in QE for this. Default is unknown
					this.analogGauge.setValue(this.analogGauge.defaultStatus);
				}
				item = null;
				items = null;
				topLabel = null;
				leftLabel = null;
				reading = null;
				minValue = null;
				maxValue = null;
			}		
		},
		
		itemPropertiesChanged: function(event) {
			console.log("GW-VB-itemPropertiesChanged");					
			if (event.getProperties()) {
				var length = event.getItems().length;	
				for (var index = 0;  index < length;  ++index) {	
					this.analogGauge.itemId = event.getItems()[index].getId();	
					if ((event.getProperties())[index]) {
						var value = this.analogGauge.value;
						var topLabel = null;
						var leftLabel = null;
						var minValue = this.analogGauge.minValue;
						var maxValue = this.analogGauge.maxValue;
						
						var propsLength = (event.getProperties())[index].length;			
						for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
							if (((event.getProperties())[index])[propIndex].getId() === this.config.value &&
									((event.getProperties())[index])[propIndex].getValue) {
								if(((event.getProperties())[index])[propIndex].getValueType != "undefined" && ((event.getProperties())[index])[propIndex].getValueType() == "status") {
									value = ((event.getProperties())[index])[propIndex].getValue();
								}else{
									value = ((event.getProperties())[index])[propIndex].getValue();
									if(typeof(value.getValueMax) != "undefined" && value.getValueMax() && value.getValueMax() != 0
										&& value.getValueType != "undefined" && value.getValueType() == "percentage") {
										value = (value * 100)/value.getValueMax();
									}
								}
							} 
							if (((event.getProperties())[index])[propIndex].getId()  === this.config.topLabel &&
									((event.getProperties())[index])[propIndex].getValue) {
								var allTopLabelProp;
								if(((event.getProperties())[index])[propIndex].getValueType != "undefined" && ((event.getProperties())[index])[propIndex].getValueType() == "status") {
									topLabel = escapeHTML(((event.getProperties())[index])[propIndex].getValue().getValue());
									allTopLabelProp = ((event.getProperties())[index])[propIndex].getValue();
								}else{
									topLabel = escapeHTML(((event.getProperties())[index])[propIndex].getValue());
									allTopLabelProp = ((event.getProperties())[index])[propIndex];
								}
								if(allTopLabelProp && typeof(allTopLabelProp.getValueMax) != "undefined" && allTopLabelProp.getValueMax() && allTopLabelProp.getValueMax() != 0
									&& allTopLabelProp.getValueType != "undefined" && allTopLabelProp.getValueType() == "percentage") {
									topLabel = (topLabel * 100)/allTopLabelProp.getValueMax();
								}
							} 
							if (((event.getProperties())[index])[propIndex].getId()  === this.config.leftLabel &&
									((event.getProperties())[index])[propIndex].getValue) {
								var allLeftLabelProp;
								if(((event.getProperties())[index])[propIndex].getValueType != "undefined" && ((event.getProperties())[index])[propIndex].getValueType() == "status") {
									leftLabel = escapeHTML(((event.getProperties())[index])[propIndex].getValue().getValue());
									allLeftLabelProp = ((event.getProperties())[index])[propIndex].getValue();
								}else{
									leftLabel = escapeHTML(((event.getProperties())[index])[propIndex].getValue());
									allLeftLabelProp = ((event.getProperties())[index])[propIndex];
								}
								if(allLeftLabelProp && typeof(allLeftLabelProp.getValueMax) != "undefined" && allLeftLabelProp.getValueMax() && allLeftLabelProp.getValueMax() != 0
									&& allLeftLabelProp.getValueType != "undefined" && allLeftLabelProp.getValueType() == "percentage") {
									leftLabel = (leftLabel * 100)/allLeftLabelProp.getValueMax();
								}	
							}
							if(((event.getProperties())[index])[propIndex].getId()  === "OslcResourcURI"  &&
									((event.getProperties())[index])[propIndex].getValue) {
								this._gauge.oslcResourceURI = escapeHTML(((event.getProperties())[index])[propIndex].getValue());
							}
							if (((event.getProperties())[index])[propIndex].getId() === this.config.minValue &&
									((event.getProperties())[index])[propIndex].getValue) {
								if(((event.getProperties())[index])[propIndex].getValueType != "undefined" && ((event.getProperties())[index])[propIndex].getValueType() == "status") {
									minValue = ((event.getProperties())[index])[propIndex].getValue().getValue();
								} else{
									minValue = ((event.getProperties())[index])[propIndex].getValue();
									var minValueProp = ((event.getProperties())[index])[propIndex];
									if(typeof(minValueProp.getValueMax) != "undefined" && minValueProp.getValueMax() && minValueProp.getValueMax() != 0
										&& minValueProp.getValueType != "undefined" && minValueProp.getValueType() == "percentage") {
										minValue = (minValue * 100)/minValueProp.getValueMax();
									}
								}
							}
							if (((event.getProperties())[index])[propIndex].getId() === this.config.maxValue &&
									((event.getProperties())[index])[propIndex].getValue) {
								if(((event.getProperties())[index])[propIndex].getValueType != "undefined" && ((event.getProperties())[index])[propIndex].getValueType() == "status") {
									maxValue = ((event.getProperties())[index])[propIndex].getValue().getValue();
								} else{
									maxValue = ((event.getProperties())[index])[propIndex].getValue();
									var maxValueProp = ((event.getProperties())[index])[propIndex];
									if(typeof(maxValueProp.getValueMax) != "undefined" && maxValueProp.getValueMax() && maxValueProp.getValueMax() != 0
										&& maxValueProp.getValueType != "undefined" && maxValueProp.getValueType() == "percentage") {
										maxValue = (maxValue * 100)/maxValueProp.getValueMax();
									}
								}
							}
						}
						if(typeof(this.analogGauge.updateTicks) == "function")
							this.analogGauge.updateTicks(minValue, maxValue);
						this.analogGauge.setLabelAndValue(topLabel, leftLabel, value);	
						//set all the references to null to ensure browser deletes the items and properties
						value = null;
						topLabel = null;
						leftLabel = null;
						minValue = null;
						maxValue = null;
					}	
							
				}			
			}
			event = null;
		},
		
				
		setWidgetParams: function(widParams) {
			//this.widgetParams = widParams;
		},
		
		getAllColumns: function() {
			return this.columnArray;
		},
		
		resize: function(w, h) {
			var dim = w < h ? w : h;
			if(this.analogGauge.domNode) {
				this.analogGauge.domNode.style.width=w+"px";
				this.analogGauge.domNode.style.height=h+"px";
			}
			var size = new Object();
			size.available = {w: w, h:h};
			this.analogGauge.resize(size);	
		}
});
});
