/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("ibm/tivoli/tip/navwidget/groupwidget/dijit/StatusGaugeGroupView",
	[
		"dojo/_base/declare",
		"ibm/tivoli/navmodel/Dataset",
		"dojoe/gauge/statusgauge/StatusGauge",
		"ibm/tivoli/tip/navwidget/groupwidget/dijit/GroupWidgetView",
		"dojoe/Util",
		"dojox/widget/Standby"
	],
	function(declare, Dataset, StatusGauge, GroupWidgetView, Util, Standby) {
		return declare("ibm.tivoli.tip.navwidget.groupwidget.dijit.StatusGaugeGroupView", [StatusGauge], {
		
		statusGauge:null,
		id:null,
		node:null,
		config:null,
		widgetParams:null,
		columnArray:null,
		
		postMixInProperties: function() {
			console.log("postMixInProperties-StatusGaugeGroupView");
			//this.inherited("postMixInProperties", arguments);
		},
		
		startup:function() {
			console.info("StatusGaugeGroupView-startup");
			var w=this.node.clientWidth;
			var h=this.node.clientHeight;
			this.widgetParams["value"] = 5;
			this.widgetParams["width"] = w;
			this.widgetParams["height"] = h;
			if(this.config.topLabel != undefined)
				this.widgetParams["topLabelPresent"] = true;
			if(this.config.leftLabel != undefined)
				this.widgetParams["leftLabelPresent"] = true;
			this.statusGauge = new dojoe.gauge.statusgauge.StatusGauge(this.widgetParams,this.node);
			this.statusGauge.value=5;
			switch(this.alignProp) {
				case "right":
							this.statusGauge.domNode.style.cssFloat="right";
							break;
				case "center":
							this.statusGauge.domNode.style.marginLeft="auto";
							this.statusGauge.domNode.style.marginRight="auto";
							break;
				case "left":
							this.statusGauge.domNode.style.cssFloat="left";
							break;
				default:
							this.statusGauge.domNode.style.marginLeft="auto";
							this.statusGauge.domNode.style.marginRight="auto";
			}
			
			if(!this.columnArray)
				this.columnArray = new Array();
			this.columnArray.push(this.config.value);
			if(this.config.topLabelCustom != "true")
				this.columnArray.push(this.config.topLabel);
			if(this.config.leftLabelCustom != "true")
				this.columnArray.push(this.config.leftLabel);
				
			this.statusGauge.startup();
		},
		
		setConfig: function(json) {
			console.info("StatusGaugeGroupView-setConfig-json-", json);
			this.config = {};
			var thresholdArray = new Array();
			var topLabel, leftLabel;
			if(this.widgetParams == null)
					this.widgetParams={};
					
			if(json) {
				for(var i=0; i<json.length; i++) {
					if(json[i].value=="") {
						continue;
					}
					if(json[i].name === "TopLabel") this.config.topLabel = json[i].value;
					else if(json[i].name === "TopLabelCustom") this.config.topLabelCustom = json[i].value;
					else if(json[i].name === "LeftLabel") this.config.leftLabel = json[i].value;
					else if(json[i].name === "LeftLabelCustom") this.config.leftLabelCustom = json[i].value;
					else if(json[i].name == "Value") this.config.value = json[i].value;
					else if(json[i].name === "Normal") thresholdArray.push({"start":json[i].value,"status":"normal","name":"normal"});
					else if(json[i].name === "Minor") thresholdArray.push({"start":json[i].value,"status":"minor","name":"minor"});
					else if(json[i].name === "Major") thresholdArray.push({"start":json[i].value,"status":"major","name":"major"});
					else if(json[i].name === "Informational") thresholdArray.push({"start":json[i].value,"status":"informational","name":"informational"});
					else if(json[i].name === "Critical") thresholdArray.push({"start":json[i].value,"status":"critical","name":"critical"});
					else if(json[i].name === "Fatal") thresholdArray.push({"start":json[i].value,"status":"fatal","name":"fatal"});
					else if(json[i].name === "Unknown") thresholdArray.push({"start":json[i].value,"status":"unknown","name":"unknown"});
					else if(json[i].name === "align") this.alignProp = json[i].value;
					else if(json[i].name === "DefaultStatus") this.widgetParams.defaultStatus = json[i].value;
					else if(json[i].name === "ColorPicker") this.widgetParams["fontColor"] = json[i].value;
					else if(json[i].name === "FontSize") this.widgetParams["fontSize"] = json[i].value;
					else if(json[i].name === "FontFamily") this.widgetParams["fontFamily"] = json[i].value;
				}
				this.widgetParams["thresholds"]=thresholdArray;
			}
			if(this.config.topLabelCustom == "true") {
				this.widgetParams["topLabel"] = this.config.topLabel;
			}
			if(this.config.leftLabelCustom == "true") {
				this.widgetParams["leftLabel"] = this.config.leftLabel;
			}
		},
		
		setItem: function(items) {
			console.log("StatusGaugeGroupView-setItem");
			var topLabel = null;
			var leftLabel = null;
			var reading = null;
			if (typeof items != "undefined" && items != null)
			{
				var item = null;
				for (var index=0; index<items.length; index++) {
					item = items[index];
					if(item.getProperty(this.config.value) && typeof(item.getProperty(this.config.value).getValue) != "undefined"){
						if(item.getProperty(this.config.value).getValueType != "undefined" && item.getProperty(this.config.value).getValueType() == "status") {
							reading = item.getProperty(this.config.value).getValue();
						}else{
							reading = item.getProperty(this.config.value).getValue();
							var valProp = item.getProperty(this.config.value);
							if(typeof(valProp.getValueMax) != "undefined" && valProp.getValueMax() && valProp.getValueMax() != 0
								&& valProp.getValueType != "undefined" && valProp.getValueType() == "percentage") {
								reading = (reading * 100)/valProp.getValueMax();
							}
						}
					}
					if(this.config.topLabel != "None" && item.getProperty(this.config.topLabel) && typeof(item.getProperty(this.config.topLabel).getValue) != "undefined" ) {
						var allTopLabelProp;
						if(item.getProperty(this.config.topLabel).getValueType != "undefined" && item.getProperty(this.config.topLabel).getValueType() == "status") {
							topLabel = escapeHTML(item.getProperty(this.config.topLabel).getValue().getValue());
							allTopLabelProp = item.getProperty(this.config.topLabel).getValue();
						}else{
							topLabel = escapeHTML(item.getProperty(this.config.topLabel).getValue());
							allTopLabelProp = item.getProperty(this.config.topLabel);
						}
						if(allTopLabelProp && typeof(allTopLabelProp.getValueMax) != "undefined" && allTopLabelProp.getValueMax() && allTopLabelProp.getValueMax() != 0
							&& allTopLabelProp.getValueType != "undefined" && allTopLabelProp.getValueType() == "percentage") {
							topLabel = (topLabel * 100)/allTopLabelProp.getValueMax();
						}
					}
					if(this.config.leftLabel != "None" && item.getProperty(this.config.leftLabel) && typeof(item.getProperty(this.config.leftLabel).getValue) != "undefined" ){
						var allLeftLabelProp;
						if(item.getProperty(this.config.leftLabel).getValueType != "undefined" && item.getProperty(this.config.leftLabel).getValueType() == "status") {
							leftLabel = escapeHTML(item.getProperty(this.config.leftLabel).getValue().getValue());		
							allLeftLabelProp = item.getProperty(this.config.leftLabel).getValue();
						}else{
							leftLabel = escapeHTML(item.getProperty(this.config.leftLabel).getValue());	
							allLeftLabelProp = item.getProperty(this.config.leftLabel);
						}
						if(allLeftLabelProp && typeof(allLeftLabelProp.getValueMax) != "undefined" && allLeftLabelProp.getValueMax() && allLeftLabelProp.getValueMax() != 0
							&& allLeftLabelProp.getValueType != "undefined" && allLeftLabelProp.getValueType() == "percentage") {
							leftLabel = (leftLabel * 100)/allLeftLabelProp.getValueMax();
						}		
					}
				}
				if(item != null) {
					this.statusGauge.setLabelAndValue(topLabel, leftLabel, reading);
					this.statusGauge.itemId = item.getId();				
					if(this.statusGauge.queryForOslcResourceURI != "false" && item.getProperty("OslcResourceURI"))
						this.statusGauge.oslcResourceURI = item.getProperty("OslcResourceURI").getValue();
				} else {
					// if item is null, set the state configured in QE for this. Default is unknown
					this.statusGauge.setValue(this.statusGauge.defaultStatus);
				}
				item = null;
				items = null;
				topLabel = null;
				leftLabel = null;
				reading = null;
			}
		},
		
		itemPropertiesChanged: function(event) {
			console.debug("StatusGaugeGroupView-itemPropertiesChanged");					
			if (event.getProperties()) {
				var length = event.getItems().length;	
				for (var index = 0;  index < length;  ++index) {	
					this.statusGauge.itemId = event.getItems()[index].getId();	
					if ((event.getProperties())[index]) {
						var value = this.statusGauge.value;
						var topLabel = null;
						var leftLabel = null;
						var propsLength = (event.getProperties())[index].length;			
						for (var propIndex = 0;  propIndex < propsLength;  ++propIndex) {
							if (((event.getProperties())[index])[propIndex].getId() === this.config.value &&
									((event.getProperties())[index])[propIndex].getValue) {
								if(((event.getProperties())[index])[propIndex].getValueType != "undefined" && ((event.getProperties())[index])[propIndex].getValueType() == "status") {
									value = ((event.getProperties())[index])[propIndex].getValue();
								}else{
									value = ((event.getProperties())[index])[propIndex].getValue();
									if(typeof(value.getValueMax) != "undefined" && value.getValueMax() && value.getValueMax() != 0
										&& value.getValueType != "undefined" && value.getValueType() == "percentage") {
										value = (value * 100)/value.getValueMax();
									}
								}
							} 
							if (((event.getProperties())[index])[propIndex].getId()  === this.config.topLabel &&
									((event.getProperties())[index])[propIndex].getValue) {
								var allTopLabelProp;
								if(((event.getProperties())[index])[propIndex].getValueType != "undefined" && ((event.getProperties())[index])[propIndex].getValueType() == "status") {
									topLabel = escapeHTML(((event.getProperties())[index])[propIndex].getValue().getValue());
									allTopLabelProp = ((event.getProperties())[index])[propIndex].getValue();
								}else{
									topLabel = escapeHTML(((event.getProperties())[index])[propIndex].getValue());
									allTopLabelProp = ((event.getProperties())[index])[propIndex];
								}
								if(allTopLabelProp && typeof(allTopLabelProp.getValueMax) != "undefined" && allTopLabelProp.getValueMax() && allTopLabelProp.getValueMax() != 0
									&& allTopLabelProp.getValueType != "undefined" && allTopLabelProp.getValueType() == "percentage") {
									topLabel = (topLabel * 100)/allTopLabelProp.getValueMax();
								}
							} 
							if (((event.getProperties())[index])[propIndex].getId()  === this.config.leftLabel &&
									((event.getProperties())[index])[propIndex].getValue) {
								var allLeftLabelProp;
								if(((event.getProperties())[index])[propIndex].getValueType != "undefined" && ((event.getProperties())[index])[propIndex].getValueType() == "status") {
									leftLabel = escapeHTML(((event.getProperties())[index])[propIndex].getValue().getValue());
									allLeftLabelProp = ((event.getProperties())[index])[propIndex].getValue();
								}else{
									leftLabel = escapeHTML(((event.getProperties())[index])[propIndex].getValue());
									allLeftLabelProp = ((event.getProperties())[index])[propIndex];
								}
								if(allLeftLabelProp && typeof(allLeftLabelProp.getValueMax) != "undefined" && allLeftLabelProp.getValueMax() && allLeftLabelProp.getValueMax() != 0
									&& allLeftLabelProp.getValueType != "undefined" && allLeftLabelProp.getValueType() == "percentage") {
									leftLabel = (leftLabel * 100)/allLeftLabelProp.getValueMax();
								}	
							}
							if(((event.getProperties())[index])[propIndex].getId()  === "OslcResourcURI"  &&
									((event.getProperties())[index])[propIndex].getValue) {
								this.statusGauge.oslcResourceURI = escapeHTML(((event.getProperties())[index])[propIndex].getValue());
							}
						}
						this.statusGauge.setLabelAndValue(topLabel, leftLabel, value);	
						//set all the references to null to ensure browser deletes the items and properties
						value = null;
						topLabel = null;
						leftLabel = null;
					}	
							
				}			
			}
			event = null;
		},
		
		setWidgetParams: function(widParams) {
			//this.widgetParams = widParams;
		},
		
		getAllColumns: function() {
			return this.columnArray;
		},
		
		resize: function(w, h) {
			console.warn("Status Gauge group widget resize...." + w +" " + h);
			var dim = w < h ? w : h;
			if(this.statusGauge.domNode) {
				this.statusGauge.domNode.style.width=w+"px";
				this.statusGauge.domNode.style.height=h+"px";
				var size = new Object();
				size.available = {w: w, h:h};
				this.statusGauge.resize(size);
			}
		}
});
});
