/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("ibm/tivoli/tip/navwidget/map/dijit/MapView",
    [ 
      "dojo/_base/declare", 
      "dojo/_base/Deferred",
      "dojo/_base/lang",
      "dojo/io/script",
      "dojo/on",
      "dojo/query",      
      "dojoe/ssd/quickedit/MapMappings",
      "dojoe/ssd/quickedit/MapOptions",
      "dojoe/ssd/utils/EventHelper",
      "dojoe/map/Map",
      "dojoe/Util", 
      "dojox/widget/Standby",
      "ibm/tivoli/navmodel/Dataset", 
      "ibm/tivoli/tip/navwidget/dojo/NavWidgetPluggableViewSelection",
      "ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper",
      "ibm/tivoli/tip/navwidget/dijit/AsynchContextMenu",
      "ibm/tivoli/tip/navwidget/dijit/RESTContextMenuProvider",
      "ibm/tivoli/tip/navwidget/dijit/NavWidgetInlineMessagePluggableView" 
    ], 
    function(declare, Deferred, lang, script, on, query, Mappings, Options, EventHelper, MapWidget, Util,
            Standby, Dataset, NavWidgetPluggableViewSelection, MenuProviderHelper, AsynchContextMenu, 
            RESTContextMenuProvider, PluggableView) {
        return declare("ibm.tivoli.tip.navwidget.map.dijit.MapView", [ PluggableView ], {
        _usesWidgetDefinition : false,
        dashletConfig : null,
        datasetInfo : null,
        
        map : null, 
        nameSpace: "",
        options: null,    
        mappings: null,
        mapDivId: null,
        mapLoaded: null,
        itemsMap: null,
        cmProvider: null,
        asyncCM: null,
        selection: null,
        
        wireEventsQueue: [],
        
        statusTypes: {
            INFO: "informational",
            NORMAL: "normal",
            MINOR: "minor",
            MAJOR: "major",
            CRITICAL: "critical",
            FATAL: "fatal",
            UNDEFINED: "undefined",
            UNKNOWN: "unknown"
        },
        
        constructor: function(cmProvider) {
            if(cmProvider) {
                this.cmProvider = cmProvider;
            } else {
                var helper = new MenuProviderHelper();
                this.cmProvider = new RESTContextMenuProvider({
                    view: this,
                    getMenuItemsCallback: dojo.hitch(helper, helper.getMenuItems),
                    getMenuCommandCallback: dojo.hitch(helper, helper.getMenuCommand)
                });    
            }
            
            this.asyncCM =  new AsynchContextMenu();
            this.asyncCM.cmProvider = this.cmProvider;
            
            var kwArgs = {
                pluggableView: this
            };
            
            this.selection = NavWidgetPluggableViewSelection(kwArgs); 
            
            this.inherited(arguments);
        },

        initMap: function(mapDivId, nameSpace) {
            console.log("MapView - initMap - Start");
            
            var div = dojo.byId(mapDivId);
            
            this.mapLoaded = new Deferred();
            this.options = new dojoe.ssd.quickedit.MapOptions();
            this.mappings = new dojoe.ssd.quickedit.MapMappings();
                       
            this.nameSpace = nameSpace;
            this.mapDivId = mapDivId;
            this.asyncCM.cmProvider.iscPortletNamespace = this.nameSpace;
            
            if(this.nameSpace) {
				//Bobby, 02Jan2014, adding launchPage support
				if(this.launchPage) this.eventHelper = new EventHelper(this.nameSpace, this.launchPage);
                else this.eventHelper = new EventHelper(this.nameSpace);
            }
            
            this._initMap();
        },
        
        processCachedEvents: function() {
            for(var i = 0; i < this.wireEventsQueue.length; i++) {
                this.eventHelper.processEvent(evt, this.datasetInfo, this.dashletConfig, this.dataset);
                this.map.destroyAllMarkers();
            }
            
            this.wireEventsQueue = [];
        },
        
        refresh: function(){
            this._datasetReady(this.dataset);
        },
        
        _datasetReady: function(dataset){
            console.log("MapView .. _datasetReady"); 
            
            if(dataset) {
                this.itemsMap = new Object();
                
                var kwArgs = {
                    onComplete: dojo.hitch(this,this._itemFetchComplete),
					//44387: Map Dashlet - Performance - Load only mapped properties from provider vs all
					//Bobby: 18Feb2014, only mapped columns need to be fetched, as per defined in dataset from View JSP
                    //properties: ["all"],
                    start: 0
                };
                
                dataset.getItems(kwArgs);
            }
        },
        
        _getStatusImage: function(status) {  
            var iconDir = dojo.baseUrl + "../dojoe/map/images/32/";
            var statusLabel;
            
            if(typeof(status) == 'object') {
                var height = status.getImageHeight();
                var width = status.getImageWidth();
                
                if(!isNumber(height)) {
                    height = 20;
                }
                
                if(!isNumber(width)) {
                    width = 20;
                }
                
                return {
                    url: this._cleanUpImagePath(status.getImage()), 
                    height: height, 
                    width: width,
                    yOffset: -1 * height + 1,
                    //44720: Map Widget: Markers are slightly off from the lat/long provided
                    //xOffset: 0
					xOffset: -1 * width/2 
                };
            }//44480: Map Widget: Label is not displayed when marker is a non-status column
			//Bobby, 05Feb2014: convert to a string as numbers would cause error 
			else status = status + ""; 
            
            statusLabel = status.toLowerCase();
            //44721: Map Widget: some status mappings are incorrect when used with File Provider
            //Bobby, 13Feb2014, trim the status before comparing to avoid mismatch due to spaces
            statusLabel = statusLabel.trim();
			console.debug("MapView.js:_getStatusImage():statusLabel:"+statusLabel);
			
            if((statusLabel == "info") || (statusLabel == "information")) {
                statusLabel = "informational";
            } else if((statusLabel == "warning") || (statusLabel == "warningminor") || (statusLabel == "warning_minor")) {
                statusLabel = "minor";
            } else if((statusLabel == "warningmajor") || (statusLabel == "warning_major")) {
                statusLabel = "major";
            } else if((statusLabel == "error")) { //mapping error to critical, Bobby:13Feb2014
                statusLabel = "critical";
            }
            
			//console.debug("MapView.js:_getStatusImage(): NEW statusLabel:"+statusLabel);
            var icon = new Object();            
            icon.url = iconDir;
            if(this._isSupportedStatus(statusLabel)) {
                icon.url += statusLabel + ".png";
            } else {
                icon.url += "undefined.png";
            }
            
            icon.height = 25;
            icon.width = 20;
            //44720: Map Widget: Markers are slightly off from the lat/long provided
            //icon.yOffset = -24;
			//icon.xOffset = 0;            
            icon.yOffset = -(icon.height+1);
			icon.xOffset = -(icon.width/2);
            
			//console.debug("MapView.js:_getStatusImage():icon.url:"+icon.url);
            return icon;
        },
        
        _getExistingItems: function(items) {
            var existing = new Array();
            
            for(var i = 0; i < items.length; i++) {
                var item = items[i];
                if(item.getId() in this.itemsMap) {
                    existing.push(item)
                }
            }
            
            return existing;
        },
        
        _getAttrNameFromMappedValue: function(attr) {
        	//44480: Map Widget: Erratic behavior when same column is mapped to multiple map attributes
        	//Bobby, 12Feb2014, return all matching mappings instead of first find
			var attrs = [];
            for(var i = 0; i < this.dashletConfig.attrNameArr.length; i++) {
                var attrName = this.dashletConfig.attrNameArr[i].replace("qe_attr_", "");
                var attrMapping = this.dashletConfig.attrValueArr[i];
                
                if(attr === attrMapping) {
                    //return attrName;
					attrs.push(attrName);
                }
            }
            
            return attrs;
        },

        _getMappedValueFromAttrName: function(attr) {
            for(var i = 0; i < this.dashletConfig.attrNameArr.length; i++) {
                var mapName = this.dashletConfig.attrNameArr[i].replace("qe_attr_", "");
                var attrMapping = this.dashletConfig.attrValueArr[i];
                
                if(mapName === attr) {
                    return attrMapping;
                }
            }
            
            return "None";
        },
        
        _getMarkerDataFromProperties: function(propsMap, item) {
            var foundStatusObject = false;
            //var imageObject = new Object();
            var image = "";
            
            var markerData = {
                lat: 0, lon: 0,
                status: "", tooltip: "", label: "",
                minZoom: 0, maxZoom: 20, zoomTo: -1
            };
             
			//console.debug("MapView.js: _getMarkerDataFromProperties() :propsMap", propsMap); 
            for(var attr in propsMap) {
                var value = propsMap[attr].value;
                var type = propsMap[attr].type;
                //console.debug("MapView.js: _getMarkerDataFromProperties() :attr", attr);
                var attrs = this._getAttrNameFromMappedValue(attr);
				//console.debug("MapView.js: _getMarkerDataFromProperties() :attrs", attrs);
				for(var i=0;i<attrs.length;i++){
					var attr = attrs[i];
					//console.debug("MapView.js: _getMarkerDataFromProperties() :attr", attr);
					switch(attr) {
						case this.mappings.defaults.MARKERLAT:
							markerData.lat = value;
							break;
						case this.mappings.defaults.MARKERLON:
							markerData.lon = value;
							break;
						case this.mappings.defaults.MARKERSTATUS:
							if(type === "status") {
								foundStatusObject = true;
								markerData.status = value;
								markerData.tooltip = markerData.status.getTooltip();
								markerData.label = markerData.status.getLabel();
							} else {
								markerData.status = value;
								if(markerData.tooltip == "") markerData.tooltip = value; //set default tooltip as status
							}                        
							break;
						case this.mappings.defaults.MARKERTOOLTIP:
							if(!foundStatusObject) {
								markerData.tooltip = value;
							}      
							break;
						case this.mappings.defaults.MARKERMINZOOM:
							markerData.minZoom = value;
							break;
						case this.mappings.defaults.MARKERMAXZOOM:
							markerData.maxZoom = value;
							break;
						case this.mappings.defaults.MARKERZOOMTO:
							markerData.zoomTo = value;
							break;
						case this.mappings.defaults.MARKERLABEL:
							if(!foundStatusObject) {
								markerData.label = value;
							}  
							break;
						case this.mappings.defaults.MARKERIMAGE:
							if(!foundStatusObject) {
								image = value;
							}  	
							break;
					}
				}	
            }
			
			//console.debug("MapView.js: _getMarkerDataFromProperties() :image", image);
            
            if(foundStatusObject || (markerData.status != '' && image == '')) {
                markerData.image = this._getStatusImage(markerData.status);
            } else if(image){
				//Bobby, 29Jan2014: intialize markerData.image
				markerData.image = new Object();
                markerData.image.url = image;
                //44720: Map Widget: Markers are slightly off from the lat/long provided
                /* markerData.image.height = 25;
                markerData.image.width = 25;
                markerData.image.yOffset = -24;
				markerData.image.xOffset = 0; */
				//Bobby: 13Feb2014, increasing size for custom images
                markerData.image.height = 35;
                markerData.image.width = 35;
                markerData.image.yOffset = -markerData.image.height;
				markerData.image.xOffset = -markerData.image.width/2;
            } else{
				//44081: Map doesn't render correctly when non-status column is used for Marker
				//Bobby, 29Jan2014: intialize markerData.image
				markerData.image = new Object();
                markerData.image.url = dojo.baseUrl + "../dojoe/map/images/32/unknown.png";
                markerData.image.height = 25;
                markerData.image.width = 25;
                //44720: Map Widget: Markers are slightly off from the lat/long provided
                //markerData.image.yOffset = -24;
				//markerData.image.xOffset = 0;                
                markerData.image.yOffset = -markerData.image.height;
				markerData.image.xOffset = -markerData.image.width/2;;
            }
			
			//42988: Item icon needs to be displayed as well as the status icon
			//Bobby: 09Jan2014, adding support for item style
            var style = item.getItemStyle({
        			mergeWidgetDef: true,
        			widgetDef: this.getNearestPluggableViewWidgetDefinition()
			});
					
			if (style) {
				markerData.bgImage = new Object();
				markerData.bgImage.url =  this._cleanUpImagePath(style.getLargeImage());
        		markerData.bgImage.height =  style.getLargeImageHeight();
				markerData.bgImage.width =  style.getLargeImageWidth();
			}
			
			//console.debug("MapView.js: _getMarkerDataFromProperties() :markerData.image", markerData.image);
			//console.debug("MapView.js: _getMarkerDataFromProperties() :markerData.bgImage", markerData.bgImage);
			//console.debug("MapView.js: _getMarkerDataFromProperties() :markerData:");
			//console.dir(markerData);
            return markerData;
        },
        
        _handleSendNodeClickedOn: function (feature, event) {
            console.log("MapView _handleSendNodeClickedOn :BEGIN", feature);
            var item = this.itemsMap[feature.data.id];
            
            this.selection.setSelectedItem(item);
                        
            this._launchContextMenu(event);
            
            this.eventHelper.publishNodeClickedOnEvent(this.dataset, item.getId(), this.namespace);

            console.log("MapView _handleSendNodeClickedOn :END");
        },
        
        _handleRecievedNodeClickedOn : function(evt) {
            console.info ("MapView: NodeClickedOn event - " + evt.name);
            
            if(this.eventHelper && this.datasetInfo && this.dashletConfig && this.dataset) {       
                this.eventHelper.processEvent(evt, this.datasetInfo, this.dashletConfig, this.dataset);
                this.map.destroyAllMarkers();
            } else {
                this.wireEventsQueue.push(evt)
            }     
        },
        
        _handlePortletResize : function(evt) {
            if(this.map) {
                this.map.onPortletResize();
            }
        },
        
        _handleChangedEvent: function(/*Object*/ event) {
            console.log('_handleChangedEvent');
            this.map.destroyAllMarkers();
            this._datasetReady(event.getDataset());
        },
        
        _handleItemsAddedEvent: function(/*Object*/ event) {
            console.debug('_handleItemsAddedEvent');
            var items = event.getItems();            
            this._itemFetchComplete(items);
        },

        _handleItemsRemovedEvent: function(/*Object*/ event) {
            console.debug('_handleItemsRemovedEvent');
            var items = event.getItems();
            
            for(var i = 0; i < items.length; i++) {
                var item = items[i];
                var id = item.getId();
                
                if(id in this.itemsMap) {
                    delete this.itemsMap[item.getId()];
                    this.map.destroyMarker(id);
                }
            }
        },
        
        _handleItemStatusChangedEvent: function(/*Object*/ event) {
            console.log('_handleItemStatusChangedEvent');
            var items = event.getItems();            
            this._itemFetchComplete(this._getExistingItems(items));
        },
        
        
        _handleItemPropertiesChangedEvent: function(/*Object*/ event) {
            console.log('_handleItemPropertiesChangedEvent');
            var items = event.getItems();
            this._itemFetchComplete(this._getExistingItems(items));
        },
        
        _handleItemStyleChangedEvent: function(/*Object*/ event) {
            console.debug('_handleItemStyleChangedEvent');
            var items = event.getItems();
            this._itemFetchComplete(this._getExistingItems(items));
        },
        
        _handleExceptionEvent: function(/*Object*/ event) {
            this.inherited(arguments);
        },
        
        _handleEventRouterEvent: function(/*Object*/ event) {
            this.inherited(arguments);
        },
        
        _initMap : function() {
            var optionNameArr = this.dashletConfig.optionNameArr;
            var optionValueArr = this.dashletConfig.optionValueArr;
            var mapArgs = new Object();
               
            for ( var i = 0; i < optionNameArr.length; i++) {
                var option = optionNameArr[i].replace("qe_optn_", "");
                var value = optionValueArr[i];
                
                switch(option) {
                    case this.options.SERVERID:
                        mapArgs.mapType = value;
                        break;
                        
                    case this.options.SERVERAPIKEY:
                        mapArgs.apiKey = value;
                        break;
                        
                    case this.options.SERVERLOCALURL:
                        mapArgs.mapLocalUrl = value;
                        break;    
                        
                    case this.options.MAPCENTERLAT:
                        mapArgs.mapCenterLat = value;
                        break;
                    
                    case this.options.MAPCENTERLON:
                        mapArgs.mapCenterLon = value;
                        break;
                    
                    case this.options.MAPCENTERZOOM:
                        mapArgs.mapCenterZoom = value;
                        break;
						
					case this.options.MAPMARKERLABEL:
                        mapArgs.markerLabelEnabled = value;
                        break;	
					
					case this.options.MAPMARKERRESOURCEICON:
                        mapArgs.markerResourceIconEnabled = value;
                    break;	
						
                }
            }
            
            this.map = new MapWidget();
            this.map.createMap(this.mapDivId, mapArgs, lang.hitch(this, function() {
                this.mapLoaded.resolve();
            }));
            
            this.map.setOnContextMenu(lang.hitch(this, this._launchContextMenu));
    
            console.log("MapView: initMap :END");
        },
        
        _isSupportedStatus: function(status) {
			//console.debug("MapView.js:_isSupportedStatus():status:"+status);
			for(var type in this.statusTypes) {
				//console.debug("MapView.js:_isSupportedStatus():this.statusTypes[type]:"+this.statusTypes[type]);
                if(status == this.statusTypes[type]) 
                    return true;
            }
            
            return false;
        },

        _itemFetchComplete: function(items, args) {
            console.log("MapView -- _itemFetchComplete");
			//Bobby, 06Feb2014: limit items to 50 for now, make it configurable in QE
            //for (var i = 0; i < items.length; i ++) {
			var maxLength = 50;
			if (items.length < 50) maxLength = items.length;
			console.debug("MapView.js:_itemFetchComplete():maxLength:"+maxLength);
			for (var i = 0; i < maxLength; i ++) {
                var item = items[i];                
                this.itemsMap[item.getId()] = item;
                
                var kwArgs = {
					//44387: Map Dashlet - Performance - Load only mapped properties from provider vs all
					//Bobby: 18Feb2014, if properties not set, all properties are fetched per item using another request to server 
					properties: this.dashletConfig.mappedColumns,
                    onComplete: lang.hitch(this, this._processItemProperties, item.getId(),item),
                    onError: lang.hitch(this, this.processError)
                };
                
                item.getProperties(kwArgs);
            }
        },
        
        _launchContextMenu: function(feature, event) {
            if(feature) {
                var id = feature.data.id;
                var item = this.itemsMap[id];
                
                if(item) {
                    this.selection.setSelectedItem(item);
                } else {
                    this.selection.deselectAll();
                }                
            } else {
                this.selection.deselectAll();
            }
            
            this.asyncCM.contextMenuOpen(event, this);
        },
        
        _processItemProperties: function(id, item, props, args) {
            var propsMap = new Object();
            
            for(var i=0;i<props.length;i++){
                propsMap[props[i].getId()] = {value: props[i].getValue(), type: props[i]._prop.valueType};
            }
                        
            var markerData = this._getMarkerDataFromProperties(propsMap,item);
            markerData.id = id;
            
            this.mapLoaded.then(lang.hitch(this, function() {
                var numZoomLevels = this.map.getNumZoomLevels();
                markerData.maxZoom = markerData.maxZoom > numZoomLevels ? numZoomLevels : markerData.maxZoom;
                
                this.map.placeMarkerOnMap(markerData, lang.hitch(this, this._handleSendNodeClickedOn));
            }));            
        },
        
        
        
        _setDatasetLoadData: function(dataset){
            this.inherited(arguments);
            this._datasetReady(this.dataset);
        }

        /** Events that arn't applicable...yet...probably when rave comes out...you know..and the map topology stuff is implemented**/
        /**
        //link(s) added to the dataset
        _handleLinksAddedEvent: function(event) {
            // summary:
            //      Handle "linksAdded" event type.
            // event:
            //      An object containing information about what changed in the Dataset.
            // description:
            //      Subclass should override to if it wishes to deal with "linksAdded" events.
            // tags:
            //      protected
            console.debug('_handleLinksAddedEvent - ' + event);
            console.debug(event);
        },
        
        //link(s) removed from the dataset
        _handleLinksRemovedEvent: function(event) {
            // summary:
            //      Handle "linksRemoved" event type.
            // event:
            //      An object containing information about what changed in the Dataset.
            // description:
            //      Subclass should override to if it wishes to deal with "linksRemoved" events.
            // tags:
            //      protected
            console.debug('_handleLinksRemovedEvent - ' + event);
            console.debug(event);
        },
        
        //relationships added/removed
        _handleLinkChangedEvent: function(event) {
            // summary:
            //      Handle "linkChanged" event type.
            // event:
            //      An object containing information about what changed in the Dataset.
            // description:
            //      Subclass should override to if it wishes to deal with "linkChanged" events.
            // tags:
            //      protected
            console.debug('_handleLinkChangedEvent - ' + event);
            console.debug(event);
        },
        
        //link(s) individual status changed
        _handleLinkStatusChangedEvent: function(event) {
            // summary:
            //      Handle "linkStatusChanged" event type.
            // event:
            //      An object containing information about what changed in the Dataset.
            // description:
            //      Subclass should override to if it wishes to deal with "linkStatusChanged" events.
            // tags:
            //      protected
            console.debug('_handleLinkStatusChangedEvent - ' + event);
            console.debug(event);
        },
        
        //link(s) individual properties changed
        _handleLinkPropertiesChangedEvent: function(event) {     
            // summary:
            //      Handle "linkPropertiesChanged" event type.
            // event:
            //      An object containing information about what changed in the Dataset.
            // description:
            //      Subclass should override to if it wishes to deal with "linkPropertiesChanged" events.
            // tags:
            //      protected
            console.debug('_handleLinkPropertiesChangedEvent - ' + event);
            console.debug(event);
        },

        //relationship(s) individual status changed
        _handleRelationshipStatusChangedEvent: function(event) {
            // summary:
            //      Handle "relationshipStatusChanged" event type.
            // event:
            //      An object containing information about what changed in the Dataset.
            // description:
            //      Subclass should override to if it wishes to deal with "relationshipStatusChanged" events.
            // tags:
            //      protected
            console.debug('_handleRelationshipStatusChangedEvent - ' + event);
            console.debug(event);
        },
        
        //relationship(s) indivdiaul properties changed
        _handleRelationshipPropertiesChangedEvent: function(event) {
            // summary:
            //      Handle "relationshipPropertiesChanged" event type.
            // event:
            //      An object containing information about what changed in the Dataset.
            // description:
            //      Subclass should override to if it wishes to deal with "relationshipPropertiesChanged" events.
            // tags:
            //      protected
            console.debug('_handleRelationshipPropertiesChangedEvent - ' + event);
            console.debug(event);
        }           
        **/
    });
});