/*********************************************************** {COPYRIGHT-TOP} ***
* Licensed Materials - Property of IBM
* Tivoli Widget Library
*
* (C) Copyright IBM Corp. 2014.  All Rights Reserved.
*
* US Government Users Restricted Rights - Use, duplication, or
* disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
************************************************************ {COPYRIGHT-END} **/
define("ibm/tivoli/tip/navwidget/smarttext/dijit/SmartTextView", 
	[
		"dojo/_base/declare",
		"dijit/layout/ContentPane",
		"ibm/tivoli/tip/navwidget/dijit/NavWidgetPluggableView",
		"dojox/widget/Standby",
		"dojo/number",
		"dojo/date/locale"
	],
	function(declare, ContentPane, PluggableView, Standby, number, dateLocale) {
		return declare("ibm.tivoli.tip.navwidget.smarttext.dijit.SmartTextView", [ContentPane, PluggableView], {
		
		datasetInfo: null,
		_usesWidgetDefinition : false,
		items: null,
		smartTextEditorValue: null,
		uniqueId: null,
		_count: null,
		oslcResourceURI: null,
		oslcResourceURIExists: false,
		itemId: null,
		standby: null,
		theme: "oneui",
		
		constructor: function(args1, args2) {
			this.smartTextEditorValue = args1.smartTextEditorValue;
			this.uniqueId = args1.datasetInfo.uniqueId;
			this._count = args1.datasetInfo.count;
			this.oslcResourceURIExists = args1.datasetInfo.oslcResourceUri;
		},
		
		_setDatasetLoadData: function(dataset){
			this.inherited(arguments);
			this._datasetReady(this.dataset);
			
			if(!this.standby) {				
				var theme = dojo.body().className;
				if (theme.indexOf( "oneui" ) != -1){
					this.theme = "oneui"	;
				}else 
					if(theme.indexOf( "breeze" ) != -1 || theme.indexOf( "TIPDark" ) != -1) {
					this.theme = "breeze";
				}
				this.standby = new Standby({target: this.domNode, color:"transparent", image: dojo.moduleUrl("themes."+this.theme,"dojoe/common/themes/"+this.theme+"/images/progress22.gif")});
				this.domNode.appendChild(this.standby.domNode);
				this.standby.startup();
			}
			
		},		
		
		_datasetReady: function(dataset){
			console.info('_datasetReady');
			//console.debug(dataset);
						
			if(dataset){
				this.dataset = dataset;
			}
			
			var uniqueCols = this.getUniqueColumns(this.smartTextEditorValue);

			if(uniqueCols.length > 0) {
			
				this._start = 0; 		
				var kwArgs = {};
				kwArgs.onComplete = dojo.hitch(this,this._itemFetchComplete);
				kwArgs.properties = uniqueCols;	
				if(dataset !== null) {
					kwArgs.count = this._count;
					kwArgs.start = 0;					
					dataset.getItems(kwArgs);
				}
			} else {
				this.setContent(this.smartTextEditorValue);				
			}
		},
	
		_itemFetchComplete: function(items) {
			console.info('_itemFetchComplete');
			//console.debug(items);
			
			this.items = items;
			/*if(this.items) 
				window[this.uniqueId+'totalItems'] = items.length;
			else
				window[this.uniqueId+'totalItems'] = 0;
			*/
			if(this.items) 
				window[this.uniqueId+'currentItemsCount'] = items.length;
			else
				window[this.uniqueId+'currentItemsCount'] = 0;
				
			//console.debug('this.smartTextWidget', this.smartTextWidget);
			//console.debug('items[0].getLabel', items[0].getLabel());
			
			if(items.length != 0){
				this.processData(items, this.smartTextEditorValue);
			} else {
				var uniqueColumns = this.getUniqueColumns(this.smartTextEditorValue);
				 for(var index=0; index<uniqueColumns.length; index++){
					var itemValueObj = uniqueColumns[index];
					var templateKey = '${'+itemValueObj+'}';
					
					while(this.smartTextEditorValue.indexOf(templateKey) != -1) {
						this.smartTextEditorValue = this.smartTextEditorValue.replace(templateKey, itemValueObj);
					}
				}
				this.setContent(this.smartTextEditorValue);				
			}
		},
				
		_handleItemStatusChangedEvent: function(event) {
			console.debug("SmartTextView.js: itemStatusChanged event received");
			
			this._datasetReady(this.dataset);
		},
		
		_handleItemPropertiesChangedEvent: function(event) {	
			console.debug("SmartTextView.js: itemPropertiesChanged event received");
			this._datasetReady(this.dataset);
		},

		_handleChangedEvent: function(event) {	
			console.debug("SmartTextView.js: datasetChanged event received");
			this._datasetReady(this.dataset);
		},
		
		showBusyPane: function(event){		
			if(this.standby) this.standby.show();		
		},	
		
		hideBusyPane: function(event){		
			if(this.standby) this.standby.hide();		
		},
		
		forceHideBusyPane: function(event){		
			if(this.standby) this.standby.hide();		
		},
		
		processData: function(data, smartTextEditorValue) {
			console.debug("SmartTextView.js: processData - Enter");
			//console.info('smartTextEditorValue:'+smartTextEditorValue);
			console.info('data.length:'+data.length);
			
			var uniqueColumns = this.getUniqueColumns(smartTextEditorValue);
			//console.debug('uniqueColumns:', uniqueColumns);
			var uniqueColumnsWithFormatTag = this.getUniqueColumnsWithFormat(smartTextEditorValue);
			//console.debug('uniqueColumnsWithFormatTag', uniqueColumnsWithFormatTag);
			
			for(i=0;i<data.length;i++) {
				if(data[i]._item.properties) {
				for(var index=0; index<uniqueColumns.length; index++){
						//console.debug('item.getProperty:', data[i].getProperty(uniqueColumns[index]));						
						
						if(data[i].getProperty(uniqueColumns[index])) {
							//console.debug('item.displayvalue:', data[i].getProperty(uniqueColumns[index]).getDisplayValue());
							var dispValue = data[i].getProperty(uniqueColumns[index]).getDisplayValue();
							//console.info('dispValue:'+dispValue+'<<');
							//43283: TWL 3.2 - SmartText - Formatting is not working
							var actualValue = data[i].getProperty(uniqueColumns[index]).getValue();							
							var colType = data[i].getProperty(uniqueColumns[index]).getValueType();
							
							//console.info('oslcResourceURIExists:'+this.oslcResourceURIExists+'<<');
							if(this.oslcResourceURIExists == true && uniqueColumns[index] == 'OslcResourceURI') {
								this.oslcResourceURI = dispValue;
							}
							//console.info('oslcResourceURI:'+this.oslcResourceURI+'<<');
							
							var colName = uniqueColumns[index];
							var templateKey = '${'+colName+'}';
							
							while(smartTextEditorValue.indexOf(templateKey) != -1) {
								smartTextEditorValue = smartTextEditorValue.replace(templateKey, dispValue);
							}
							
							//to replace formatTag with actual data formatting
							//43283: TWL 3.2 - SmartText - Formatting is not working - use actualValue
							smartTextEditorValue = this.doFormatColData(uniqueColumnsWithFormatTag, colName, smartTextEditorValue, actualValue, data[i]);

						} else {
							var itemValueObj = uniqueColumns[index];
							var templateKey = '${'+itemValueObj+'}';
							
							while(smartTextEditorValue.indexOf(templateKey) != -1) {
								smartTextEditorValue = smartTextEditorValue.replace(templateKey, itemValueObj);
							}
						}
					}
					
					this.setContent(smartTextEditorValue);
				}
				
				window[this.uniqueId+'itemIndex'] = 0;
				var methodNameIndexUpdate = this.uniqueId+'updateIndexDisplay';
				window[methodNameIndexUpdate]();
				
				break;					
			}	
			
			console.debug("SmartTextView.js: processData-Exit");
		},
		
		
		getUniqueColumns: function(smartTextEditorValue) {
			//console.info('getUniqueColumns - start');
			//console.info('this.smartTextEditorValue:'+smartTextEditorValue);
			var colNamesUnique = new Array();
			if(smartTextEditorValue != null) {
			
				var regExprColFormat = /\%\{([^\}]+)\}/g;
				smartTextEditorValue = smartTextEditorValue.replace(regExprColFormat, '');
				//console.info('modStr:'+smartTextEditorValue);
				
				var regExprMatch = /\$\{([^\}]+)\}/g;
				var colArray = smartTextEditorValue.match(regExprMatch);
				//console.info('colArray:'+colArray);
				if(colArray!=null)
				 for(var index=0; index < colArray.length; index++) {
					var colName = colArray[index].replace('${', '').replace('}', '');
					//console.info('colName:'+colName);
					if(dojo.indexOf(colNamesUnique, colName) == -1)
						colNamesUnique.push(colName);
				 }
				console.info('colNamesUnique:'+colNamesUnique);
			}
			
			//console.info('oslcResourceURIExists:'+this.oslcResourceURIExists+'<<');
			if(this.oslcResourceURIExists == true) {
				colNamesUnique.push('OslcResourceURI');
			}
			console.info('getUniqueColumns - end');
		    return colNamesUnique;
		},
		
		getUniqueColumnsWithFormat: function(smartTextEditorValue) {
			console.info('getUniqueColumnsWithFormat - start');
			//console.info('this.smartTextEditorValue:'+smartTextEditorValue);
			var colNamesUnique = new Array();
			if(smartTextEditorValue != null) {

				var regExprMatch = /\$\{([^\}]+)\}+\}/g;
				var colArray = smartTextEditorValue.match(regExprMatch);
				//console.info('colArray:'+colArray);
				if(colArray!=null)
				 for(var index=0; index<colArray.length; index++) {
					var colName = colArray[index];
					//console.info('colName:'+colName);
					if(dojo.indexOf(colNamesUnique, colName) == -1)
						colNamesUnique.push(colName);
				 }
				console.debug('colNamesUnique:', colNamesUnique);
			}
			
			//console.info('oslcResourceURIExists:'+this.oslcResourceURIExists+'<<');
			if(this.oslcResourceURIExists == true) {
				colNamesUnique.push('OslcResourceURI');
			}
			console.info('getUniqueColumnsWithFormat - end');
		    return colNamesUnique;
		},		
		
		getNextItem: function(smartTextEditorValue, itemIndex) {
			console.info('getNextItem in view - start');
			console.info('itemIndex:'+itemIndex);
			
			var currentIndexVarName = this.uniqueId+'itemIndex';			
			var currentIndex = window[currentIndexVarName];
			//console.info('currentIndex:'+currentIndex);
			currentIndex = itemIndex;
			var data = this.items;
			
			//console.info('data.length:'+data.length);			
			var uniqueColumns = this.getUniqueColumns(smartTextEditorValue);			
			var uniqueColumnsWithFormatTag = this.getUniqueColumnsWithFormat(smartTextEditorValue);
			
			if(uniqueColumns.length > 0) {
					for(var index=0; index<uniqueColumns.length; index++) {						
						//console.debug('item.getProperty:', data[currentIndex].getProperty(uniqueColumns[index]));						
						
						if(data[currentIndex].getProperty(uniqueColumns[index])) {
							//console.debug('item.displayvalue:', data[currentIndex].getProperty(uniqueColumns[index]).getDisplayValue());
							this.itemId = data[currentIndex].getId();
							var dispValue = data[currentIndex].getProperty(uniqueColumns[index]).getDisplayValue();
							//47031
							var actualValue = data[currentIndex].getProperty(uniqueColumns[index]).getValue();								
							var colType = data[currentIndex].getProperty(uniqueColumns[index]).getValueType();
							
							if(this.oslcResourceURIExists == true && uniqueColumns[index] == 'OslcResourceURI') {
								this.oslcResourceURI = dispValue;
							}
							
							var colName = uniqueColumns[index];						
							var templateKey = '${'+colName+'}';						
							//console.info('templateKey:'+templateKey+'<<');
							
							while(smartTextEditorValue.indexOf(templateKey) != -1) {
								smartTextEditorValue = smartTextEditorValue.replace(templateKey, dispValue);
							}
							
							//to replace formatTag with actual data formatting
							//47031
							smartTextEditorValue = this.doFormatColData(uniqueColumnsWithFormatTag, colName, smartTextEditorValue
													, actualValue, data[currentIndex]);
					   } else {
							var itemValueObj = uniqueColumns[index];
							var templateKey = '${'+itemValueObj+'}';
							
							while(smartTextEditorValue.indexOf(templateKey) != -1) {
								smartTextEditorValue = smartTextEditorValue.replace(templateKey, itemValueObj);
							}
					   }	
					this.setContent(smartTextEditorValue);	
					
					//console.info('currentIndexVarName'+currentIndexVarName);
					window[currentIndexVarName] = currentIndex;					
				}
			}
			//console.info('getNextItem in view - end');
		},
		
		getDatasetItemsCount: function() {
			console.info('getDatasetItemsCount');
			console.debug('dataset', this.dataset.getItemCount());
		},
		
		doFormatColData: function(uniqueColumnsWithFormatTag, colName, smartTextEditorValue, 
									dispValue, data) {
			//to replace formatTag with actual data formatting
			for(j=0;j<uniqueColumnsWithFormatTag.length;j++) {
				var colWithFormatTagOrig = uniqueColumnsWithFormatTag[j];
				//console.info('colWithFormatTagOrig:'+colWithFormatTagOrig);
				
				var colWithFormatTag = unescapeHTML(colWithFormatTagOrig);
				//console.info('colWithFormatTag:'+colWithFormatTag);				
				//console.info('colName:'+colName);
				//console.info('colName index:'+colWithFormatTag.indexOf(colName));
				
				//Eg. ${DATE%{"formatTag":"dojoDate", "datePattern":"EEE, MMM d, ''yy"}}
				if(colWithFormatTag.indexOf(colName) == 2) {
					console.info('do formatting');
					
					var str1 = colWithFormatTag;
					var colFmtArray = str1.match(/\%\{([^\}]+)\}/g);
					var str2 = colFmtArray[0];
					str2 = str2.replace('%', '');
					
					var jsonObj = null;
					var strFormatTag = '';
					try {
						jsonObj = JSON.parse(str2);
					} catch(err) {
						console.error('JSON parse exception:'+err+', '+str2);
					}
					//console.debug('jsonObj:', jsonObj);
					
					if(jsonObj) {
						strFormatTag = jsonObj.formatTag;				
					}
					//console.debug('strFormatTag:', strFormatTag);
					//console.debug('dispValue:', dispValue);
					var dispValueFmtd = null;
					if(strFormatTag == 'dojoNumber') {						
						dispValueFmtd = number.format(new Number(dispValue), jsonObj);
					} else if(strFormatTag == 'dojoDate') {
						dispValueFmtd = dateLocale.format(new Date(dispValue), jsonObj);
					} else if(strFormatTag == 'dashStatus') {
						var imageRootPath = "/ibm/console/secure/isclite/scripts/tipdojo/ibm/tivoli/tip/navwidget/themes";
						//console.debug('statusObj', dispValue);
						switch(jsonObj.selector){
							case "value" :
							 dispValueFmtd = dispValue.getValue();
							 break;
							case "imageUrl" :
							 if(dispValue.getScalarImage()!=null && dispValue.getScalarImage()!=""){
								dispValueFmtd = imageRootPath+dispValue.getScalarImage().replace("[widget]","");
							 }else if(dispValue.getImage()!=null){
								dispValueFmtd = imageRootPath+dispValue.getImage().replace("[widget]","");
							 }
							 break;
							case "label" :
							 dispValueFmtd = dispValue.getLabel();
							 break;
							default :
							 console.debug('Incorrect selector for STATUS type');
						} 	
							
					} else {
						console.info('formatTag is unknown, no formatting done.');
					}
					//console.debug('dispValueFmtd:', dispValueFmtd);
					while(smartTextEditorValue.indexOf(colWithFormatTagOrig) != -1) {
						smartTextEditorValue = smartTextEditorValue.replace(colWithFormatTagOrig, dispValueFmtd);
					}									
				}
			} // end of for loop
			
			return smartTextEditorValue;
		}		
	});
});
