/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */


require(["dojo/_base/config",
         "dojo/dom",
         "dojo/dom-attr",
         "dojo/parser",
         "dojo/topic",
         "dijit/registry",
         "dijit/layout/BorderContainer",
         "dijit/layout/ContentPane",
         "ibm/usmi/core/framework/frameworkdata"],
    function (config, dom, domAttr, parser, topic, registry, BorderContainer, ContentPane, frameworkdata) {

        var mainTitle = "IBM Systems Director 6.4",
            requestParms = {},
            myframework;

        function init() {
            //nothing right now. 
        }

        /*
        * mainLoader:
        * callback thats going to do all the work after we get our CURI for product name and console session
        *
        */
        //TODO: revisit the structure of product info. Instead of 3 random strings in an array, use an object, or label them
        function mainLoader(productInfo) {
            //set the product name with string we received from frameworkdata curi call
            var requiredClasses,
                selector,
                workAreaDijit,
                container,
                cpane,
                cpane2,
                productDiv = dom.byId("header-product"),
                usernameDiv = dom.byId("username_inner1"),
                servernameDiv = dom.byId("servername"),
                logoutAnchor = dom.byId("logout_link");

            if (productDiv !== null) {
                productDiv.innerHTML = productInfo[0];
            }
            if (usernameDiv !== null) {
                usernameDiv.innerHTML = productInfo[1];
            }
            if (servernameDiv !== null) {
                servernameDiv.innerHTML = location.hostname;
            }
            if (logoutAnchor !== null) {
                domAttr.set(logoutAnchor, "href", "/ibm/console/logout.do?XSS=" + productInfo[2]);
            }

            try {
                if (config.workAreaDijit) {
                    requiredClasses = [config.workAreaDijit];
                    console.log("***config.workAreaDijit name on url: " + config.workAreaDijit);

                    if (config.selector === "tree") {
                        console.info("adding a TREE selector");
                        requiredClasses[1] = "ibm/usmi/widgets/TreeSelector";
                    } else if (config.selector === "details") {
                        console.info("adding a DETAILS selector");
                        requiredClasses[1] = "ibm/usmi/widgets/DetailsSelector";
                    } else if (config.selector) {   //class specified
                        console.info("adding a CUSTOM selector: " + config.selector);
                        requiredClasses[1] = config.selector;
                    } else {
                        console.info("NO selector requested!!");
                    }

                    //listen for task title changes. Do this before creating the dijits so you don't miss anything
                    topic.subscribe("taskTitleChange",
                                   function (title) {
                            console.info("received title change event!!!");
                            dom.byId("titleHeader").innerHTML = title;
                            document.title = mainTitle + " : " + title;
                        });

                    //require classes as needed on URL and create dijits
                    require(requiredClasses,
                            function (WorkAreaDijit, Selector) {
                            if (Selector) {
                                selector = new Selector({id: "mainSelector"});
                            }
                            workAreaDijit = new WorkAreaDijit({id: "mainWorkAreaDijit"});
                        });

                    container = registry.byId("pageLayout");
                    if (selector !== undefined) {
                        console.info("Adding Selector");
                        cpane = new ContentPane({
                            region: "left",
                            layoutPriority: "4",
                            splitter: true,
                            "class": "temptest", //have to have a width set
                            content: selector,
                            id: "selectorContentPane"
                        });
                        container.addChild(cpane);
                    }

                    cpane2 = new ContentPane({
                        region: "center", //center does nothing for some reason
                        layoutPriority: "3",
                        "class": "test2",
                        content: workAreaDijit,
                        id: "workAreaContentPane"
                    });
                    container.addChild(cpane2);

                } else {
                    console.log("***NO myNameSpace.dijit name on url");
                }
            } catch (e) {
                console.warn("caught exception loading: " + config.workAreaDijit);
                console.warn(e);
                dom.byId("pageLayout").innerHTML = "<span>error loading <b>" + config.workAreaDijit + " </b>";
            }
        }//end mainLoader


        //all this method should really do is set the location
        //user clicks menu, we set location here. Then on load of this html, the dijitname gets parsed out (see above)
        //the input needs to be longer and uglier to handle the modulepath
        function menuClicked(URLparms) {
            console.log("passed parm:" + URLparms);
        }

        function signout() {
            var logoutForm = document.createElement("form"),
                field1 = document.createElement("input"),
                field2 = document.createElement("input");
            logoutForm.setAttribute("method", "POST");
            logoutForm.setAttribute("action", "ibm_security_logout");
            logoutForm.setAttribute("name", "logout");
            logoutForm.setAttribute("style", "display: none");

            field1.setAttribute("type", "submit");
            field1.setAttribute("name", "logout");
            field1.setAttribute("value", "LOGOUT");

            field2.setAttribute("type", "hidden");
            field2.setAttribute("name", "logoutExitPage");
            field2.setAttribute("value", document.location);

            logoutForm.appendChild(field1);
            logoutForm.appendChild(field2);
            document.body.appendChild(logoutForm);
            logoutForm.submit();
        }

        function debugLogging() {
            var infoDiv = dom.byId("nonsense"),
                infoTxt = "<br/><br/><span>location: <i>" +
                        document.location + "</i><BR/> location.search : <i>" + document.location.search + "</i><br/><br/></span>";
            if (infoDiv) {
                infoDiv.innerHTML = infoDiv.innerHTML + infoTxt;
            } else {
                console.info("No info div present.");
                console.info(infoTxt);
            }
        }

        //init, debug, then get some CURI for session and product name
        init();
        debugLogging();
        console.info("pre-ready");
        //when we're ready, parse the page and make our CURI call. 
        //This manual parse is used b/c we're parseOnLoad: false, the preferred way to roll
        require(["dojo/ready"], function (ready) {
            ready(function () {
                console.info("ENTER: ready block");
                parser.parse();
                myframework = new frameworkdata();
                topic.subscribe("curi-framework-ready", mainLoader);
                console.info("EXIT: ready block");
            });
        });
    }); //end function