/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
/**
 * This file is your application's main JavaScript file. It is listed as a
 * dependency in run.js and will automatically load when run.js loads. Because
 * this file has the special filename main.js, and because we've registered the
 * appspackage in run.js, whatever object this module returns can be loaded by
 * other files simply by requiring apps(instead of apps/main). Our first
 * dependency is to the dojo/has module, which allows us to conditionally
 * execute code based on configuration settings or environmental information.
 * Unlike a normal conditional, these branches can be compiled away by the build
 * system; see staticHasFeatures in app.profile.js for more information. Our
 * second dependency is to the special module require; this allows us to make
 * additional require calls using relative module IDs within the body of our
 * define function. In all cases, whatever function is passed to
 * define("ibm/usmi/core/framework/main", "ibm/usmi/core/framework/main", ) is only invoked once, and the return
 * value is cached. More information about everything described about the loader
 * throughout this file can be found at
 * http://livedocs.dojotoolkit.org/loader/amd.
 */
/*
 * This require calls first dependency, ./Dialog, uses a relative module identifier; you should use this
 * type of notation for dependencies *within* a package in order to ensure the package is fully portable. It
 * works like a path, where ./refers to the current directory and ../ refers to the parent directory. If
 * you are referring to a module in a *different* package, you should *not* use a relative module identifier.
 *
 * The second dependency is a plugin dependency; in this case, it is a dependency on the special functionality
 * of the dojo/domReady plugin, which waits until the DOM is ready before finishing loading.
 * The ! after the module name indicates you want to use special plugin functionality; if you were to
 * require just dojo/domReady, it would load that module just like any other module, without any of the
 * special plugin functionality.
 */

require(
		[
			"dojo/_base/kernel",
			"dijit/Menu",
			"dijit/MenuItem",
			"ibm/usmi/core/framework/ContentTitle",
			"ibm/usmi/menus/Menu",
			"ibm/usmi/menus/NextGenUIMenuBar",
			"ibm/usmi/menus/NextGenUIHelpMenu",
			"ibm/usmi/core/framework/frameworkdata",
			"ibm/usmi/widgets/util/SessionKeepAlive",
			"dojo/topic",
			"ibm/usmi/core/utils/Logger",
			"idx/app/Header",
			"dijit/registry",
			"dijit/layout/ContentPane",
			"dojo/i18n!ibm/usmi/nls/frameworkResources",
			"ibm/usmi/util/DefaultDialogTask",
			"ibm/usmi/util/DefaultURLLaunchTask",
			"ibm/usmi/util/DefaultUserInterfaceTask",
			"ibm/usmi/util/DefaultMessageHandler",
			"dojo/_base/lang",
			"dojo/json",
			"dojo/date/locale",
			"dijit",
			"dojo/parser",
			"dijit/registry",
			"idx/widget/MenuBar",
			"dijit/PopupMenuBarItem",
			"dijit/form/TextBox",
			"dijit/form/Button",
			"idx/widget/MenuDialog",
			"idx/widget/Menu",
			"idx/widget/MenuHeading",
			"dijit/layout/BorderContainer",
			"dojo/domReady!"
		],
		function (dojo, Menu, MenuItem, ContentTitle, NavMenu, NextGenUIMenuBar, NextGenUIHelpMenu, frameworkdata,
				SessionKeepAlive, topic, Logger, Header, registry, ContentPane, MRI, DefaultDialogTask, DefaultURLLaunchTask,
				DefaultUserInterfaceTask, DefaultMessageHandler, djLang, JSON, locale) {

			var _frameworkCoreMainLogger = new Logger("ibm.usmi.core.framework", "main");
			var selector;
			var workAreaDijit;
			var lastHash;
			var lastKnownWidgetCount = -1;
			var myframework;

			dojo.addOnUnload(function () {

				// don't destroy stuff on a logout because we will get
				// lots of popups in recent FF browsers asking us if we
				// really want to leave the page.  This is caused by CURI
				// calls to kill the datasets.  We don't need to make those
				// calls in the case of logout anyway.
				if (!nguiProperties.logoutEventProcessed) {
					clearMessageHandlers();
					topic.publish("DefaultDialogDismissal");
					if (selector) {
						selector.destroyRecursive();
						selector = null;
					}
					if (workAreaDijit) {
						workAreaDijit.destroyRecursive();
						workAreaDijit = null;
					}
					if (dijit.byId("jobStatusPod")) {
						dijit.byId("jobStatusPod").destroy();
					}
					if (dijit.byId("alertStatusPod")) {
						dijit.byId("alertStatusPod").destroy();
					}
					if (dijit.byId("finder")) {
						dijit.byId("finder").destroyRecursive();
					}
					if (dijit.byId("menus")) {
						dijit.byId("menus").destroyRecursive();
					}
					if (dijit.byId("menus2")) {
						dijit.byId("menus2").destroyRecursive();
					}
					if (dijit.byId("sessionKeepAlive")) {
						dijit.byId("sessionKeepAlive").destroy();
					}

					if (myframework) {
						myframework.destroy();
						myframework = null;
					}
					//TODO: if there are more widgets with datasets, or plain datasets lying around, they need to be cleaned up as well.
					//TODO: do we need to destroy the task helper instantiations here as well? (_urlTaskUtil,_dialogTaskUtil,_uiTaskUtil ?)
					//TODO: we could add a nicer DOM element that covers the page during cleanup so that it 
					//      doesn't look goofy as widgets dissapear one by one as they are cleaned up
				}
			});

			// construct the task listeners in the framework, so each individual widget doesn't have 
			// to add/cleanup the utilities
			var _urlTaskUtil = new DefaultURLLaunchTask();
			var _dialogTaskUtil = new DefaultDialogTask();
			var _uiTaskUtil = new DefaultUserInterfaceTask();

			var redirectOpenPagesOnLogout = function () {
				// a page refresh triggers the same exception with event_router as logging out does 
				// check to see if this was a page refresh, if it was, reset the page refresh value
				// and don't log out
				if (sessionStorage.refreshStatus === 'refreshHappened') {
					sessionStorage.refreshStatus = 'noRefresh';
				} else {
					//logout events will keep coming while the server is down, only want
					//to process the first logout event that we get
					if (!nguiProperties.logoutEventProcessed) {
						nguiProperties.logoutEventProcessed = true;

						var iscPageName = "ISC_";
						if (sessionId) {
							iscPageName += sessionId.replace(/-/g, "_");
						}
						// there's an isc window and it's not closed
						if (iscWindow && !iscWindow.closed) {
							iscWindow.window.location = "/ibm/console";
						}
						// there may still be an isc window but the parent/child link was broken
						else if (!iscWindow) {
							iscWindow = (window.open("", iscPageName));
							if (iscWindow && iscWindow.window.frames.length > 0) {
								iscWindow.window.location = "/ibm/console";
							} else {
								if (iscWindow) {
									iscWindow.window.close();
								}
							}
						}

						window.open("login.html", window.name);
					}
				}
			};

			function populateMenuBar () {
				var menuBar = dijit.byId("menus2");
				menuBar.populateMenuBarMenus();

			}

			function populateHelpMenu () {
				var help = dijit.byId("displayHelpMenu");
				help.createHelpMenu();
			}

			function mainLoader (data) {
				var sessionKeepAliveWidget, actionsMenu, helpMenu, nextGenUIMegaMenuBar, navMenu, oneUIHeader, primaryTitleValue, primaryTitleLinkValue, logoLinkValue, navigationValue, helpValue, userValue, secondaryNavigationValue, secondarySearchValue;

				sessionId = data[2];
				popupBlockerWarningMessage = MRI["PopupBlockerWarning"];

				//only set the window name if it hasn't already been set yet. May have been set
				//when launched from ISC and we want to keep that name
				if (window.name === '' && sessionId) {
					window.name = "NGUI_" + sessionId.replace(/-/g, "_");
				}

				window.onbeforeunload = function () {
					sessionStorage.refreshStatus = 'refreshHappened';
				};

				if (!nguiProperties.hideBanner) {
					if (!nguiProperties.hideMenuBar) {
						nextGenUIMegaMenuBar = new NextGenUIMenuBar({
							id : "menus2"
						});
					}
					if (!nguiProperties.hideStatus || !nguiProperties.hideJobs || !nguiProperties.hideUser
							|| !nguiProperties.hideHelp) {

						if (!nguiProperties.hideHelp) {
							helpMenu = new NextGenUIHelpMenu({
								id : "displayHelpMenu"
							});
						}
						if (!nguiProperties.hideUser) {
							actionsMenu = new Menu({
								id : "displayNameMenu"
							});
							actionsMenu.addChild(new MenuItem({
								label : MRI["Signout"],
								onClick : function () {
									signout();
								}
							}));
							userValue = {
								'displayName' : data[1],
								'menu' : actionsMenu
							};
						}
						navMenu = new NavMenu({
							id : "menus",
							showProblemStatusPod : data[3] && !nguiProperties.hideStatus,
							showJobStatusPod : data[4] && !nguiProperties.hideJobs,
							userMenu : userValue,
							helpMenu : helpMenu
						});
					}
				}

				document.title = data[0];

				dojo.parser.parse();

				if (!nguiProperties.hideBanner) {

					if (!nguiProperties.hideProductName) {
						primaryTitleValue = data[0] + ' - ' + data[5]; // - VERSION

						if (!nguiProperties.productNameUnclickable) {
							primaryTitleLinkValue = "menuClicked('selector=ibm/usmi/dashboard/ResourceDashBoard&workAreaDijit=ibm/usmi/widgets/WorkArea')";
							logoLinkValue = "launchURL('http://www.ibm.com')";
						}
					}
					if (!nguiProperties.hideMenuBar) {

						secondaryNavigationValue = 'menus2';
						secondarySearchValue = {
							entryPrompt : MRI.FinderTooltipHelp,
							finderWidget : 'ibm/usmi/widgets/finder/Finder'
						};
					}
					if (!nguiProperties.hideStatus || !nguiProperties.hideJobs || !nguiProperties.hideUser
							|| !nguiProperties.hideHelp) {
						navigationValue = 'menus';
					}
					oneUIHeader = new Header({
						id : 'oneUIHeader',
						region : 'top',
						layoutPriority : 1,
						primaryBannerType : 'thin',
						primaryTitle : primaryTitleValue,
						primaryTitleLink : primaryTitleLinkValue,
						logoLink : logoLinkValue,
						secondaryNavigation : secondaryNavigationValue,
						secondarySearch : secondarySearchValue,
						navigation : navigationValue
					});
				}

				var pageLayout = dijit.byId("pageLayout");
				if (pageLayout && !nguiProperties.hideBanner) {
					pageLayout.addChild(oneUIHeader);
				} else {
					if (_frameworkCoreMainLogger.isError()) {
						_frameworkCoreMainLogger.error("Page layout widget not found.", "mainLoader");
					}
				}

				sessionKeepAliveWidget = new SessionKeepAlive({
					id : "sessionKeepAlive"
				}, 'sessionKeepAliveContainer');

				new ContentTitle({
					'productName' : data[0]
				}, "contentTitleDiv");

				processWorkArea();

				topic.subscribe("logout-event", redirectOpenPagesOnLogout);
				
				if (localStorage.serverIsNISTCompliant === "true") {
					var sslMessageHandler = new DefaultMessageHandler({}), userResponse, fullDescription, title;
					fullDescription = MRI["NISTComplianceInfoExplan"];
					if(localStorage.infoCenterURL !== "") {
						userResponse = MRI["NISTComplianceInfoResponse"];
						// TODO:  need to replace with actual URL
						userResponse = userResponse.replace('href=""', 'href="javascript:void(0)" onclick=launchURL("' + localStorage.infoCenterURL + '/topic/com.ibm.acc.8731.doc/com.ibm.director.plan.helps.doc/fqm0_t_planning_ssl_configuration.html");return false;');
						fullDescription = fullDescription + "<p>" + userResponse + "</p>";
					} 
					
					title = djLang.replace(MRI["NISTComplianceInfoTitle"], [data[0]]);
					sslMessageHandler.displayMessage("", "warning", title, fullDescription , false);
					
				}

				if (gInvalidUser) {
					// no data for this user, which means it has no right to do anything
					var handler = new DefaultMessageHandler({});
					var disMsg = djLang.replace(MRI["InvalidUser"], [ userValue.displayName
					]);
					handler.displayTMSMessage("ATKCOR034E", DefaultMessageHandler.ERROR, disMsg);
				}
				
				if(localStorage.sslCertInfo && localStorage.sslCertInfo !== "") {
					displaySSLMessage(JSON.parse(localStorage.sslCertInfo), data[0]);
				}	
			}
			
			/*
				determines if the user needs to be warned if the SSL certificate is about to expire or
				is expired. 
				sslCertInfo: object that contains the ssl warning type and expiration date information
			*/
			function displaySSLMessage(sslCertInfo, productName) {
				var sslMessageHandler = new DefaultMessageHandler({}), userResponse, fullDescription, title;
				
				fullDescription = MRI["SSLCertExpiryExplan"];
				if(localStorage.infoCenterURL !== "") {
					userResponse = MRI["SSLCertExpiryResponse"];
					userResponse = userResponse.replace('href=""', 'href="javascript:void(0)" onclick=launchURL("' + localStorage.infoCenterURL + '/topic/com.ibm.acc.8731.doc/com.ibm.director.plan.helps.doc/fqm0_t_planning_ssl_configuration.html");return false;');
					fullDescription = fullDescription + "<p>" + userResponse + "</p>";
				} 
				if(sslCertInfo.messageType === 'sslWarning') {
						//make sure the expiration date is valid
					if(sslCertInfo.expiryMonth && getMonthName(sslCertInfo.expiryMonth) !== "" && sslCertInfo.expiryDay && sslCertInfo.expiryDay !== null && sslCertInfo.expiryYear && sslCertInfo.expiryYear !== null)
					{	
						var expDate = new Date(sslCertInfo.expiryYear, sslCertInfo.expiryMonth, sslCertInfo.expiryDay, 0, 0, 0, 0); 
						expDate = locale.format( expDate, {selector:"date", formatLength: "medium" } );
						title = djLang.replace(MRI["SSLCertExpiryWarningTitle"], [productName, expDate]);
						sslMessageHandler.displayMessage("", "warning", title, fullDescription , false);
					} 
				} else if(sslCertInfo.messageType === 'sslExpired') {
					//make sure the expiration date is valid
					if(sslCertInfo.expiryMonth && getMonthName(sslCertInfo.expiryMonth) !== "" && sslCertInfo.expiryDay && sslCertInfo.expiryDay !== null && sslCertInfo.expiryYear && sslCertInfo.expiryYear !== null)
					{			
						var expDate = new Date(sslCertInfo.expiryYear, sslCertInfo.expiryMonth, sslCertInfo.expiryDay, 0, 0, 0, 0); 
						expDate = locale.format( expDate, {selector:"date", formatLength: "medium" } );
						title = djLang.replace(MRI["SSLCertExpiryTitle"], [productName, expDate]);
						sslMessageHandler.displayMessage("", "warning", title, fullDescription , false);
					}
				}
			}
			
			/*
				gets the localized month name given the month number. The month number corresponds to how months are ordered by javascript, starting
				at 0 and going to 11
			*/
			function getMonthName (month) {
				var monthNames = locale.getNames("months", "wide"), name = "";
				if(month < monthNames.length)
				{
					name = monthNames[month];
				}
				return name;
			}
			
			function checkMemoryLeak () {
			/*if (lastKnownWidgetCount === -1) {
				lastKnownWidgetCount = dijit.registry.toArray().length;
			} else {
				var registryArray = dijit.registry.toArray();
				if (lastKnownWidgetCount === registryArray.length || lastKnownWidgetCount > registryArray.length) {
					return;
				}
				console.info("**MEMORY LEAK CHECKER** current number of potentially leaked widgets: "
						+ (registryArray.length - lastKnownWidgetCount)
						+ ". These are listed below, pay more attention to multiple copies:");
				var idx = lastKnownWidgetCount;
				for (; idx < registryArray.length; idx++) {
					console.info(registryArray[idx]);
				}

			}*/

			//As long as events don't come in for the status pods and the finder, you can use code above to check for memory Leaks.
			//Make sure you navigate back and forth to the suspected page, so that you can get multiple copies of those leaked widgets which are very easy to spot.
			//If you just see one widget (not multiple copies) You either haven't navigated to the faulty page more than once, or it just means it is not a leak.
			}

			function processWorkArea () {
				// This is a place-holder for a yet-to-be-designed method of handling
				// security, and getting the title early at the same time.
				// A utility class like this will be required, most likely.

				if (dojoConfig.packages[3]) {
					//TODO: deprecated, but can't figure out how to dynamically add a package without a full page load, Dojo doesn't seem to refresh its list of packages
					dojo.registerModulePath(dojoConfig.packages[3].name, dojoConfig.packages[3].location);
				}
				try {
					require([ dojoConfig.workAreaDijit + "Preload"
					], function (WorkAreaPreload) {
						var workAreaPreload = new WorkAreaPreload();
					});
				} catch (e) {
					if (e.message === "xhrFailed") {
						if (_frameworkCoreMainLogger.isWarn()) {
							_frameworkCoreMainLogger
									.warn(
											dojo.config.workAreaDijit
													+ "Preload is not found; consider adding it for title to appear faster. Otherwise ignore this.",
											"mainLoader");
						}
					} else {
						if (_frameworkCoreMainLogger.isError()) {
							_frameworkCoreMainLogger.error("Could not load " + dojo.config.workAreaDijit
									+ "Preload for this reason: " + e, "mainLoader");
						}
					}
				}
				var requiredClasses = [ dojoConfig.workAreaDijit
				];

				if (dojoConfig.selector === "tree") {
					if (_frameworkCoreMainLogger.isInfo()) {
						_frameworkCoreMainLogger.info("adding a TREE selector", "mainLoader");
					}
					requiredClasses[1] = "ibm/usmi/widgets/TreeSelector";
				} else if (dojoConfig.selector === "details") {
					if (_frameworkCoreMainLogger.isInfo()) {
						_frameworkCoreMainLogger.info("adding a DETAILS selector", "mainLoader");
					}
					requiredClasses[1] = "ibm/usmi/widgets/DetailsSelector";
				} else if (dojoConfig.selector) { //class specified
					if (_frameworkCoreMainLogger.isInfo()) {
						_frameworkCoreMainLogger.info("adding a CUSTOM selector: " + dojoConfig.selector, "mainLoader");
					}
					requiredClasses[1] = dojoConfig.selector;
				} else {
					if (_frameworkCoreMainLogger.isInfo()) {
						_frameworkCoreMainLogger.info("NO selector requested!!", "mainLoader");
					}
				}
			try {
				require(requiredClasses, function (WorkAreaDijit, Selector) {

					//Clear messages
					clearMessageHandlers();
					if (selector) {
						selector.destroyRecursive();
						selector = null;
					}

					var selectorContainer = dijit.byId("selectorContainer");
					var pageLayout = dijit.byId("pageLayout");
					if (selectorContainer) {
						pageLayout.removeChild(selectorContainer);
						selectorContainer.destroyRecursive();
					}
					if (dijit.byId('selectorDiv')) {
						dijit.byId('selectorDiv').destroyRecursive();
					}

					var selectorDom = dojo.byId('selectorDiv');
					if (selectorDom) {
						selectorDom.parentNode.removeChild(selectorDom);
					}
					if (dojoConfig.selector) {
						var selectorContainer = dijit.byId("selectorContainer");
						if (!selectorContainer) {
							var selectorContainerDom = dojo.create('div', {
								id : "selectorContainer",
								'class' : "claro spacious",
								style : "border: none;"
							}, 'contentAreaLayout');
							var selectorPane = new ContentPane({
								region : 'left',
								splitter : true
							}, selectorContainerDom);
							dijit.byId("mainContentAreaLayout").addChild(selectorPane);
						}
						dojo.create('div', {
							id : "selectorDiv",
							'class' : "interiorSpacious"
						}, 'selectorContainer');
					}

					if (workAreaDijit) {
						workAreaDijit.destroyRecursive();
						if (!dojo.byId('workAreaDiv')) {
							dojo.create('div', {
								id : "workAreaDiv",
								'class' : "interiorSpacious"
							}, 'workAreaContainer');
						}
					}
					checkMemoryLeak();
					if (Selector) {
						selector = new Selector({}, "selectorDiv");
						selector.startup();
					}
					workAreaDijit = new WorkAreaDijit({}, "workAreaDiv");
					workAreaDijit.startup();
					setTimeout('dijit.byId("pageLayout").resize()', 500);
					if (!lastHash) {
						lastHash = dojo.hash();
					}
				});
				} catch (e) {
					if (e.message === "xhrFailed") {
						if (_frameworkCoreMainLogger.isWarn()) {
							_frameworkCoreMainLogger
									.warn(
											dojo.config.workAreaDijit
													+ " We were not able to located our required items for this reason: " + e,
											"mainLoader");
						}
					} else {
						if (_frameworkCoreMainLogger.isError()) {
							_frameworkCoreMainLogger.error("Could not load items for this reason: " + e, "mainLoader");
						}
					}
				}
			}

			require([ "dojo/ready"
			], function (ready) {
				ready(function () {
					if (_frameworkCoreMainLogger.isFinest()) {
						_frameworkCoreMainLogger.entering("", "require");
					}
					// parser.parse();
					myframework = new frameworkdata();
					topic.subscribe("curi-framework-ready", mainLoader);
					//don't populate the menus if we're just displaying a simple header
					if (!nguiProperties.hideBanner || !nguiProperties.hideMenuBar) {
						topic.subscribe("nextgen-ui-menubar-loaded", populateMenuBar);
						topic.subscribe("nextgen-ui-helpmenu-loaded", populateHelpMenu);
					}
					if (_frameworkCoreMainLogger.isFinest()) {
						_frameworkCoreMainLogger.exiting("", "require");
					}
				});
				if (pageLoadAssembly) {
					return;
				}
				require([ "dojo/hash"
				], function (connect, hash) {
					topic.subscribe("/dojo/hashchange", function (changedHash) {
						if ("" === changedHash) {
							// Don't parse if there's no parameters, then just create widgets.
							processWorkArea();
							return;
						}
						var i = 0, j = 0, lastDijit, currentDijit, lastToken, currentToken, lastSelectorHome;
						var currentSelectorHome, lastSelector, currentSelector, refresh = true, forceRefresh = false;
						var forceRefreshStr, currentForceRefresh = false, lastForceRefresh = false;
						var changedParms = changedHash.split('&');
						for (; j < changedParms.length; j++) {
							if (changedParms[j].indexOf('workAreaDijit') !== -1) {
								currentDijit = changedParms[j].split('=')[1].replace(/%2F/g, '/');
							}
							if (changedParms[j].indexOf('sourceToken=') !== -1) {
								currentToken = changedParms[j].split('=')[1].replace(/%2F/g, '/');
							}
							if (changedParms[j].indexOf('selectorHome=') !== -1) {
								currentSelectorHome = changedParms[j].split('=')[1].replace(/%2F/g, '/');
							}
							if (changedParms[j].indexOf('selector=') !== -1) {
								currentSelector = changedParms[j].split('=')[1].replace(/%2F/g, '/');
							}
							if (changedParms[j].indexOf('forceRefresh=') !== -1) {
								forceRefreshStr = changedParms[j].split('=')[1].replace(/%2F/g, '/');
								if (forceRefreshStr === "true") {
									forceRefresh = true;
									currentForceRefresh = true;
								} else {
									currentForceRefresh = false;
								}
							}
						}
						if (lastHash) {
							var lastParms = lastHash.split('&');
							for (; i < lastParms.length; i++) {
								if (lastParms[i].indexOf('workAreaDijit') !== -1) {
									lastDijit = lastParms[i].split('=')[1].replace(/%2F/g, '/');
								}
								if (lastParms[i].indexOf('sourceToken=') !== -1) {
									lastToken = lastParms[i].split('=')[1].replace(/%2F/g, '/');
								}
								if (lastParms[i].indexOf('selectorHome=') !== -1) {
									lastSelectorHome = lastParms[i].split('=')[1].replace(/%2F/g, '/');
								}
								if (lastParms[i].indexOf('selector=') !== -1) {
									lastSelector = lastParms[i].split('=')[1].replace(/%2F/g, '/');
								}
								if (lastParms[i].indexOf('forceRefresh=') !== -1) {
									forceRefreshStr = lastParms[i].split('=')[1].replace(/%2F/g, '/');
									if (forceRefreshStr === "true") {
										lastForceRefresh = true;
									} else {
										lastForceRefresh = false;
									}
								}
							}
							if (lastDijit) {
								if (currentDijit) {
									if (lastDijit === currentDijit) {
										if (!lastSelectorHome && !currentSelectorHome) {
											if (currentSelector
													&& lastSelector
													&& ((currentSelector === lastSelector
															&& currentSelector === 'details' && currentToken
															&& lastToken && currentToken !== lastToken))
													|| (currentSelector !== lastSelector)) {
												//Means we're either going from different selectors, or same selectors but different source token, refresh
											} else {
												refresh = false; //Means we're clicking different "drawers" on the details selector, don't refresh - widget handles it
											}
										} else if (lastSelectorHome && currentSelectorHome
												&& lastSelectorHome === currentSelectorHome) {
											refresh = false;
										}
									}
								}
							} else {
								if (!currentDijit) {
									refresh = false;
								}
							}

						} else {
							if (currentDijit) {
								if (dojoConfig.workAreaDijit === currentDijit) {
									refresh = false;
								}
							} else {
								if (dojoConfig.workAreaDijit.indexOf('ibm/usmi/dashboard/ResourceDashBoard') !== -1) {
									refresh = false;
								}
							}
						}
						if (!currentForceRefresh && lastForceRefresh) {
							//previous hash tell to forceRefresh, now forceRefresh is missed from hash, we still need to refresh it
							forceRefresh = true;
						}
						lastHash = changedHash;
						if (forceRefresh || refresh) {
							processHash(changedHash);
							processWorkArea();

						}
					});
				});

			});

		});
