/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(
    "ibm/usmi/core/utils/_UsmiBrowserPersistence", [
        "dojo/_base/lang",
        "dojo/_base/declare",
        "dojo/cookie",
        "dojox/mvc",
        "dijit/registry",
        "ibm/usmi/core/utils/Logger"
    ],
    function (lang, declare, cookie, mvc, registry, Logger) {
        "use strict";
        return declare(
            null,
            {
                // the logger
                _usmiBrowserPersistenceLogger: new Logger("ibm.usmi.core.utils", "_UsmiBrowserPersistence"),

                constructor: function () {
                    if (this._usmiBrowserPersistenceLogger.isFinest()) {
                        this._usmiBrowserPersistenceLogger.entering("", "constructor");
                    }
                    if (this._usmiBrowserPersistenceLogger.isFinest()) {
                        this._usmiBrowserPersistenceLogger.exiting("", "constructor");
                    }
                },

                //set up watchers and persistence for the model passed to us
                initPersistence: function (model) {
                    if (this._usmiBrowserPersistenceLogger.isFinest()) {
                        this._usmiBrowserPersistenceLogger.entering("", "initPersistence");
                    }
                    var modelObj = model.toPlainObject(),
                        property = null,
                        ss = null;
                    for (property in modelObj) {
                        if (modelObj.hasOwnProperty(property)) {
                            // look for saved value for model property
                            ss = this.getData(property);
                            if (ss !== null) {
                                if (this._usmiBrowserPersistenceLogger.isFiner()) {
                                    this._usmiBrowserPersistenceLogger.finer("STORAGE:  Found '" + property + "' in session storage.  Value is " + ss,
                                        "initPersistence");
                                }
                                model[property].set("value", ss);
                            }

                            // set up watcher for value changes
                            //javascripty & clean
                            model[property].watch("value", this.propertyChangeListener(property));

                            //dojo'y & dirty
                            /*model[property].watch("value",
                                lang.hitch({property: property,
                                        setData: this.setData,
                                        validateUser: this.validateUser,
                                        getSessionIdFromLocalStorage: this.getSessionIdFromLocalStorage,
                                        getSessionIdFromSessionStorage: this.getSessionIdFromSessionStorage },
                                            function (name, oldValue, newValue) {
                                                console.log(this.property + " was updated from: " + oldValue + " to: " + newValue);
                                                this.setData(this.property, newValue);
                        }));*/
                        }
                    }
                    if (this._usmiBrowserPersistenceLogger.isFinest()) {
                        this._usmiBrowserPersistenceLogger.entering("", "initPersistence");
                    }
                },

                propertyChangeListener: function (property) {
                    var myObject = this; //for scope on the callback
                    return function (name, oldValue, newValue) {
                        var newType = Object.prototype.toString.call(newValue);
                        if (myObject._usmiBrowserPersistenceLogger.isFiner()) {
                            myObject._usmiBrowserPersistenceLogger.finer("NOTIFICATION:  '" + property + "' was updated from: " + oldValue + " to: " + newValue,
                                "propertyChangeListener");
                        }
                        myObject.setData(property, newValue);
                        if (myObject._usmiBrowserPersistenceLogger.isFiner()) {
                            myObject._usmiBrowserPersistenceLogger.finer("STORAGE:  Updated session storage for '" + property + "':"  + myObject.getData(property),
                                "propertyChangeListener");
                        }
                    };
                },

                radioButtonChangeListener: function (modelEntry, domValue) {
                    return function (isChecked) {
                        if (isChecked) {
                            modelEntry.set("value", domValue);
                        }
                    };
                },

                checkboxChangeListener: function (modelEntry, domValue) {
                    return function (isChecked) {
                        modelEntry.set("value", isChecked ? domValue : "");
                    };
                },

                getSessionIdFromLocalStorage: function () {
                    return localStorage.getItem("usmiLocalStorageKey");
                },

                getSessionIdFromSessionStorage: function () {
                    return sessionStorage.getItem("usmiSessionStorageKey");
                },

                validateUser: function () {
                    var sessionIdFromLocalStorage = this.getSessionIdFromLocalStorage(),
                        sessionIdFromSessionStorage = this.getSessionIdFromSessionStorage();

                    if ((!sessionIdFromLocalStorage) ||
                            (!sessionIdFromSessionStorage) ||
                            (sessionIdFromLocalStorage !== sessionIdFromSessionStorage)) {
                        // Local storage session id is different.  The most probable
                        // cause of this is due to a logout in another window or tab.
                        // The data stored in the session is now invalid.
                        if (this._usmiBrowserPersistenceLogger.isInfo()) {
                            this._usmiBrowserPersistenceLogger.info("STORAGE:  Session storage session id DOES NOT MATCH local storage session id.",
                                "validateUser");
                            this._usmiBrowserPersistenceLogger.info("STORAGE:  Clearing session storage.",
                                "validateUser");
                        }

                        // Delete old saved data and save new session id.
                        sessionStorage.clear();
                        sessionStorage.setItem("usmiSessionStorageKey", sessionIdFromLocalStorage);
                    }
                    return true;
                },

                getFromStorage: function (storage, key, storedAsJSON) {
                    var valueFromStorage = null,
                        valueToReturn = null;
                    if (this.validateUser()) {
                        valueFromStorage = storage.getItem(key);
                        if (valueFromStorage !== null) {
                            if (storedAsJSON === false) {
                                valueToReturn = valueFromStorage;
                            } else {
                                return JSON.parse(valueFromStorage).valueData;
                            }
                        }
                    }
                    return valueToReturn;
                },

                writeToStorage: function (storage, key, value, storeAsJSON) {
                    var valueToStore = null,
                        valueType = null;
                    if (this.validateUser()) {
                        if (storeAsJSON === false) {
                            valueToStore = value;
                        } else {
                            valueType = Object.prototype.toString.call(value);
                            valueToStore = JSON.stringify({valueType: valueType, valueData: value});
                        }
                        storage.setItem(key, valueToStore);
                    }
                },

                removeFromStorage: function (storage, key) {
                    if (this.validateUser()) {
                        storage.removeItem(key);
                    }
                },

                // storedAsJSON is an optional parameter.  The default is true.
                getData: function (key, storedAsJSON) {
                    return this.getFromStorage(sessionStorage, key, storedAsJSON);
                },

                // storeAsJSON is an optional parameter.  The default is true.
                setData: function (key, value, storeAsJSON) {
                    this.writeToStorage(sessionStorage, key, value, storeAsJSON);
                },

                removeData: function (key) {
                    this.removeFromStorage(sessionStorage, key);
                },

                // storedAsJSON is an optional parameter.  The default is true.
                getGlobalData: function (key, storedAsJSON) {
                    return this.getFromStorage(localStorage, key, storedAsJSON);
                },

                // storedAsJSON is an optional parameter.  The default is true.
                setGlobalData: function (key, value, storeAsJSON) {
                    this.writeToStorage(localStorage, key, value, storeAsJSON);
                },

                removeGlobalData: function (key) {
                    this.removeFromStorage(localStorage, key);
                },

                initializeRadioButtons: function (ids, modelEntry) {
                    var i,
                        domValue;
                    for (i = 0; i < ids.length; i += 1) {
                        domValue = registry.byId(ids[i]).value;
                        if (modelEntry.get("value") === domValue) {
                            registry.byId(ids[i]).set("checked", "true");
                        } else {
                            registry.byId(ids[i]).set("checked", "");
                        }
                        registry.byId(ids[i]).on("change", this.radioButtonChangeListener(modelEntry, domValue), true);
                    }
                },

                initializeCheckbox: function (id, modelEntry) {
                    var domValue = registry.byId(id).value;
                    if (modelEntry.get("value") === domValue) {
                        registry.byId(id).set("checked", "true");
                    } else {
                        registry.byId(id).set("checked", "");
                    }

                    registry.byId(id).on("change", this.checkboxChangeListener(modelEntry, domValue), true);
                }
            }
        );
    }
);
