/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/util/DefaultDialogTask", [
	"dijit",
	"ibm/tivoli/navmodel/Trace",
	"dojo/_base/declare",
	"dojox/data/dom",
	"dojo/topic",
	"dojo/_base/lang",
	"dijit/registry",
	"ibm/usmi/util/DefaultDialog",
	"ibm/usmi/core/utils/Logger"
],

function (dijit, _Trace, declare, dom, topic, lang, registry, DefaultDialog, Logger) {

	return declare("ibm.usmi.util.DefaultDialogTask", [], {

		//		Utility for logging
		_defaultDialogLogger: new Logger("ibm.usmi.util", "DefaultDialogTask"),

		_subscriptions: [],

		constructor: function () {
			// Register for a DialogTask event
			var sub = topic.subscribe("DefaultDialogLaunch", lang.hitch(this, this._handleDialogLaunch));
			this._subscriptions.push(sub);
			sub = topic.subscribe("DefaultNoninteractiveTask", lang.hitch(this, this._handleDialogLaunch));
			this._subscriptions.push(sub);
		},
		
		destroy: function () {
		
		//clean up subscriptions
		if (this._subscriptions) {
			for (var i = 0; i < this._subscriptions.length; i++) {
				this._subscriptions[i].remove();
			}
			this._subscriptions = null;
		}
		this.inherited(arguments);
	},

		_handleDialogLaunch: function (launchParms, menuItem, context) {

			var args = {
				launchParms: launchParms,
				menuItem: menuItem,
				context: context,
				title: launchParms.dialogTitle
			};

			if ((launchParms && launchParms.dialogTitle && (launchParms.dialogTitle !== "") ) || ( menuItem && menuItem.label && (menuItem.label !== "") )) {
				//check if we need to set title to the context label
				if(!launchParms.dialogTitle || (launchParms.dialogTitle === "")){
					args.title = menuItem.label;
				}
				//ceate a new DefaultDialog in response to this event
				var defaultDialog = new DefaultDialog(args);
			} else {
				if (this._defaultDialogLogger.isError()) {
					this._defaultDialogLogger.error("Title must be specified for a task in order to launch dialog.", "_handleDialogLaunch");
				}
			}
		}

	});
});
