/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/util/DefaultUserInterfaceTask", [
	"dojo/_base/kernel",
	"dojo/_base/declare",
	"dojox/data/dom",
	"dojo/_base/lang",
	"dojo/_base/connect",
	//NextGen Imports
	"ibm/tivoli/navmodel/Trace",
	"ibm/usmi/util/DefaultMessageHandler",
	"ibm/tivoli/navmodel/Selection",
	"ibm/usmi/core/utils/Logger"

],

function (dojo, declare, dom, lang, connect, _Trace, DefaultMessageHandler, _Selection, Logger) {

	return declare([],
			{

				// private members
				_launchParms: null,
				_menuItem: null,
				_context: null,
				_trace: new _Trace("ibm.usmi.util.DefaultUserInterfaceTask"),
				//	Utility for logging
				_defaultUserInterfaceLogger: new Logger("ibm.usmi.util", "DefaultUserInterfaceTask"),

				constructor: function () {
					// summary:
					//		Constructor
					// description:
					//		Register for a URL launch task event

					// Register for a URL launch task event
					connect.subscribe("DefaultUserInterfaceTask", this, "_handleUserInterfaceTask");

				},

				_handleUserInterfaceTask: function (launchParms, menuItem, context) {
					// summary:
					//		Handles a User Interface Task 
					// description:
					//		Executes the Server task by checking the LaunchParms to see how to handle

					this._launchParms = launchParms;
					this._menuItem = menuItem;
					this._context = context;

					var runServerTask = function (tasks) {
						var task = null;

						task = this.fetchTask(tasks);

						// Call execute on the server task
						task.execute({
							onError: lang.hitch(this, this._handleExecuteOnServerError),
							selection: this.getSelection(context)
						});
					};

					// Process the user interface task
					// Check launchParms to decide how to handle
					if (launchParms) {
						if (launchParms.launchType === "UserInterfaceTask") {
							// Get the task and execute it
							context.dataset.getTasks({
								onComplete: lang.hitch(this, runServerTask),
								onError: lang.hitch(this, this.getTasksError),
								taskCategory: "menu",
								selection: this.getSelection(context)
							});
						} else {
							if (this._defaultUserInterfaceLogger.isError()) {
								this._defaultUserInterfaceLogger.error("launchParms are not valid",
										"_handleUserInterfaceTask");
							}

						}
					} else {
						if (this._defaultUserInterfaceLogger.isError()) {
							this._defaultUserInterfaceLogger.error("undefined launch parms from server task",
									"_handleUserInterfaceTask");
						}
					}
				},

				fetchTask: function (tasks) {
					// summary:
					//      Get the complete list of menu tasks, and find the one that matches the menu item that launched me, so I can use it when the user clicks OK.
					// description:
					//      This routine is recursive to handle the submenus
					var i = 0, subTasks = null, task = null;
					//Want to loop through tasks looking for match to menuItem actionID
					if (tasks) {
						for (i = 0; i < tasks.length; i++) {
							if (tasks[i].getId() === this._menuItem.actionID) {
								return tasks[i];
							} else if (tasks[i].getChildTasks()) {
								subTasks = tasks[i].getChildTasks();
								if (subTasks.length > 0) {
									task = this.fetchTask(subTasks);
									if (task && (task.getId() === this._menuItem.actionID)) {
										return task;
									}
								}
							}
						}
					}
				},

				getSelection: function (context) {
					// summary:
					//		return the selected items, links, and relationships
					var selectedThings = context.selection;
					var currentSelection = null;
					if (selectedThings) {
						currentSelection = new _Selection({
							version: context.dataset.getVersion()
						});
						currentSelection.addItems(selectedThings.getItemIds());
						currentSelection.addLinks(selectedThings.getLinkIds());
						currentSelection.addRelationships(selectedThings.getRelationshipIds());
					}
					return currentSelection;
				},

				getTasksError: function (error, request) {
					// summary:
					//		Error getting task from dataset
					//There was a problem getting the tasks from the dataset
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
				},

				_handleExecuteOnServerError: function (error, exception, ioArgs) {
					// summary:
					//		Server Error
					// description:
					//		If there was problems executing the task on the server send error

					if (this._trace.isError()) {
						this._trace.error(">>> error executing server task = " + error, "_handleExecuteOnServerError");
					}

					//There was a problem executing the task on the server
					var msgHandler = new DefaultMessageHandler({});
					msgHandler.displayException(exception);
				}
			});
});
