/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/util/Utils", [ "dojo/dom",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dojo/dom-attr",
	"dojo/dom-geometry" ], function (
	djDom, djDomC, djClass, djDomAttr, djDomG) {
	return {

		FSMFlightRecorder : {
			curSize : 0,
			startIndex : 0,
			entries : [],
			maxSize : 75,
			add : function (
				traceEntry) {
				var now, timeString;
				now = new Date();
				timeString = now.getFullYear() + "/" + now.getMonth() + 1 + "/" + now.getDate() + " " + now.getHours() +
					":" + now.getMinutes() + ":" + now.getSeconds() + "." + now.getMilliseconds();
				//var timeString = now.format("yyyy/mm/dd HH:MM:ss.l");
				if (this.curSize < this.maxSize) {
					this.entries[this.curSize] = timeString + " - " + traceEntry;
					this.curSize++;
				} else {
					this.entries[this.startIndex] = timeString + " - " + traceEntry;
					this.startIndex += 1;
					if (this.startIndex >= this.maxSize) {
						this.startIndex = 0;
					}
				}
			},
			test : function () {
				var i;
				try {
					console.info("DEBUG -- BEGIN -- TEST FLIGHT RECORDER DATA");
					for (i = 0; i < this.curSize; i++) {
						if (this.startIndex + i >= this.curSize) {
							console.info(this.entries[this.startIndex + i - this.curSize]);
						} else {
							console.info(this.entries[this.startIndex + i]);
						}
					}
					console.info("DEBUG -- END -- TEST FLIGHT RECORDER DATA");
				} catch (e) {}
			},

			toString : function (
				newLine) {
				var i, outputString = "";
				if (!newLine) {
					newLine = "<br>";
				}
				try {
					outputString = outputString +
						" -- BEGIN -- FSM SCRIPT FLIGHT RECORDER DATA (time values are from the browsers system)" +
						newLine;
					for (i = 0; i < this.curSize; i++) {
						if ((this.startIndex) + i >= this.curSize) {
							outputString = outputString + this.entries[this.startIndex + i - this.curSize] + newLine;
						} else {
							outputString = outputString + this.entries[this.startIndex + i] + newLine;
						}
					}
					outputString = outputString + " -- END -- FSM SCRIPT FLIGHT RECORDER DATA";
				} catch (e) {}
				return outputString;
			}

		},

		// --- TRACING --- 

		TraceLevel : 20, // 10 = DEBUG , 15 = FINEST, 20 = TRACE , 25 = INFO ,  30 = WARN , 40 = ERROR 

		// REMOVE ANY USE WHEN DONE
		CMDEBUG : function (
			outputText) {
			if (this.TraceLevel > 10) {
				return;
			}
			var out = "DEBUG (" + arguments.callee.caller.nom + ") : " + outputText;
			console.log(out);
			this.FSMFlightRecorder.add(out);
		},

		CMFinest : function (
			outputText) {
			if (this.TraceLevel > 15) {
				return;
			}
			var out = "FINEST (" + arguments.callee.caller.nom + ") : " + outputText;
			console.log(out);
			this.FSMFlightRecorder.add(out);
		},

		// FINE level tracing
		CMTrace : function (
			outputText) {
			if (this.TraceLevel > 20) {
				return;
			}
			var out = "TRACE (" + arguments.callee.caller.nom + ") : " + outputText;
			console.log(out);
			this.FSMFlightRecorder.add(out);
		},
		// Info level tracing
		CMInfo : function (
			outputText) {
			if (this.TraceLevel > 25) {
				return;
			}
			var out = "INFO (" + arguments.callee.caller.nom + ") : " + outputText;
			console.info(out);
			this.FSMFlightRecorder.add(out);
		},
		CMWarn : function (
			outputText) {
			if (this.TraceLevel > 30) {
				return;
			}
			var out = "WARN (" + arguments.callee.caller.nom + ") : " + outputText;
			console.warn(out);
			this.FSMFlightRecorder.add(out);
		},
		CMError : function (
			outputText) {
			if (this.TraceLevel > 40) {
				return;
			}
			var out = "ERROR (" + arguments.callee.caller.nom + ") : " + outputText;
			console.error(out);
			this.FSMFlightRecorder.add(out);
		},

		_traceTimers : [],
		CMTraceTimeInit : function (
			timerName) {
			if (this.TraceLevel > 20) {
				return;
			}
			this._traceTimers[timerName] = new Date();
		},

		CMTraceTimeDelta : function (
			timerName, message, restartTimer) {
			if (this.TraceLevel > 20) {
				return;
			}

			if (this._traceTimers[timerName]) {
				var out = "TRACE TIME (" + arguments.callee.caller.nom + ") [ " +
					((new Date()) - this._traceTimers[timerName]) + "ms ] : " + message;
				console.log(out);
				this.FSMFlightRecorder.add(out);
			}
			if (restartTimer) {
				this._traceTimers[timerName] = new Date();
			}
		},

		// make a page element visible
		showPageElement : function (
			elName) {
			var tbl = document.getElementById(elName);
			tbl.style.display = 'block'; // consider if using for text elements "inline"
		},

		// hide a page element
		hidePageElement : function (
			elName) {
			var tbl = document.getElementById(elName);
			tbl.style.display = 'none';
		},

		getProperty : function (
			item, propertyName) {
			var propValue = null, prop;
			try {
				prop = item.getProperty(propertyName);
				if (prop) {
					propValue = prop.getValue();
				}
			} catch (err) {
				this.CMError("an error occurred getting properties for " + item.getLabel() + " : " + err);
			}
			return propValue;
		},

		isChrome : function () {
			return (navigator.userAgent.toLowerCase().indexOf('chrome') > -1);
		},

		// creates and returns the element
		/* parmameters:
		 * elementType: html element type "span", "div", etc.. (DEFAULT: DIV)
		 * styleClass: "one or more style classes seperated by a space"
		 * style: object containing html style info.  Used as dojo.attr(<>,"Style",style)
		 * parentNode: append this element to...
		 * baseDimensionsOnNode: set default dimensions based on node...:
		 * innerHTML: a string that will be put within the elements opening and closing tags
		 */
		createElement : function (
			elementId, parentNode, parms) {
			var newElement = null, elementType, sizeToElementCBox;
			if (!parms) {
				parms = {};
			}
			elementType = (parms.elementType) ? parms.elementType : "div";

			if (djDom.byId(elementId)) {
				return djDom.byId(elementId);
			}
			try {
				newElement = document.createElement(elementType);
				djDomAttr.set(newElement, "id", elementId);
				if (parms.styleClass) {
					djDomAttr.set(newElement, "class", parms.styleClass);
				}
				if (parms.style) {
					djDomAttr.set(newElement, "style", parms.style);
				}
				// IE hack, we have to clone the node and do future work from the clone
				if (parentNode) {
					parentNode.appendChild(newElement.cloneNode(true));
				} else {
					newElement.cloneNode(true);
				}
				newElement = djDom.byId(elementId);
			} catch (e) {
				this.CMError("error appending new element (" + elementId + "):" + e.message);
			}

			if (!newElement) {
				return null;
			}

			if (parms.baseDimensionsOnNode) {
				sizeToElementCBox = djDomG.getContentBox(parms.baseDimensionsOnNode);
				djDomG.setContentSize(newElement, {
					w : sizeToElementCBox.w,
					h : sizeToElementCBox.h
				});
			}

			if (parms.innerHTML) {
				newElement.innerHTML = parms.innerHTML;
			}

			return newElement;
		},

		myJoin : function (
			theArray, seperator) {
			var first = true, retString = "", elementIndex;
			if (!seperator) {
				seperator = ",";
			}
			for (elementIndex in theArray) {
				if (theArray.hasOwnProperty(elementIndex)) {
					if (first === false) {
						retString += seperator;
					}
					retString += theArray[elementIndex];
					first = false;
				}
			}
			return retString;
		},

		associativeArrayLength : function (
			theArray) {
			var size = 0, elementIndex;
			for (elementIndex in theArray) {
				if (theArray.hasOwnProperty(elementIndex)) {
					size++;
				}
			}
			return size;
		},

		//nice way to attach events with params; from http://stackoverflow.com/questions/95731/why-does-an-onclick-property-set-with-setattribute-fail-to-work-in-ie
		makeEvent1Param : function (
			element, callback, param1, event) {
			function local (
				e) {
				var ret = callback(param1, e);
				return ret;
			}

			if (element.addEventListener) {
				//Mozilla
				element.addEventListener(event, local, false);
			} else if (element.attachEvent) {
				//IE
				element.attachEvent("on" + event, local);
			}
		},

		// returns true if control or shift was pressed during the mouseEvent
		isMultiSelectKeyUsed : function (
			mouseEvent) {
			if (mouseEvent.ctrlKey === true || mouseEvent.shiftKey === true) {
				return true;
			}
			return false;
		},

		isArray : function (
			someVar) {
			if (Object.prototype.toString.call(someVar) === '[object Array]') {
				return true;
			}
			return false;
		},

		/*
		 * Will create a new span element as a child of 'element' parameter containing 'text' ellipsed if necessary
		 * parms { // all optional parms
		 *  styleClass: will append this style to the new span element
		 *  ellipseAt: int index to insert the ellipse at, use for inserting an ellipse at a point within the 'text'
		 *  ellipseAfter: string to insert ellipse after, use to insert ellipse at a point within the 'text'
		 *  returnOnly: true if we just want the string
		 *  }
		 */
		insertEllipsedTextSpanIntoElement : function (
			element, text, parms) {
			var elementCount = 0, elementId, width, wi, i, txtSpan, returnStr;
			if (!parms) {
				parms = {};
			}
			elementId = element.id + "_txtSpan" + elementCount;
			while (djDom.byId(elementId)) {
				elementCount++;
				elementId = element.id + "_txtSpan" + elementCount;
			}

			txtSpan = djDomC.create("span", {
				innerHTML : text
			}, element);
			djClass.add(txtSpan, parms.styleClass);

			width = djDomG.getContentBox(element).w;

			wi = djDomG.getContentBox(txtSpan).w;
			if (wi > width) {
				i = 1;
				txtSpan.innerHTML = '';
				wi = djDomG.getContentBox(txtSpan).w;
				while (wi < (width) && i < text.length) {
					txtSpan.innerHTML = text.substr(0, i) + '...';
					i++;
					wi = djDomG.getContentBox(txtSpan).w;
				}

				if (parms.insertAfter) {
					parms.insertAt = text.indexOf(parms.insertAfter) + parms.insertAfter.length;
				}
				if (parms.insertAt) {
					txtSpan.innerHTML = text.substr(0, parms.insertAt) + "..." + text.substr(parms.insertAt, i - 2);
				} else {
					txtSpan.innerHTML = text.substr(0, (i - 2)) + '...'; // 1 less than what sent us over the size limit
				}
			}

			if (parms.returnOnly) {
				returnStr = txtSpan.innerHTML;
				djDomC.destroy(txtSpan);
				return returnStr;
			}
		},

		splitFRMDividedProperty : function (
			item, propName) {
			var i, dataArray = [], property, propList;
			property = this.getProperty(item, propName);
			if (property) {
				this.CMTrace("OVERLAY", propName + ": " + property);
				propList = property.split("-,-");
				for (i = 0; i < propList.length; i++) {
					if (propList[i] !== "") {
						this.CMTrace("OVERLAY", "Property : " + propList[i]);
						var keyValuePair = propList[i].split("=");
						if (keyValuePair[1]) {
							dataArray[keyValuePair[0]] = keyValuePair[1];
						}
					}
				}
			} else {
				this.CMTrace("OVERLAY", propName + ": was empty or not passed");
			}
			return dataArray;
		},

		unSetBadPrototypes : function () {
			// json*.js sets the below prototypes.  This breaks dojo's tooltip.  Deleting the prototype
			//   appears to fix the problem.  Note however that dynamic refresh won't work without these 
			//   defined prototypes.
			delete Object.prototype.toJSONString;
			//delete Array.prototype.toJSONString;
			//String.prototype.parseJSON = "";
		}

	};
});