/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2014 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/finder/FinderTabbedContainer", [
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/i18n!ibm/usmi/nls/frameworkResources",
	"dojo/dom-construct",
	"dojo/dom-class",
	"dijit/layout/ContentPane",
	"dijit/layout/TabContainer"
], function (declare, lang, MRI, domConstruct, domClass, ContentPane, TabContainer) {
	return declare([ TabContainer
	], {
		finderObject : null,
		contextFinder : null,
		init : function () {
			this.startup();
			var selectedTab = this.selectedChildWidget;
			domClass.remove(selectedTab.domNode, "dijitHidden");
			domClass.add(selectedTab.domNode, "dijitVisible");
			
			// It appears as though with dojo 1.9.3 there is a wrapper around our elements.  We need to set the wrapper to visible, too
			// to get it to appear
			if(selectedTab._wrapper) {
				domClass.replace(selectedTab._wrapper, "dijitVisible", "dijitHidden");
			}			
		},
		addChildrenTabs : function (currentView) {
			this.addChild(this.initializeMenu('resources', MRI.FinderResourceMenu, (currentView === 'resources')));
			this.addChild(this.initializeMenu('groups', MRI.FinderGroupMenu, (currentView === 'groups')));
			this.addChild(this.initializeMenu('tasks', MRI.FinderTaskMenu, (currentView === 'tasks')));
		},
		initializeMenu : function (type, headerText, isSelected) {
			var div, i = 0, numberOfResources = 0, resources;
			var selectedResource, resourceName = "", contextTaskMri;
			if (type === "tasks") {
				div = domConstruct.create("div", {
					id : 'findertasksContainerDiv'
				});
				if (this.contextFinder.selectedOids) {
					resources = this.contextFinder.selectedOids.split(",");
					if (resources.length === 1) {
						selectedResource = this.contextFinder.contextFinderDatastore.getItemFromProperty("id",
								this.contextFinder.selectedOids);
						if (selectedResource) {
							resourceName = "'" + selectedResource.getProperty("DisplayName").getValue() + "'";
						}
					} else {
						for (; i < resources.length; i++) {
							selectedResource = this.contextFinder.contextFinderDatastore.getItemFromProperty("id",
									resources[i]);
							if (selectedResource) {
								resourceName = lang.replace(MRI.SelectedResource, [ selectedResource.getProperty(
										"DisplayName").getValue()
								]);
								numberOfResources++;
							}
						}
						if (numberOfResources > 1) {
							resourceName = lang.replace(MRI.SelectedResources, [ numberOfResources
							]);
						}
					}

				}
				contextTaskMri = lang.replace(MRI.SelectedResourceText, [ resourceName
				]);
				var style = "color: #454545; font-size: 11pt; font-weight: bolder;"
						+ " padding-top:10px; display:block;";
				domConstruct.create("div", {
					id : "findercontexttasks_containerDiv",
					innerHTML : "<span style=\"" + style + "\">" + contextTaskMri + "</span>"
				}, div);
				domConstruct.create("div", {
					id : "findertasks_containerDiv",
					innerHTML : "<div id=\"findercontexttasks_border\" "
							+ "style=\"border-top: 1px solid #769DC0; left: -9px; "
							+ "margin-top: 10px; position: relative; width: 429px;\">"
							+ "</div> <span id=\"findertasksTitle\" style=\"" + style + "\">" + MRI.GlobalTasksText
							+ "</span>"
				}, div);
			} else {
				div = domConstruct.create("div", {
					id : 'finder' + type + "_containerDiv"
				});
			}
			var cp1 = new ContentPane({
				title : headerText,
				content : div,
				refreshOnShow : true,
				selected : isSelected
			});
			return cp1;
		},
		setTabbedTitleCounts : function (typeOfSearch, numItems) {
			var children = this.getChildren();
			var idx = 0;
			if (typeOfSearch === 'groups') {
				idx = 1;
			} else if (typeOfSearch === 'tasks' || typeOfSearch === 'contexttasks') {
				idx = 2;
			}
			//quick dirty check if we haven't already added the count (in case of tasks)
			if (children[idx].title.indexOf("(") === -1) {
				children[idx].title = children[idx].title + "(" + numItems + ")";
			} else {
				//Add context tasks count to already found global tasks count
				var splitText, initialTitle, currentNumber, sum;
				splitText = children[idx].title.split("(");
				initialTitle = splitText[0];
				currentNumber = splitText[1].split(")")[0];
				sum = numItems + parseInt(currentNumber, 10);
				children[idx].set("title", initialTitle + "(" + sum + ")");
			}
		}
	});
});
