/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define("ibm/usmi/widgets/table/DefaultItemsTableViewPersist", [
    "dojo/_base/declare",
    "dojo/_base/array",
    "dojo/_base/lang",
    "dojo/_base/json",
    "dojo/_base/unload",
    "dojo/cookie",
    "ibm/usmi/core/utils/Logger",
    "gridx/modules/Persist",
    "ibm/usmi/core/utils/_UsmiBrowserPersistence"
], function(declare, array, lang, json, unload, cookie, Logger, Persist, _UsmiBrowserPersistence) {

    return declare([
        Persist,
        _UsmiBrowserPersistence
    ], {
        // summary:
        //      Persistence module for DefaultItemsTableView
        // description:
        //      Define a custom gridx Persist module to get/put values from persistence.  

        // Override function to store a value in persistence using our unique key
        put: function(key, value, options) {
            var newkey = null;
            // Check null to ensure TWL issue 61869 does not blow up the code
            if (this.model != null) {
                newkey = key + this.model.store.declaredClass;
                // only persist if there are columns in the table...  helps avoids empty table persistence
                if (value && value.column && value.column.length > 0) {
                    this.setGlobalData(newkey, value);
                }
            }
        },

        // Override function to restore a value from persistence using our unique key
        get: function(key) {
            var newkey = null;
            // Check to get past real issue which will be fixed under 61869
            if (this.model != null) {
                newkey = key + this.model.store.declaredClass;
                return this.getGlobalData(newkey);
            }
        },

        // Override column state restore to restore column widths
        _restoreColumnState: function() {
            var t = this, grid = t.grid, col, cols = [], found = false, columns =
                    t.registerAndLoad('column', t._columnStateSaver, t);

            if (!lang.isArray(columns)) {
                return;
            }

            //1. we do not change the order of the columns as current structure, columns chooser can sort the order
            //2. we need to set the col width via the width in persist storage
            cols = array.map(this.grid._columns, function(colInGrid) {
                col = colInGrid;
                array.some(columns, function(colInPersist) {
                    if (colInPersist.id === col.id) {
                        col.declaredWidth = col.width = colInPersist.width;
                        return true;
                    }
                });
                return col;
            });

            grid.setColumns(cols);
        }

    });
});