/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */
define(
        "ibm/usmi/widgets/util/DefaultMenuProviderHelper", [
            "dojo/_base/declare",
            "dojo/topic",
            "dojo/json",
            // TWL imports
            "ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelper",
            "ibm/tivoli/tip/navwidget/topology/dijit/MenuProviderHelperBase",
            "ibm/usmi/util/DefaultMessageHandler",
            "ibm/tivoli/navmodel/Dataset",
            "ibm/usmi/core/utils/Logger"
        ],
        function(declare, topic, json, MenuProviderHelper, MenuProviderHelperBase, DefaultMessageHandler, Dataset, Logger) {
            // module:
            //      ibm/usmi/widget/util/DefaultMenuProvider
            // summary:
            //      The DefaultMenuProvider extends the TWL construct MenuProviderHelper which provides a base implementation 
            //      for the two callbacks needed to support the menu provider interface.   This allows us to augment the menu items
            //      that appear in the context menu, and the opportunity to be involved in the execution of a clicked menu item
            return declare(
                    [ MenuProviderHelper
                    ],
                    {
                        // the logger
                        _defaultMenuProviderHelperLogger: new Logger("ibm.usmi.widgets.util",
                                "DefaultMenuProviderHelper"),

                        getMenuItems: function(/*Object*/context) {
                            // Turn off properties menu item by saying selection does not support properties.
                            // We probably could have just not inherited from MenuProviderHelper since that class
                            // just sets up the properties menu item, but this seems safer in case TWL decides
                            // to add to MenuProviderHelper in the future.
                            var selectedObjects = context.selection, showPropertiesInContextMenu = context.showPropertiesInContextMenu, object, items;

                            if ((selectedObjects.getItemCount() + selectedObjects.getLinkCount() + selectedObjects
                                    .getRelationshipCount()) === 1) {

                                if (selectedObjects.getItemCount() === 1) {

                                    object = selectedObjects.getItems()[0];
                                    if (showPropertiesInContextMenu === true) {
                                        object.setSupportsProperties(true);
                                    } else {
                                        object.setSupportsProperties(false);
                                    }

                                }
                            }

                            items = this.inherited(arguments);

                            return items;
                        },

                        getMenuCommand: function(/*Object*/commandName, /*Object*/context, /*Object*/menuItem) {
                            //The getMenuCommand function offers the opportunity to be involved in the execution of a clicked menu item
                            // For our tasks, we want to launch them client side via publish, which our task utilities are
                            // listening for and will handle appropriately
                            var result = null, generalTab, launchParms;

                            // make sure we have a menu item (default action/task processing can call with just a
                            // command and no menuItem)
                            if (menuItem && menuItem.launchType === "JavaScript") {
                                launchParms = json.parse(menuItem.uri);
                                if (launchParms.topic) {
                                    topic.publish(launchParms.topic, launchParms, menuItem, context);

                                    // since we handled the action, set the result to a no-op command so the TIPContextMenu will
                                    // take no action on the returned result
                                    result = {
                                        "launchType": "NOOP"
                                    };
                                } else {
                                    if (this._defaultMenuProviderHelperLogger.isWarn()) {
                                        this._defaultMenuProviderHelperLogger
                                            .warn("topic not defined in launch parms from server task");
                                    }
                                }
                                return result;
                            } else {
                                // Note: If this is NOT one of our client-side actions, DO NOT set result and let
                                // the parent class command be returned
                                result = (result) ? result : this.inherited(arguments);
                                if (commandName === "nwmi_PropertiesNotebookDialog") {
                                    if (!this._propDialog) {
                                        if (this._defaultMenuProviderHelperLogger.isWarn()) {
                                            this._defaultMenuProviderHelperLogger
                                                .warn("property dialog is null");
                                        }
                                        return;
                                    }
                                    if (!this._propDialog._propNotebook) {
                                        if (this._defaultMenuProviderHelperLogger.isWarn()) {
                                            this._defaultMenuProviderHelperLogger
                                                .warn("property notebook is null");
                                        }
                                        return;
                                    }
                                    generalTab = this._propDialog._propNotebook
                                            .getPluggableView('GENERAL_PROPERTIES_VIEW');
                                    if (!generalTab) {
                                        if (this._defaultMenuProviderHelperLogger.isWarn()) {
                                            this._defaultMenuProviderHelperLogger
                                                .warn("property's general tab is null");
                                        }
                                        return;
                                    }
                                    if (!generalTab._propertyRendererObj) {
                                        if (this._defaultMenuProviderHelperLogger.isWarn()) {
                                            this._defaultMenuProviderHelperLogger
                                                .warn("property dialog renderer is null");
                                        }
                                        return;
                                    }
                                    if (context.daHelper) {
                                        generalTab._propertyRendererObj._daHelper = context.daHelper;
                                    }

                                }
                                return result;
                            }
                        },

                        executeDefaultAction: function(/*Object*/objectContext, /*Property*/property) {
                            var defaultTask = property.getTask(), jsonLaunchParms, launchParms, oid, url;
                            if(!defaultTask || (defaultTask.getType() !== "javascript")) {
                                return;
                            }
     
                            jsonLaunchParms = defaultTask.getLaunchJavascript();
                            launchParms = json.parse(jsonLaunchParms);
                            launchParms.selection = objectContext.selection;
                            if (launchParms.topic) {
                                if (launchParms.topic === "DefaultDojoDetailsTask") {
                                    oid = -1;
                                    if (launchParms.selection.getItemCount() === 1) {
                                        var object = launchParms.selection.getItems()[0];
                                        oid = object.getId();
                                    }
                                    url = "workAreaDijit=ibm/usmi/widgets/WorkArea";
                                    url += "&selector=details";
                                    url += "&taskSelection=" + launchParms.dijit;
                                    if (oid > 0) {
                                        url += "&sourceToken=" + oid;
                                    }
                                    launchNextGenPage(url);
                                } else {
                                    topic.publish(launchParms.topic, launchParms, defaultTask, objectContext);
                                }
                            }
                                
                            
                        }
                    });
        });
