/*
 * Licensed Materials - Property of IBM
 *
 * OCO Source Materials
 *
 * (C) Copyright IBM Corp. 2012, 2013 All Rights Reserved
 *
 * The source code for this program is not published or other-
 * wise divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 */

define("ibm/usmi/widgets/util/TextAbbr", ["dojo/_base/kernel",
        "dojo/_base/declare",
        "dijit/_WidgetBase",
        "dijit/_TemplatedMixin",
        "dijit/Tooltip",
        // NextGen imports
        "ibm/usmi/widgets/util/Utils"],
    function (dojo, declare, _WidgetBase, _TemplatedMixin, Tooltip, Utils) {

    return declare([_WidgetBase, _TemplatedMixin], {

/** copied from TPC Util.js **/


    widgetsInTemplate: true,
    templateString: '<span ></span>',

    text: "",
    maxLength: 23, //default value, can be passed in as a parameter
    _minLength: 10, //the minimum length that maxLength can be
    tooltip: null,
    textContainer: null,
    center: false,
    format: null,

    postCreate: function _postCreate(){
        this.inherited(arguments);
        if(this.maxLength < this._minLength){
            this.maxLength = this._minLength;
        }
        //setup the widget
        this._setupTextAbbr();
    },

    _setupTextAbbr: function __setupTextAbbr(){
        //destroy the tooltip if it exists
        if(this.tooltip){ this.tooltip.destroyRecursive(); }

        //if the textContainer already exists replace it with the new textContainer
        if(this.textContainer){
            var newTextContainer = this._createTextContainer(this.text);
            dojo.place(newTextContainer, this.textContainer, "replace");
            this.textContainer = newTextContainer;
        } else {
            this.textContainer = this._createTextContainer(this.text);
            this.domNode.appendChild(this.textContainer); //add node to this domNode
        }
        if(this.textContainer.connectTooltip){
           this.tooltip = new Tooltip({connectId: [this.textContainer], label: this.text}); //create tooltip
        }
    },

        _createTextContainer: function __createTextContainer(/*String*/text){
        if(text.length <= this.maxLength){
            return dojo.create("span", { innerHTML: text,
                                         "connectTooltip": false });
        } else {
            return dojo.create("span", { innerHTML: this.abbreviate(text),
                                         "connectTooltip": true,
                                         "aria-label": text});
        }
    },

    update: function(/*String*/newText, /*int*/maxLength){
        this.text = newText;
        this.maxLength = maxLength || this.maxLength;

        //setup the widget with new text
        this._setupTextAbbr();
    },

    /* Abbreviate the string */
    abbreviate: function _abbreviate(/*String*/st){
        return Utils.abbreviate(st, {min: this._minLength, max: this.maxLength, center: this.center, format: this.format});
    },

    /* Destroy the tooltip if it exists */
    destroy: function _destroy(){
        if(this.tooltip){ this.tooltip.destroyRecursive(); }
        this.inherited(arguments);
    }

});
});
