/*
 * Licensed Materials - Property of IBM
 * (C) Copyright IBM Corp. 2010, 2012 All Rights Reserved
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */

define("idx/layout/FlipCardItem", [
	"dojo/_base/kernel", // kernel.isAsync
	"dojo/_base/array", // array.forEach array.indexOf array.map
	"dojo/_base/declare", // declare
	"dojo/_base/html", // Deferred
	"dojo/_base/connect",
	"dojo/_base/event", // event.stop
	"dojo/_base/lang", // lang.mixin lang.hitch
	"dojo/_base/window",
	"dojo/_base/json",
	"dojo/_base/sniff",
	"dojo/_base/xhr",
	"dojo/_base/NodeList",
	"dojo/_base/fx",
	"dojo/fx",
	"dojo/fx/easing", 
	"dojo/dom", // attr.set
	"dojo/dom-attr", // attr.set
	"dojo/dom-class", // domClass.add domClass.contains
	"dojo/dom-style", // domStyle.set
	"dojo/dom-construct",
	"dojo/dom-geometry",
	"dojo/i18n", // i18n.getLocalization
	"dojo/keys",
	"dojo/topic", // topic.publish()
	"dojo/on",
	"dojo/window",
	"dojo/ready",
	"dojo/cache",
	"dojo/text",
	"dojo/query",
	"dojo/mouse",
	"dojo/touch",
	"dojo/cookie", 
	"dojo/json",
	"dojo/hash",
	"dojo/data/util/filter", 
	"dojo/date/locale",
	"dojo/data/ItemFileWriteStore",
	"dojo/store/Memory",
	"dojo/store/DataStore",
	
	"dijit/registry",
	"dijit/_base/wai",
	"dijit/_base/manager",	// manager.defaultDuration
	"dijit/_base/focus", // dijit.getFocus()
	"dijit/a11y",
	"dijit/focus",
	"dijit/_WidgetBase",
	"dijit/_Widget",
	"dijit/_TemplatedMixin",
	"dijit/_WidgetsInTemplateMixin",
	"dijit/_CssStateMixin",
	"dijit/Viewport",
	"dijit/form/Button",
	"dijit/form/DropDownButton",
	"dijit/form/ComboButton",
	"dijit/form/ToggleButton",
	"dijit/CheckedMenuItem",
	"dijit/PopupMenuItem",
	"dijit/_Contained",
	"dijit/_Container",
	"dijit/layout/_LayoutWidget",
	"dijit/layout/ContentPane",
	"dijit/Dialog", 
	"dijit/TitlePane", 
	"dijit/Menu", 
	"dijit/MenuItem", 
	"dijit/MenuSeparator", 
	"dijit/MenuBar", 
	"dijit/MenuBarItem", 
	"dijit/PopupMenuBarItem", 	
	
	"dojox/fx",
	"dojox/fx/flip",
	"dojox/html/_base",
	
	"idx/widget/ModalDialog",
	
	"dojo/i18n!./nls/FlipCardItem"
	
], function(kernel, array, declare, htmlUtil, connect, event, lang, winUtil, baseJson, has, 
		xhrUtil, NodeList, baseFx, coreFx, easingUtil,
		dom, domAttr, domClass, domStyle, domConstruct, domGeom, i18n, keys, topic, on, windowLib, 
		ready, cache, text, query, mouse, touch, cookie, json, hash,
		filterUtil, locale, ItemFileWriteStore, Memory, DataStore,
		registry, wai, manager, baseFocus, a11y, focus, _WidgetBase, _Widget, _TemplatedMixin, _WidgetsInTemplateMixin, _CssStateMixin, 
		Viewport, Button, DropDownButton, ComboButton, ToggleButton,  CheckedMenuItem, PopupMenuItem, 
		_Contained, _Container, _LayoutWidget, ContentPane, Dialog, TitlePane, 
		Menu, MenuItem, MenuSeparator,  MenuBar, MenuBarItem, PopupMenuBarItem,
		dojoxFx, flip, dojoxHtmlUtil, ModalDialog
	){
		
		
	kernel.experimental("idx/layout/FlipCardItem");
	
	//css3 support detector(program once)
	if(!window.supportCSS3AnimationDetected){
		var supports = (function() {
			var div = document.createElement('div');
			return function(prop) {
			 	vendors = 'Khtml O Moz Webkit'.split(' '), len = vendors.length;
				if ( prop in div.style)
					return true;
				if ('-ms-' + prop in div.style)
					return true;
				prop = prop.replace(/^[a-z]/, function(val) {
					return val.toUpperCase();
				});
				while (len--) {
					if (vendors[len] + prop in div.style) {
						return true;
					}
				}
				return false;
			};
		})();
		_supportCSS3Animation = supports('transform') && supports('transition') && supports('perspective') && supports('animation');
	}
	window.supportCSS3AnimationDetected = true;
	
	
	//platform detector(program once)
	(function(){
		if(window.platformDetected){return;}
		
		var themeMap = {
			"Holodark":"holodark",
			"Android 3":"holodark",
			"Android 4":"holodark",
			"Android":"android",
			"BlackBerry":"blackberry",
			"BB10":"blackberry",
			"iPhone":"iphone",
			"iPad":"ipad",
			"MSIE 10":"windows",
			"WindowsPhone":"windows",
			"Custom":"custom"
		}
		var dua = navigator.userAgent;
		var platform = ""; //"tablet", "phone", "mobile", ......
		if(dua.match(/(iPhone|iPod|iPad|Android|Holodark|BlackBerry|BB10|WindowsPhone)/)){
			platform += "mobile||";
			if((dua.indexOf("iPod")>=0) || (dua.indexOf("iPhone")>=0) || (dua.indexOf("WindowsPhone")>=0)){
				platform += "phone||";
			} else if (dua.indexOf("iPad")>=0) {
				platform += "tablet||";
			}
			for(var key in themeMap){
				if(dua.indexOf(key) > -1){
					var theme = themeMap[key];
					platform += theme + "||";
					// dm.currentTheme = theme;
				}
			}
		}else{
			platform += "desktop||";
		}
		
		//For IE10
		if (dua.match(/IEMobile\/10\.0/)) {
			var msViewportStyle = document.createElement("style");
			msViewportStyle.appendChild(document.createTextNode("@-ms-viewport{width:auto!important}"));
			document.getElementsByTagName("head")[0].appendChild(msViewportStyle);
		}

		has.add("desktop",(platform.indexOf("desktop")>-1));
		has.add("mobile",(platform.indexOf("mobile")>-1));
		has.add("phone",(platform.indexOf("phone")>-1));
		has.add("tablet",(platform.indexOf("tablet")>-1));
		
		//document class
		var pOriginArray = platform.split("||"), pArray = [];
		for(var i = 0; i < pOriginArray.length; i++){
			if(pOriginArray[i]){
				pArray.push(pOriginArray[i] + "Platform");
			}
		}
		document.documentElement.className += " " + pArray.join(" ");
		
		window.platformDetected = true;
	})();
	
	
	
	var _Button = declare("idx/layout/_Button", [Button], {
		postCreate: function(){
			this.inherited(arguments);
			
			domStyle.set(this.valueNode, "display", "none");
		}
	});
	
	
	var _MenuItem = declare("idx/layout/_MenuItem", [MenuItem], {
		//TODO
	});
	var _PopupMenuItem = declare("idx/layout/_PopupMenuItem", [PopupMenuItem], {
		//TODO
	});
	var _Menu = declare("idx/layout/_Menu", [Menu], {
		//TODO
	});
	
	
	var _MenuBarItem = declare("idx/layout/_MenuBarItem", [MenuBarItem], {
		postCreate: function(){
			this.inherited(arguments);
			
			domClass.add(this.domNode, "portletAction portletMenuBarAction");
			
			if(this.itemType == "icon"){
				domConstruct.empty(this.containerNode);
				this.iconNode = domConstruct.create("div",{
					className: "portletActionIconNode " + (this.itemClass||"defaultIconNode"),
					title: this.label || "Action",
					src:"",
					waiRole: "presentation"
				}, this.containerNode);
			}
		}
	});
	var _PopupMenuBarItem = declare("idx/layout/_PopupMenuBarItem", [PopupMenuBarItem], {
		//TODO
	});
	var _MenuBar = declare("idx/layout/_MenuBar", [MenuBar], {
		onItemClick: function(/*dijit/_WidgetBase*/ item, /*Event*/ evt){
			this.inherited(arguments);
			evt && event.stop(evt);
		}
	});
	
	
	var _ResizeHelper = dojo.declare("idx/layout/_ResizeHelper", [_Widget], {
		show: function(){
			domStyle.set(this.domNode, "display", "");
		},
		
		hide: function(){
			domStyle.set(this.domNode, "display", "none");
		},
		
		resize: function(/* Object */dim){
			domGeom.setMarginBox(this.domNode, dim);
		}
	});
	var _ResizeHandle = declare("idx/layout/_ResizeHandle", [_Widget, _TemplatedMixin], {
		targetId: "",
		targetContainer: null,
		resizeAxis: "xy",
		activeResize: false,
		activeResizeClass: "dojoxResizeHandleClone",
		animateSizing: true,
		animateMethod: "chain",
		animateDuration: 225,
		minHeight: 30,
		minWidth: 50,
		constrainMax: false,
		maxHeight:0,
		maxWidth:0,
		fixedAspect: false,
		intermediateChanges: false,
	
		startTopic: "/dojo/resize/start",
		endTopic:"/dojo/resize/stop",
	
		templateString: '<div dojoAttachPoint="resizeHandle" class="dojoxResizeHandle"><div></div></div>',
		
		postCreate: function(){
			this.connect(this.resizeHandle, touch.press, "_beginSizing");
			if(!this.activeResize){
				this._resizeHelper = manager.byId('dojoxGlobalResizeHelper');
				if(!this._resizeHelper){
					this._resizeHelper = new _ResizeHelper({
							id: 'dojoxGlobalResizeHelper'
					}).placeAt(winUtil.body());
					domClass.add(this._resizeHelper.domNode, this.activeResizeClass);
				}
			}else{ this.animateSizing = false; }
	
			if(!this.minSize){
				this.minSize = { w: this.minWidth, h: this.minHeight };
			}
			
			if(this.constrainMax){
				this.maxSize = { w: this.maxWidth, h: this.maxHeight }
			}
			
			this._resizeX = this._resizeY = false;
			var addClass = lang.partial(domClass.add, this.resizeHandle);
			switch(this.resizeAxis.toLowerCase()){
				case "xy" :
					this._resizeX = this._resizeY = true;
					addClass("dojoxResizeNW");
					break;
				case "x" :
					this._resizeX = true;
					addClass("dojoxResizeW");
					break;
				case "y" :
					this._resizeY = true;
					addClass("dojoxResizeN");
					break;
			}
		},
		
		_beginSizing: function(/*Event*/ e){
			if(this._isSizing){ return; }
	
			connect.publish(this.startTopic, [ this ]);
			this.targetWidget = manager.byId(this.targetId);
	
			this.targetDomNode = this.targetWidget ? this.targetWidget.domNode : dom.byId(this.targetId);
			if(this.targetContainer){ this.targetDomNode = this.targetContainer; }
			if(!this.targetDomNode){ return; }
	
			if(!this.activeResize){
				var c = domGeom.position(this.targetDomNode, true);
				this._resizeHelper.resize({l: c.x, t: c.y, w: c.w, h: c.h});
				this._resizeHelper.show();
				if(!this.isLeftToRight()){
					this._resizeHelper.startPosition = {l: c.x, t: c.y};
				}
			}
	
			this._isSizing = true;
			this.startPoint  = { x:e.clientX, y:e.clientY };
	
			var style = domStyle.getComputedStyle(this.targetDomNode), 
				borderModel = domGeom.boxModel==='border-model',
				padborder = borderModel?{w:0,h:0}:domGeom.getPadBorderExtents(this.targetDomNode, style),
				margin = domGeom.getMarginExtents(this.targetDomNode, style),
				mb;
			mb = this.startSize = { 
					w: domStyle.get(this.targetDomNode, 'width', style), 
					h: domStyle.get(this.targetDomNode, 'height', style),
					pbw: padborder.w, pbh: padborder.h,
					mw: margin.w, mh: margin.h};
			if(!this.isLeftToRight() && domStyle.get(this.targetDomNode, "position") == "absolute"){
				var p = domGeom.position(this.targetDomNode, true);
				this.startPosition = {l: p.x, t: p.y};
			}
			
			this._pconnects = [
				connect.connect(winUtil.doc, touch.move, this, "_updateSizing"),
				connect.connect(winUtil.doc, touch.release, this, "_endSizing")
			];
			
			event.stop(e);
		},
		
		_updateSizing: function(/*Event*/ e){
			if(this.activeResize){
				this._changeSizing(e);
			}else{
				var tmp = this._getNewCoords(e, 'border', this._resizeHelper.startPosition);
				if(tmp === false){ return; }
				this._resizeHelper.resize(tmp);
			}
			e.preventDefault();
		},
	
		_getNewCoords: function(/* Event */ e, /* String */ box, /* Object */startPosition){
			try{
				if(!e.clientX  || !e.clientY){ return false; }
			}catch(e){
				return false;
			}
			this._activeResizeLastEvent = e;
	
			var dx = (this.isLeftToRight()?1:-1) * (this.startPoint.x - e.clientX),
				dy = this.startPoint.y - e.clientY,
				newW = this.startSize.w - (this._resizeX ? dx : 0),
				newH = this.startSize.h - (this._resizeY ? dy : 0),
				r = this._checkConstraints(newW, newH)
			;
			
			startPosition = (startPosition || this.startPosition);
			if(startPosition && this._resizeX){
				r.l = startPosition.l + dx;
				if(r.w != newW){
					r.l += (newW - r.w);
				}
				r.t = startPosition.t;
			}
	
			switch(box){
				case 'margin':
					r.w += this.startSize.mw;
					r.h += this.startSize.mh;
				case "border":
					r.w += this.startSize.pbw;
					r.h += this.startSize.pbh;
					break;
			}
	
			return r; // Object
		},
		
		_checkConstraints: function(newW, newH){
			if(this.minSize){
				var tm = this.minSize;
				if(newW < tm.w){
					newW = tm.w;
				}
				if(newH < tm.h){
					newH = tm.h;
				}
			}
			
			if(this.constrainMax && this.maxSize){
				var ms = this.maxSize;
				if(newW > ms.w){
					newW = ms.w;
				}
				if(newH > ms.h){
					newH = ms.h;
				}
			}
			
			if(this.fixedAspect){
				var w = this.startSize.w, h = this.startSize.h,
					delta = w * newH - h * newW;
				if(delta<0){
					newW = newH * w / h;
				}else if(delta>0){
					newH = newW * h / w;
				}
			}
			
			return { w: newW, h: newH }; // Object
		},
			
		_changeSizing: function(/*Event*/ e){
			var isWidget = this.targetWidget && lang.isFunction(this.targetWidget.resize),
				tmp = this._getNewCoords(e, isWidget && 'margin');
			if(tmp === false){ return; }
	
			if(isWidget){
				this.targetWidget.resize(tmp);
			}else{
				if(this.animateSizing){
					var anim = coreFx[this.animateMethod]([
						baseFx.animateProperty({
							node: this.targetDomNode,
							properties: {
								width: { start: this.startSize.w, end: tmp.w }
							},
							duration: this.animateDuration
						}),
						baseFx.animateProperty({
							node: this.targetDomNode,
							properties: {
								height: { start: this.startSize.h, end: tmp.h }
							},
							duration: this.animateDuration
						})
					]);
					anim.play();
				}else{
					domStyle.set(this.targetDomNode,{
						width: tmp.w + "px",
						height: tmp.h + "px"
					});
				}
			}
			this.targetSize = tmp;
			if(this.intermediateChanges){
				this.onResize(e);
			}
		},
	
		_endSizing: function(/*Event*/ e){
			array.forEach(this._pconnects, connect.disconnect);
			var pub = lang.partial(connect.publish, this.endTopic, [ this ]);
			if(!this.activeResize){
				this._resizeHelper.hide();
				this._changeSizing(e);
				setTimeout(pub, this.animateDuration + 15);
			}else{
				pub();
			}
			this._isSizing = false;
			this.onResize(e);
		},
		
		_getTargetSizeAttr: function(){
			return this.targetSize;
		},
		
		_setTargetSizeAttr: function(size){
			this.targetSize = size;
		},
		
		onResize: function(e){
			var portletItemWidget = this.getParent();
			var gridContainer = portletItemWidget.gridContainer;
			if(gridContainer){
				if(gridContainer.layoutMode == "relative"){
					gridContainer.columnResizedWidth = 0;
					gridContainer.columnPreservedWidth = 0;
					var gridGeom = domGeom.position(gridContainer.domNode);
					if(portletItemWidget){
						for(i = 0; i < gridContainer._grid.length; i++){
							var column = gridContainer._grid[i];
							if(portletItemWidget.domNode.parentNode == column.node){
								column.columnWidthPercentage = (this.targetSize.w+20)*100/gridGeom.w;
								column.columnResized = true;
								gridContainer.columnResizedWidth = column.columnWidthPercentage;
							}else{
								var columnGeom = domGeom.position(column.node);
								column.columnWidthPercentage = columnGeom.w*100/gridGeom.w;
								column.columnResized = false;
								gridContainer.columnPreservedWidth += column.columnWidthPercentage;
							}
						}
						for(i = 0; i < gridContainer._grid.length; i++){
							var column = gridContainer._grid[i];
							if(column.columnResized){
								domStyle.set(column.node, {
									width: column.columnWidthPercentage + "%"
								});
							}else{
								domStyle.set(column.node, {
									width: (100-gridContainer.columnResizedWidth)*column.columnWidthPercentage/gridContainer.columnPreservedWidth + "%"
								});
							}
							array.forEach(column.node.childNodes, function(itemNode){
								domStyle.set(itemNode, {
									width: "auto"
								});
							}, this);
						}
						
						gridContainer.resize();
					}
				}else if(gridContainer.layoutMode == "absolute"){
					//TODO
				}else if(gridContainer.layoutMode == "floating"){
					//TODO
				}else{
					//TODO
				}
			}
			
		}
	});
	
	
	var _PortletSettings = declare("idx/layout/_PortletSettings", [ContentPane], {
		portletIconClass: "dojoxPortletSettingsIcon",
		portletIconHoverClass: "dojoxPortletSettingsIconHover",
	
		postCreate: function(){
			this.inherited(arguments);
			
			domStyle.set(this.domNode, "display", "none");
			domClass.add(this.domNode, "dojoxPortletSettingsContainer");
	
			domClass.remove(this.domNode, "dijitContentPane");
		},
	
		_setPortletAttr: function(portlet){
			this.portlet = portlet;
		},
	
		toggle: function(){
			var n = this.domNode;
			if(domStyle.get(n, "display") == "none"){
				domStyle.set(n,{
					"display": "block",
					"height": "1px",
					"width": "auto"
				});
				dojoxFx.wipeIn({
					node: n
				}).play();
			}else{
				dojoxFx.wipeOut({
					node: n,
					onEnd: lang.hitch(this, function(){
						domStyle.set(n,{"display": "none", "height": "", "width":""});
					}
				)}).play();
			}
		}
	});
	
	var _PortletDialogSettings = declare("idx/layout/_PortletDialogSettings", [_PortletSettings], {
		dimensions: null,

		constructor: function(props, node){
			this.dimensions = props.dimensions || [300, 200];
		},

		toggle: function(){
			if(!this.dialog){
				this.dialog = new ModalDialog({
					type: "information",
					text: this.title || "Settings",
					info: this.get("content") || this.domNode.innerHTML.toString(),
					parentWidget: this
				});

				winUtil.body().appendChild(this.dialog.domNode);

				// Move this widget inside the dialog
				// this.dialog.containerNode.appendChild(this.domNode);

				domStyle.set(this.dialog.domNode,{
					"width" : this.dimensions[0] + "px",
					"height" : this.dimensions[1] + "px"
				});
				// domStyle.set(this.domNode, "display", "");
			}
			if(this.dialog.open){
				this.dialog.hide();
			}else{
				this.dialog.show(this.domNode);
			}
		}
	});
	
	var _Portlet = declare("idx/layout/_Portlet", [TitlePane, _Container], {
		
		resizeChildren: true,
		closable: true,
		_parents: null,
		_size: null,
		dragRestriction : false,
		
		//contentpane for script
		adjustPaths: false,
		cleanContent: false,
		renderStyles: false,
		executeScripts: true,
		scriptHasHooks: false,
		ioMethod: xhrUtil.get,
		ioArgs: {},
		
		preventCache: true,
		preload: true,
		
		destroySettingsWidget: true,
		
		onExecError: function(/*Event*/ e){
		},
	
		_setContent: function(cont){
			this.destroySettingsWidget = false;
			var setter = this._contentSetter;
			if(! (setter && setter instanceof dojoxHtmlUtil._ContentSetter)) {
				setter = this._contentSetter = new dojoxHtmlUtil._ContentSetter({
					node: this.containerNode,
					_onError: lang.hitch(this, this._onError),
					onContentError: lang.hitch(this, function(e){
						var errMess = this.onContentError(e);
						try{
							this.containerNode.innerHTML = errMess;
						}catch(e){
							console.error('Fatal '+this.id+' could not change content due to '+e.message, e);
						}
					})
				});
			};
	
			this._contentSetterParams = {
				adjustPaths: Boolean(this.adjustPaths && (this.href||this.referencePath)),
				referencePath: this.href || this.referencePath,
				renderStyles: this.renderStyles,
				executeScripts: this.executeScripts,
				scriptHasHooks: this.scriptHasHooks,
				scriptHookReplacement: "dijit.byId('"+this.id+"')"
			};
	
			this.inherited("_setContent", arguments);
		},

		buildRendering: function(){
			this.inherited(arguments);
			domStyle.set(this.domNode, "visibility", "hidden");
		},

		postCreate: function(){
			this.inherited(arguments);
			
			this.commonActionNodeList = [];
			
			this.cardActions = this.cardActions || [];
			
			// Add the portlet classes
			domClass.add(this.domNode, "dojoxPortlet");
			domClass.remove(this.arrowNode, "dijitArrowNode");
			domClass.add(this.arrowNode, "dojoxPortletIcon dojoxArrowDown");
			domClass.add(this.titleBarNode, "dojoxPortletTitle css3Animations");
			if(_supportCSS3Animation){
				domClass.add(this.titleBarNode, "css3Animations");
			}
			domClass.add(this.focusNode, "dijitTitlePaneTitleFocusNode");
			domClass.add(this.hideNode, "dojoxPortletContentOuter");

			domClass.add(this.domNode, "dojoxPortlet-" + (!this.dragRestriction ? "movable" : "nonmovable"));

			var _this = this;
			if(this.resizeChildren){

				this.subscribe("/dnd/drop", function(){_this._updateSize();});

				this.subscribe("/Portlet/sizechange", function(widget){_this.onSizeChange(widget);});
				this.connect(window, "onresize", function(){_this._updateSize();});

				var doSelectSubscribe = lang.hitch(this, function(id, lastId){
					var widget = registry.byId(id);
					if(widget.selectChild){
						var s = this.subscribe(id + "-selectChild", function(child){
							var n = _this.domNode.parentNode;

							while(n){
								if(n == child.domNode){

									// Only fire this once, as the widget is now visible
									// at least once, so child measurements should be accurate.
									_this.unsubscribe(s);
									_this._updateSize();
									break;
								}
								n = n.parentNode;
							}
						});

						var child = registry.byId(lastId);
						if(widget && child){
							_this._parents.push({parent: widget, child: child});
						}
					}
				});
				var lastId;
				this._parents = [];

				for(var p = this.domNode.parentNode; p != null; p = p.parentNode){
					var id = p.getAttribute ? p.getAttribute("widgetId") : null;
					if(id){
						doSelectSubscribe(id, lastId);
						lastId = id;
					}
				}
			}

			if(this.titleHidden){
				domStyle.set(this.titleNode, "display", "none");
				domStyle.set(this.hideNode, "background", "none");
			}
			
			this.connect(this.titleBarNode, touch.press, function(evt){
				if (domClass.contains(evt.target, "dojoxPortletIcon")) {
					event.stop(evt);
					return false;
				}
				return true;
			});
			
			//title focus bar back for card actions
			this.focusNodeBack = domConstruct.create("div", {
				className: "dijitTitlePaneTitleFocusNode dijitTitlePaneTitleFocusBack"
			}, this.focusNode, "after");

			this.connect(this._wipeOut, "onEnd", function(){_this._publish();});
			this.connect(this._wipeIn, "onEnd", function(){_this._publish();});

			if(this.closable){
				this.closeIcon = this._createIcon("dojoxCloseNode", "dojoxCloseNodeHover", lang.hitch(this, "onClose"), "Close");
				domStyle.set(this.closeIcon, "display", "");
				domClass.add(this.closeIcon, "portletAction portletContentAction");
				this.commonActionNodeList.push(this.closeIcon);
			}
			
		},
		
		addCardAction: function(action){
			if(!this.parentPortletItem){
				this.parentPortletItem = this.getParent();
			}
			var customizedActionNode = this._createIcon(action.normalClass || "fcCustomActionNode", 
					action.hoverClass || "fcCustomActionNodeHover", lang.hitch(this, action.pressHandler), action.title);
			// domClass.toggle(customizedActionNode, "animationHidden", !this.parentPortletItem.actionDisplayed);
			domClass.add(customizedActionNode, "portletAction portletContentAction");
			this.commonActionNodeList.push(customizedActionNode);
			return customizedActionNode;
		},
		
		buildCardActionBar: function(){
			if(this.cardActions.length){
				this.cardActionBar = new _MenuBar({
					//TODO
				}).placeAt(this.focusNodeBack);
				array.forEach(this.cardActions, function(cardAction){
					var actionType = cardAction.type || "text";
					if(cardAction.items && cardAction.items.length){
						var rootMenu = new _Menu({});
						array.forEach(cardAction.items, function(item){
							var menuItem = new _MenuItem({
								label:item.label
							});
							on(menuItem, touch.press, lang.hitch(this, function(it, evt){
								if(it.pressHandler && lang.isFunction(it.pressHandler)){
									it.pressHandler.apply(this, arguments);
								}
								evt && event.stop(evt);
							}, item));
							rootMenu.addChild(menuItem);
						}, this);
						var popupActionBarItem = new _PopupMenuBarItem({
							label: cardAction.label,
							itemType: actionType,
							itemClass: cardAction.actionClass,
							popup: rootMenu
						});
						this.commonActionNodeList.push(popupActionBarItem.domNode);
						this.cardActionBar.addChild(popupActionBarItem);
					}else{
						var actionBarItem = new _MenuBarItem({
							label: cardAction.label,
							itemType: actionType,
							itemClass: cardAction.actionClass
						});
						on(actionBarItem, touch.press, lang.hitch(this, function(ca, evt){
							if(ca.pressHandler && lang.isFunction(ca.pressHandler)){
								ca.pressHandler.apply(this, arguments);
							}
							evt && event.stop(evt);
						}, cardAction));
						this.commonActionNodeList.push(actionBarItem.domNode);
						this.cardActionBar.addChild(actionBarItem);
					}
				}, this);
				
				this.cardActionBar.startup();
			}
		},

		startup: function(){
			if(this._started){return;}
			
			//action bar and actions
			this.buildCardActionBar();

			var children = this.getChildren();
			this._placeSettingsWidgets();

			// Start up the children
			array.forEach(children, function(child){
				try{
					if(!child.started && !child._started){
						child.startup()
					}
				}
				catch(e){
					console.log(this.id + ":" + this.declaredClass, e);
				}
			});

			this.inherited(arguments);
			
			//this._updateSize();
			domStyle.set(this.domNode, "visibility", "visible");
			
			//portlet communication 
			this.parentContainer = this.getParent();
			if(this.parentContainer){
				this.topicHead = (this.parentContainer.rootContainer ? (this.parentContainer.rootContainer.get("flipCardModelId") + "_") : "rootId_")
					 + (this.parentContainer.gridContainer ? this.parentContainer.gridContainer.get("gridId") : "gridId");
				this.topicId = this.parentContainer.get("itemName") + "_" + this.contentType;
				this.topicHandler = topic.subscribe(this.topicHead + "_" + this.topicId, lang.hitch(this, this.topicHandlerStub));
			}
			
			this._started = true;
		},
		
		switchCardSkin: function(skin){
			if(!this.parentPortletItem){
				this.parentPortletItem = this.getParent();
			}
			domClass.toggle(this.parentPortletItem.domNode, skin || "defaultSkin");
		},
		
		topicHandlerStub: function(data){
			//TODO 
			this.topicProcess(data.data);
		},
		topicProcess: function(data){
			//Stub function
		},
		
		topicPublisherStub: function(data, cardId, contentType){
			topic.publish(this.topicHead + "_" + cardId + "_" + (contentType||"main"), { data: data });
		},

		_placeSettingsWidgets: function(){
			array.forEach(this.getChildren(), lang.hitch(this, function(child){
				if(child.portletIconClass && child.toggle && !child.get("portlet")){
					this.settingsIcon = this._createIcon(child.portletIconClass, child.portletIconHoverClass, lang.hitch(child, "toggle"), "Portlet Settings");
					domClass.add(this.settingsIcon, "portletAction portletContentAction");
					this.commonActionNodeList.push(this.settingsIcon);
					domConstruct.place(child.domNode, this.containerNode, "before");
					child.set("portlet", this);
					this._settingsWidget = child;
				}
			}));
		},

		_createIcon: function(clazz, hoverClazz, fn, desc){

			var icon = domConstruct.create("div",{
				"class": "dojoxPortletIcon " + clazz,
				"title": desc || "Action",
				"waiRole": "presentation"
			});
			domConstruct.place(icon, this.focusNodeBack);

			//this.connect(icon, "onclick", fn);
			this.own(on(icon, touch.press, lang.hitch(this, function(e){
				fn.call();
				e && event.stop(e);
			})));

			if(hoverClazz){
				this.connect(icon, touch.over, function(){
					domClass.add(icon, hoverClazz);
				});
				this.connect(icon, touch.out, function(){
					domClass.remove(icon, hoverClazz);
				});
			}
			return icon;
		},
		
		refreshCard: function(){
			if(this.content){
				this.set("content", this.get("content"));
			}else if(this.href){
				this.set("href", this.get("href"));
			}
		},
		
		onClose: function(evt){
			domStyle.set(this.domNode, "display", "none");
		},

		onSizeChange: function(widget){
			if(widget == this){
				return;
			}
			this._updateSize();
		},

		_updateSize: function(){
			if(!this.open || !this._started || !this.resizeChildren){
				return;
			}

			if(this._timer){
				clearTimeout(this._timer);
			}
			this._timer = setTimeout(lang.hitch(this, function(){
				var size ={
					w: domStyle.get(this.domNode, "width"),
					h: domStyle.get(this.domNode, "height")
				};

				for(var i = 0; i < this._parents.length; i++){
					var p = this._parents[i];
					var sel = p.parent.selectedChildWidget
					if(sel && sel != p.child){
						return;
					}
				}

				if(this._size){
					if(this._size.w == size.w && this._size.h == size.h){
						return;
					}
				}
				this._size = size;

				var fns = ["resize", "layout"];
				this._timer = null;
				var kids = this.getChildren();

				array.forEach(kids, function(child){
					for(var i = 0; i < fns.length; i++){
						if(lang.isFunction(child[fns[i]])){
							try{
								child[fns[i]]();
							} catch(e){
								console.log(e);
							}
							break;
						}
					}
				});
				this.onUpdateSize();
			}), 100);
		},

		onUpdateSize: function(){
		},

		_publish: function(){
			topic.publish("/Portlet/sizechange",[this]);
		},

		_onTitleClick: function(evt){
			if(evt.target == this.arrowNode){
				this.inherited(arguments);
			}
		},

		addChild: function(child){
			this._size = null;
			this.inherited(arguments);

			if(this._started){
				this._placeSettingsWidgets();
				this._updateSize();
			}
			if(this._started && !child.started && !child._started){
				child.startup();
			}
		},

		destroyDescendants: function(/*Boolean*/ preserveDom){
			this.inherited(arguments);
			if(this._settingsWidget && this.destroySettingsWidget){
				this._settingsWidget.destroyRecursive(preserveDom);
			}
		},

		destroy: function(){
			if(this._timer){
				clearTimeout(this._timer);
			}
			this.inherited(arguments);
		},

		_setCss: function(){
			this.inherited(arguments);
			domStyle.set(this.arrowNode, "display", this.toggleable ? "":"none");
		}
	});
	
	
	
	// module:
	//		idx/layout/FlipCardItem
	// summary:
	//		A resizable, flippable, range-bound container with customizable actions

	/**
	* @name idx.layout.FlipCardItem
	* @class A resizable, flippable, range-bound container with customizable actions
	* @augments dijit.layout.ContentPane
	* @augments dijit._CssStateMixin
	*/ 
	
	
	return declare("idx/layout/FlipCardItem", [ContentPane, _CssStateMixin], {
		/**@lends idx.layout.FlipCardItem*/ 
		// summary:
		//		A validating, serializable, range-bound date text box with a drop down calendar
		//
		//		Example:
		// |	new FlipCardItem({maxable: true,flipable:true})
		//
		//		Example:
		// |	<div data-dojo-type='idx.layout.FlipCardItem' data-dojo-props='maxable:true,flipable:true'></div>
		
		
		// flipable: Boolean
		//		whether the card widget can be flipped 
		flipable: true,
		
		// flipableAction: Boolean
		//		whether to have the flip action iten in the card's action bar
		flipableAction: false,
		
		// itemResizable: Boolean
		//		whether the card widget can be resized 
		itemResizable: true,
		
		// settingsAct: Boolean
		//		whether the card widget can have actions
		settingsAct: true,
		
		// minable: Boolean
		//		whether the card widget can be minimized 
		minable: true,
		
		// maxable: Boolean
		//		whether the card widget can be maximized 
		maxable: false,
		
		// stackable: Boolean
		//		whether the card widget can be hided 
		stackable: false,
		
		/** @ignore */
		postMixInProperties: function(){
			this.inherited(arguments);
			
			this._nlsResources = i18n.getLocalization("idx/layout", "FlipCardItem");
		},
		
		/** @ignore */
		postCreate: function(){
			this.inherited(arguments);
			
			this.commonActionNodeList = [];
			
			this.itemMode = "main", //"main", "detail"
			this.itemStatus = "normal", //"max", "min", "normal"
			
			this.actionDisplayed = false;
			this.actionDisplayedInMain = false;
		},
		
		/**
		 * show the actions.
		 * @param {Boolean} show
		 */
		displayActions: function(show, e){
			
			if(show === undefined){
				this.actionDisplayed = !this.actionDisplayed;
			}else{
				this.actionDisplayed = show;
			}
			if(this.itemMode == "main"){
				this.actionDisplayedInMain = this.actionDisplayed;
			}
			
			// domClass.toggle(this[this.itemMode+"Content"].titleBarNode, "settings", this.actionDisplayed);
			domClass.toggle(this["mainContent"].titleBarNode, "settings", this.actionDisplayed);
			domClass.toggle(this["detailContent"].titleBarNode, "settings", this.actionDisplayed);
			
			array.forEach(this.commonActionNodeList, function(actionItem){
				domClass.toggle(actionItem, "actionsDetail", this.actionDisplayed);
			}, this);
			
			domClass.toggle(this.settingsActNode, "mainActDetail", this.actionDisplayed);
			
			//transform animation extra effect
			domClass.add(this.domNode, "contentTitleAnimating");
			setTimeout(lang.hitch(this, function(){
				domClass.remove(this.domNode, "contentTitleAnimating");
			}),1000);
			
			e && event.stop(e);
		},
		
		startup: function(){
			if(this._started){return;}
			
			this.inherited(arguments);
			
			//create maximum node
			if(this.settingsAct){
				this.settingsActBtn = new _Button({
					showLabel: false,
					title: this._nlsResources.settingsActionTitle,
					iconClass: "actionIcon settingActIcon"
				}).placeAt(this.domNode);
				domClass.add(this.settingsActBtn.domNode, "portletItemSettingsNode portletAction portletItemAction portletItemActionMain");
				this.own(on(this.settingsActBtn, touch.press, lang.hitch(this, this.handleSettings)));
				this.settingsActNode = this.settingsActBtn.domNode;
				
				this.settingsActDetailBtn = new _Button({
					showLabel: false,
					title: this._nlsResources.settingsActionDetailTitle,
					iconClass: "actionIcon settingActDetailIcon"
				}).placeAt(this.domNode);
				domClass.add(this.settingsActDetailBtn.domNode, "portletItemSettingsNodeDetail portletAction portletItemAction");
				this.own(on(this.settingsActDetailBtn, touch.press, lang.hitch(this, this.handleSettings)));
				this.settingsActDetailNode = this.settingsActDetailBtn.domNode;
				this.commonActionNodeList.push(this.settingsActDetailNode);
			}
			
			if(this.gridContainer || (this.getParent() && this.getParent().declaredClass == "idx/layout/FlipCardGridContainer")){
				if(this.maxable){
					this.maximumBtn = new _Button({
						showLabel: false,
						title: this._nlsResources.maxActionTitle,
						iconClass: "actionIcon maximumIcon"
					}).placeAt(this.domNode);
					domClass.add(this.maximumBtn.domNode, "portletItemMaxNode portletAction portletItemAction");
					this.own(on(this.maximumBtn, touch.press, lang.hitch(this, this.handleMaximum)));
					this.maximumNode = this.maximumBtn.domNode;
					this.commonActionNodeList.push(this.maximumNode);
				}
				if(this.stackable){
					this.stackBtn = new _Button({
						showLabel: false,
						title: this._nlsResources.stackActionTitle,
						iconClass: "actionIcon stackIcon"
					}).placeAt(this.domNode);
					domClass.add(this.stackBtn.domNode, "portletItemStackNode portletAction portletItemAction");
					this.own(on(this.stackBtn, touch.press, lang.hitch(this, this.handleStack)));
					this.stackNode = this.stackBtn.domNode;
					this.commonActionNodeList.push(this.stackNode);
				}
				
				//dnd handler
				this.dndNode = domConstruct.create("div", {
					className: "dndBarNode"
				}, this.domNode);
			}
			
			if(this.minable){
				this.minimumBtn = new _Button({
					showLabel: false,
					title: this._nlsResources.minActionTitle,
					iconClass: "actionIcon minimumIcon"
				}).placeAt(this.domNode);
				domClass.add(this.minimumBtn.domNode, "portletItemMinNode portletAction portletItemAction");
				this.own(on(this.minimumBtn, touch.press, lang.hitch(this, this.handleMinimum)));
				this.minimumNode = this.minimumBtn.domNode;
				this.commonActionNodeList.push(this.minimumNode);
			}
			
			if(this.flipable && this.flipableAction){
				this.flipBtn = new _Button({
					showLabel: false,
					title: "Click to Flip",
					iconClass: "actionIcon flipIcon"
				}).placeAt(this.domNode);
				domClass.add(this.flipBtn.domNode, "portletItemFlipNode portletAction portletItemAction");
				this.own(on(this.flipBtn, touch.press, lang.hitch(this, this.handleFlip)));
				this.flipNode = this.flipBtn.domNode;
				this.commonActionNodeList.push(this.flipNode);
			}
			
			array.forEach(this.commonActionNodeList, function(node, index){
				domStyle.set(node, {
					right: 5 + (20*index) + "px"
				});
			}, this);
			
			
			//main and detail portlet content
			domClass.add(this.domNode, "portletItem");
			if(_supportCSS3Animation){
				domClass.add(this.domNode, "css3Animations");
			}
			
			var mainProps = lang.mixin({}, {
				contentType: "main",
				toggleable: false,
				closable: false
			}, this.main_props);
			var mainContent = new _Portlet(mainProps);
			domClass.add(mainContent.domNode, "mainContent");
			var detailProps = lang.mixin({}, {
				contentType: "detail",
				toggleable: false,
				closable: false
			}, this.detail_props);
			var detailContent = new _Portlet(detailProps);
			domClass.add(detailContent.domNode, "detailContent");
			
			
			//portlet settings
			if(this.settings && this.flipCardModel != "view"){
				var ms = this.settings.main_settings, ds = this.settings.detail_settings;
				if(ms){
					var type = ms.type || "normal";
					var settingsWidget = (type == "normal" ? _PortletSettings : _PortletDialogSettings);
					var settingsWidgetInstance = new settingsWidget({
						title: ms.title || "",
						content: ms.content || ""
					});
					mainContent.addChild(settingsWidgetInstance);
				}
				if(ds){
					var type = ds.type || "normal";
					var settingsWidget = (type == "normal" ? _PortletSettings : _PortletDialogSettings);
					var settingsWidgetInstance = new settingsWidget({
						title: ds.title || "",
						content: ds.content || ""
					});
					detailContent.addChild(settingsWidgetInstance);
				}
			}
			
			if(_supportCSS3Animation){
				domClass.add(mainContent.domNode, "css3Animations");
				domClass.add(detailContent.domNode, "css3Animations");
			}else{
				domStyle.set(detailContent.domNode, "display", "none");
			}
			
			this.addChild(mainContent);
			this.mainContent = mainContent;
			this.addChild(detailContent);
			this.detailContent = detailContent;
			
			
			//resizer
			if(this.itemResizable){
				this.resizer = new _ResizeHandle({
					targetContainer: this.containerNode,
					activeResize: true,
					resizeAxis: "xy"
				});
				this.own(on(this.resizer, "resize", lang.hitch(this, function(evt){
					this.onResizeHandleEnd(evt, this.resizer.get("targetSize"));
				})));
				domClass.add(this.resizer.domNode, "contentResizer");
				if(_supportCSS3Animation){
					domClass.add(this.resizer.domNode, "css3Animations");
				}
				this.addChild(this.resizer);
			}
			
			// this.displayActions(false);
			
			this._started = true;
		},
		
		onResizeHandleEnd: function(e){
			//Stub function 
		},
		
		
		/**
		 * flip the card widget.
		 */
		processFlip: function(e){
			//Stub for manually handle flip
			this.handleFlip(e);
		},
		
		/**
		 * handle the hidden action.
		 */
		handleStack: function(e){
			var parentGrid = this.gridContainer || this.getParent();
			if(!parentGrid){
				return;
			}
			if(this.itemStatus == "normal"){
				if(this.stackable && parentGrid.dockContainer){
					parentGrid.stackCardItem(this);
				}
			}else{
				alert(this._nlsResources.statusIssueMessage);
			}
			e && event.stop(e);
		},
		
		/**
		 * hide the card widget.
		 */		
		stackItem: function(pItem){
			pItem = pItem || this;
			domClass.add(pItem.domNode, "portletItemDisappear");
			this.displayActions(false);
			this.itemStatus = "stacked";
		},
		
		/**
		 * show the card widget when in hidden.
		 */
		unStackItem: function(pItem){
			pItem = pItem || this;
			domClass.remove(pItem.domNode, "portletItemDisappear");
			this.itemStatus = "normal";
		},
		
		handleSettings: function(e){
			if(this.itemStatus == "min"){
				this.handleMinimum(e);
			}else{
				this.displayActions(undefined, e);
			}
		},
		
		handleMaximum: function(e){
			var parentGrid = this.gridContainer || this.getParent();
			if(parentGrid){
				//append CSS3 animation for max/min resize
				if(_supportCSS3Animation){
					clearTimeout(this.clearResizeCSS3Anim);
					domClass.add(parentGrid.domNode, "css3AnimationsForResize");
					this.clearResizeCSS3Anim = setTimeout(lang.hitch(this, function(){
						domClass.remove(parentGrid.domNode, "css3AnimationsForResize");
						this.resize();
					}), 1000);
				}
				
				if(this.itemStatus == "normal"){
					this.itemGeom = domGeom.position(this.domNode);
					// this.itemGeom = {
						// width: domStyle.get(this.domNode, "width") + "px",
						// height: domStyle.get(this.domNode, "height") + "px"
					// }
					parentGrid.disableDnd();
					domClass.add(parentGrid.domNode, "gridContainerMaximum");
					query("> .gridContainerZone", parentGrid.gridNode).forEach(lang.hitch(this, function(tdNode, index){
						if(this.column!=index){
							domClass.add(tdNode, "gridColumnDisappear");
						}else{
							domClass.add(tdNode, "gridColumnMaximum");
						}
						query("> .portletItem", tdNode).forEach(lang.hitch(this, function(portletNode){
							var portletItemWidget = registry.byNode(portletNode);
							if(portletItemWidget != this){
								//add to minimum dock bar
								if(portletItemWidget.stackable && parentGrid.dockContainer){
									parentGrid.stackCardItem(portletItemWidget, true);
								}
							}
						}));
					}));
					
					var gridGeom = domGeom.position(parentGrid.domNode);
					this.resize({w: gridGeom.w-10, h: gridGeom.h-10});
					// domStyle.set(this.domNode, {
						// width: "99%",
						// height: "93%"
					// });
					
					if(parentGrid.layoutMode == "absolute"){
						domStyle.set(this.domNode, {
							left: 0 + "px",
							top: 0 + "px"
						});
					}
					
					domClass.add(this.domNode, "portletItemMaximum");
					
					//title bar 
					// this.displayActions(false);
					
					this.itemStatus = "max";
				}else if(this.itemStatus == "max"){
					query("> .gridContainerZone", parentGrid.gridNode).forEach(lang.hitch(this, function(tdNode, index){
						if(this.column!=index){
							domClass.remove(tdNode, "gridColumnDisappear");
						}else{
							domClass.remove(tdNode, "gridColumnMaximum");
						}
						query("> .portletItem", tdNode).forEach(lang.hitch(this, function(portletNode){
							var portletItemWidget = registry.byNode(portletNode);
							if(portletItemWidget != this){
								//clear minimum dock bar
								if(portletItemWidget.stackable && parentGrid.dockContainer){
									parentGrid.unStackCardItem(portletItemWidget, true);
								}
							}
						}));
					}));
					
					domClass.remove(parentGrid.domNode, "gridContainerMaximum");
					
					domClass.remove(this.domNode, "portletItemMaximum");
					// domGeom.setMarginBox(this.domNode, this.itemGeom);
					domStyle.set(this.domNode, {
						width: this.itemGeom.w + "px",
						height: this.itemGeom.h + "px"
					});
					
					if(parentGrid.layoutMode == "absolute"){
						domStyle.set(this.domNode, {
							left: this.itemGeom.x + "px",
							top: this.itemGeom.y + "px"
						});
					}
					
					parentGrid.enableDnd();
					
					this.itemStatus = "normal";
				}else{
					alert(this._nlsResources.statusIssueMessage);
				}
			}
			e && event.stop(e);
		},
		
		handleMinimum: function(e){
			var parentGrid = this.gridContainer || this.getParent();
			
			if(_supportCSS3Animation){
				clearTimeout(this.clearResizeCSS3Anim);
				domClass.add(this.domNode, "css3AnimationsForResize");
				this.clearResizeCSS3Anim = setTimeout(lang.hitch(this, function(){
					domClass.remove(this.domNode, "css3AnimationsForResize");
					this.resize();
				}), 1000);
			}
			
			if(parentGrid){
				parentGrid.disableDnd();
			}
			
			if(this.itemStatus == "normal"){
				domClass.add(this.domNode, "portletItemMinimum");
				this.displayActions(false);
				this.itemStatus = "min";
			}else if(this.itemStatus == "min"){
				domClass.remove(this.domNode, "portletItemMinimum");
				this.itemStatus = "normal";
			}else{
				alert(this._nlsResources.statusIssueMessage);
			}
			
			if(parentGrid){
				parentGrid.enableDnd();
			}
			
			e && event.stop(e);
		},
		
		handleFlip: function(e){
			if(!this.flipable){return;}
			if(this.animating){
				//|| (e && e.target && !dom.isDescendant(e.target, this[this.itemMode+"Content"].titleBarNode))){
				return;
			}
			this.animating = true;
			if(_supportCSS3Animation){ //css3 supported
				clearTimeout(this.clearFlipCSS3Anim);
				domClass.add(this.domNode, "css3AnimationsFlipping");
				this.clearFlipCSS3Anim = setTimeout(lang.hitch(this, function(){
					domClass.remove(this.domNode, "css3AnimationsFlipping");
				}), 1000);
				if(this.itemMode == "main"){
					domClass.add(this.domNode, "portletItemFlip");
					this.resizer && domClass.add(this.resizer.domNode, "contentResizerFlip");
					
					this.animating = false;
					this.itemMode = "detail";
					this.displayActions(false);
				}else if(this.itemMode == "detail"){
					domClass.remove(this.domNode, "portletItemFlip");
					this.resizer && domClass.remove(this.resizer.domNode, "contentResizerFlip");
					
					this.animating = false;
					this.itemMode = "main";
					this.displayActions(this.actionDisplayedInMain);
				}
			}else{
				domStyle.set(this.mainContent.domNode, "display", "none");
				domStyle.set(this.detailContent.domNode, "display", "none");
				if(this.itemMode == "main"){
					var anim = flip["flip"]({ 
						node: this.domNode,
						dir: "right",
						depth: .5,
						duration: 1000
					});
					connect.connect(anim, "onEnd", this, function(){ 
						domStyle.set(this.mainContent.domNode, "display", "none");
						domStyle.set(this.detailContent.domNode, "display", "");
						
						this.itemMode = "detail";
						this.animating = false;
						this.displayActions(false);
					});
					anim.play();
				}else if(this.itemMode == "detail"){
					var anim = flip["flip"]({ 
						node: this.domNode,
						dir: "left",
						depth: .5,
						duration: 1000
					});
					connect.connect(anim, "onEnd", this, function(){ 
						domStyle.set(this.mainContent.domNode, "display", "");
						domStyle.set(this.detailContent.domNode, "display", "none");
						
						this.itemMode = "main";
						this.animating = false;
						this.displayActions(this.actionDisplayedInMain);
					});
					anim.play();
				}
			}
			e && event.stop(e);
		}
	});
	
});